@@ -81,7 +81,7 @@ typedef enum MuOfficeDocErrorType
 /**
  *Structure holding the detail of the layout of a bitmap. b5g6r5 is assumed.
  */
-typedef struct MuOfficeBitmap_s
+typedef struct
 {
 	void *memptr;
 	int   width;
@@ -95,7 +95,7 @@ typedef struct MuOfficeBitmap_s
  *    x           x coord of point
  *    y           y coord of point
  */
-typedef struct MuOfficePoint_s
+typedef struct
 {
 	float x;
 	float y;
@@ -109,7 +109,7 @@ typedef struct MuOfficePoint_s
  *    width       width of area
  *    height      height of area
  */
-typedef struct MuOfficeBox_s
+typedef struct
 {
 	float x;
 	float y;
@@ -123,7 +123,7 @@ typedef enum MuOfficePointType
 	MuOfficePointType_LineTo
 } MuOfficePointType;
 
-typedef struct MuOfficePathPoint
+typedef struct
 {
 	float x;
 	float y;
@@ -137,16 +137,16 @@ typedef struct MuOfficePathPoint
  *    origin            coordinates of the document origin within the bitmap
  *    renderArea        the part of the bitmap to which to render
  */
-typedef struct MuOfficeRenderArea_s
+typedef struct
 {
 	MuOfficePoint   origin;
 	MuOfficeBox     renderArea;
 } MuOfficeRenderArea;
 
-typedef struct MuOfficeLib_s MuOfficeLib;
-typedef struct MuOfficeDoc_s MuOfficeDoc;
-typedef struct MuOfficePage_s MuOfficePage;
-typedef struct MuOfficeRender_s MuOfficeRender;
+typedef struct MuOfficeLib MuOfficeLib;
+typedef struct MuOfficeDoc MuOfficeDoc;
+typedef struct MuOfficePage MuOfficePage;
+typedef struct MuOfficeRender MuOfficeRender;
 
 /**
  * Allocator function used by some functions to get blocks of memory.
