@@ -7,47 +7,54 @@
 #define PATH_MAX 2048
 #endif
 
-#include "mupdf/helpers/pkcs7-check.h"
 #include "mupdf/helpers/pkcs7-openssl.h"
 
 static pdf_widget *sig_widget;
-static char sig_designated_name[500];
-static enum pdf_signature_error sig_cert_error;
-static enum pdf_signature_error sig_digest_error;
-static int sig_subsequent_edits;
+static char *sig_designated_name = NULL;
+static pdf_signature_error sig_cert_error;
+static pdf_signature_error sig_digest_error;
+static int sig_valid_until;
 
 static char cert_filename[PATH_MAX];
 static struct input cert_password;
 
-static void do_sign(void)
+int do_sign(void)
 {
 	pdf_pkcs7_signer *signer = NULL;
+	int ok = 1;
 
 	fz_var(signer);
 
 	fz_try(ctx)
 	{
+		trace_action("widget.sign(new PDFPKCS7Signer(%q, %q));\n", cert_filename, cert_password.text);
 		signer = pkcs7_openssl_read_pfx(ctx, cert_filename, cert_password.text);
-		pdf_sign_signature(ctx, pdf, sig_widget, signer);
+		pdf_sign_signature(ctx, sig_widget, signer);
 		ui_show_warning_dialog("Signed document successfully.");
 	}
 	fz_always(ctx)
 	{
-		if (signer)
-			signer->drop(signer);
+		pdf_drop_signer(ctx, signer);
 	}
 	fz_catch(ctx)
+	{
 		ui_show_warning_dialog("%s", fz_caught_message(ctx));
+		ok = 0;
+	}
 
 	if (pdf_update_page(ctx, sig_widget->page))
+	{
+		trace_page_update();
 		render_page();
+	}
+	return ok;
 }
 
 static void do_clear_signature(void)
 {
 	fz_try(ctx)
 	{
-		pdf_clear_signature(ctx, pdf, sig_widget);
+		pdf_clear_signature(ctx, sig_widget);
 		ui_show_warning_dialog("Signature cleared successfully.");
 	}
 	fz_catch(ctx)
@@ -76,7 +83,7 @@ static void cert_password_dialog(void)
 			if (ui_button("Okay") || is == UI_INPUT_ACCEPT)
 			{
 				ui.dialog = NULL;
-				do_sign();
+				do_save_signed_pdf_file();
 			}
 		}
 		ui_panel_end();
@@ -91,7 +98,7 @@ static int cert_file_filter(const char *fn)
 
 static void cert_file_dialog(void)
 {
-	if (ui_open_file(cert_filename))
+	if (ui_open_file(cert_filename, "Select a certificate file to sign with:"))
 	{
 		if (cert_filename[0] != 0)
 		{
@@ -162,10 +169,14 @@ static void sig_verify_dialog(void)
 
 		if (sig_digest_error)
 			ui_label("Digest error: %s", pdf_signature_error_description(sig_digest_error));
-		else if (sig_subsequent_edits)
-			ui_label("The signature is valid but there have been edits since signing.");
+		else if (sig_valid_until == 0)
+			ui_label("The fields signed by this signature are unchanged.");
+		else if (sig_valid_until == 1)
+			ui_label("This signature was invalidated in the last update by the signed fields being changed.");
+		else if (sig_valid_until == 2)
+			ui_label("This signature was invalidated in the penultimate update by the signed fields being changed.");
 		else
-			ui_label("The document is unchanged since signing.");
+			ui_label("This signature was invalidated %d updates ago by the signed fields being changed.", sig_valid_until);
 
 		ui_layout(B, X, NW, 2, 2);
 		ui_panel_begin(0, ui.gridsize, 0, 0, 0);
@@ -193,10 +204,23 @@ static void show_sig_dialog(pdf_widget *widget)
 
 		if (pdf_signature_is_signed(ctx, pdf, widget->obj))
 		{
-			sig_cert_error = pdf_check_certificate(ctx, pdf, widget->obj);
-			sig_digest_error = pdf_check_digest(ctx, pdf, widget->obj);
-			sig_subsequent_edits = pdf_signature_incremental_change_since_signing(ctx, pdf, widget->obj);
-			pdf_signature_designated_name(ctx, pdf, widget->obj, sig_designated_name, sizeof(sig_designated_name));
+			pdf_pkcs7_verifier *verifier;
+			pdf_pkcs7_designated_name *dn;
+
+			sig_valid_until = pdf_validate_signature(ctx, widget);
+
+			verifier = pkcs7_openssl_new_verifier(ctx);
+
+			sig_cert_error = pdf_check_certificate(ctx, verifier, pdf, widget->obj);
+			sig_digest_error = pdf_check_digest(ctx, verifier, pdf, widget->obj);
+
+			dn = pdf_signature_get_signatory(ctx, verifier, pdf, widget->obj);
+			fz_free(ctx, sig_designated_name);
+			sig_designated_name = pdf_signature_format_designated_name(ctx, dn);
+			pdf_signature_drop_designated_name(ctx, dn);
+
+			pdf_drop_verifier(ctx, verifier);
+
 			ui.dialog = sig_verify_dialog;
 		}
 		else
@@ -234,9 +258,13 @@ static void tx_dialog(void)
 			ui_spacer();
 			if (ui_button("Okay") || is == UI_INPUT_ACCEPT)
 			{
+				trace_action("widget.setTextValue(%q);\n", tx_input.text);
 				pdf_set_text_field_value(ctx, tx_widget, tx_input.text);
 				if (pdf_update_page(ctx, tx_widget->page))
+				{
+					trace_page_update();
 					render_page();
+				}
 				ui.dialog = NULL;
 			}
 		}
@@ -258,15 +286,15 @@ static void ch_dialog(void)
 {
 	const char *label;
 	const char *value;
-	const char **options;
+	char **options;
 	int n, choice;
 	int label_h;
 
 	label = pdf_field_label(ctx, ch_widget->obj);
 	label_h = ui_break_lines((char*)label, NULL, 20, 394, NULL);
-	n = pdf_choice_widget_options(ctx, ch_widget->page->doc, ch_widget, 0, NULL);
-	options = fz_malloc_array(ctx, n, const char *);
-	pdf_choice_widget_options(ctx, ch_widget->page->doc, ch_widget, 0, options);
+	n = pdf_choice_widget_options(ctx, ch_widget, 0, NULL);
+	options = fz_malloc_array(ctx, n, char *);
+	pdf_choice_widget_options(ctx, ch_widget, 0, (const char **)options);
 	value = pdf_field_value(ctx, ch_widget->obj);
 
 	ui_dialog_begin(400, (ui.gridsize+4)*3 + ui.lineheight*(label_h-1));
@@ -274,9 +302,12 @@ static void ch_dialog(void)
 		ui_layout(T, X, NW, 2, 2);
 
 		ui_label("%s", label);
-		choice = ui_select("Widget/Ch", value, options, n);
+		choice = ui_select("Widget/Ch", value, (const char **)options, n);
 		if (choice >= 0)
+		{
+			trace_action("widget.setChoiceValue(%q);\n", options[choice]);
 			pdf_set_choice_field_value(ctx, ch_widget, options[choice]);
+		}
 
 		ui_layout(B, X, NW, 2, 2);
 		ui_panel_begin(0, ui.gridsize, 0, 0, 0);
@@ -288,7 +319,10 @@ static void ch_dialog(void)
 			if (ui_button("Okay"))
 			{
 				if (pdf_update_page(ctx, ch_widget->page))
+				{
+					trace_page_update();
 					render_page();
+				}
 				ui.dialog = NULL;
 			}
 		}
@@ -304,11 +338,12 @@ void do_widget_canvas(fz_irect canvas_area)
 	pdf_widget *widget;
 	fz_rect bounds;
 	fz_irect area;
+	int idx;
 
 	if (!pdf)
 		return;
 
-	for (widget = pdf_first_widget(ctx, page); widget; widget = pdf_next_widget(ctx, widget))
+	for (idx = 0, widget = pdf_first_widget(ctx, page); widget; ++idx, widget = pdf_next_widget(ctx, widget))
 	{
 		bounds = pdf_bound_widget(ctx, widget);
 		bounds = fz_transform_rect(bounds, view_page_ctm);
@@ -317,19 +352,28 @@ void do_widget_canvas(fz_irect canvas_area)
 		if (ui_mouse_inside(canvas_area) && ui_mouse_inside(area))
 		{
 			if (!widget->is_hot)
+			{
+				trace_action("page.getWidgets()[%d].eventEnter();\n", idx);
 				pdf_annot_event_enter(ctx, widget);
+			}
 			widget->is_hot = 1;
 
 			ui.hot = widget;
 			if (!ui.active && ui.down)
 			{
 				ui.active = widget;
+				trace_action("page.getWidgets()[%d].eventDown();\n", idx);
 				pdf_annot_event_down(ctx, widget);
 				if (selected_annot != widget)
 				{
 					if (selected_annot && pdf_annot_type(ctx, selected_annot) == PDF_ANNOT_WIDGET)
+					{
+						trace_action("widget.eventBlur();\n", idx);
 						pdf_annot_event_blur(ctx, selected_annot);
+					}
+					trace_action("widget = page.getWidgets()[%d];\n", idx);
 					selected_annot = widget;
+					trace_action("widget.eventFocus();\n");
 					pdf_annot_event_focus(ctx, widget);
 				}
 			}
@@ -337,7 +381,10 @@ void do_widget_canvas(fz_irect canvas_area)
 		else
 		{
 			if (widget->is_hot)
+			{
+				trace_action("page.getWidgets()[%d].eventExit();\n", idx);
 				pdf_annot_event_exit(ctx, widget);
+			}
 			widget->is_hot = 0;
 		}
 
@@ -372,6 +419,7 @@ void do_widget_canvas(fz_irect canvas_area)
 
 		if (ui.hot == widget && ui.active == widget && !ui.down)
 		{
+			trace_action("widget.eventUp();\n");
 			pdf_annot_event_up(ctx, widget);
 
 			if (pdf_widget_type(ctx, widget) == PDF_WIDGET_TYPE_SIGNATURE)
@@ -389,6 +437,7 @@ void do_widget_canvas(fz_irect canvas_area)
 					break;
 				case PDF_WIDGET_TYPE_CHECKBOX:
 				case PDF_WIDGET_TYPE_RADIOBUTTON:
+					trace_action("widget.toggle();\n");
 					pdf_toggle_widget(ctx, widget);
 					break;
 				case PDF_WIDGET_TYPE_TEXT:
@@ -406,5 +455,8 @@ void do_widget_canvas(fz_irect canvas_area)
 	}
 
 	if (pdf_update_page(ctx, page))
+	{
+		trace_page_update();
 		render_page();
+	}
 }
