@@ -4,10 +4,16 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <sys/stat.h>
+#ifdef _MSC_VER
+#define stat _stat
+#endif
 #ifndef _WIN32
 #include <signal.h>
 #endif
 
+#include "mupdf/helpers/pkcs7-openssl.h"
+
 #include "mujs.h"
 
 #ifndef PATH_MAX
@@ -17,7 +23,7 @@
 #ifndef _WIN32
 #include <unistd.h> /* for fork, exec, and getcwd */
 #else
-char *realpath(const char *path, char *resolved_path); /* in gl-file.c */
+#include <direct.h> /* for getcwd */
 #endif
 
 #ifdef __APPLE__
@@ -29,6 +35,17 @@ void glutLeaveMainLoop(void)
 }
 #endif
 
+time_t
+stat_mtime(const char *path)
+{
+	struct stat info;
+
+	if (stat(path, &info) < 0)
+		return 0;
+
+	return info.st_mtime;
+}
+
 fz_context *ctx = NULL;
 pdf_document *pdf = NULL;
 pdf_page *page = NULL;
@@ -92,7 +109,7 @@ static const int zoom_list[] = {
 static int zoom_in(int oldres)
 {
 	int i;
-	for (i = 0; i < nelem(zoom_list) - 1; ++i)
+	for (i = 0; i < (int)nelem(zoom_list) - 1; ++i)
 		if (zoom_list[i] <= oldres && zoom_list[i+1] > oldres)
 			return zoom_list[i+1];
 	return zoom_list[i];
@@ -101,7 +118,7 @@ static int zoom_in(int oldres)
 static int zoom_out(int oldres)
 {
 	int i;
-	for (i = 0; i < nelem(zoom_list) - 1; ++i)
+	for (i = 0; i < (int)nelem(zoom_list) - 1; ++i)
 		if (zoom_list[i] < oldres && zoom_list[i+1] >= oldres)
 			return zoom_list[i];
 	return zoom_list[0];
@@ -164,10 +181,11 @@ static int oldinvert = 0, currentinvert = 0;
 static int oldicc = 1, currenticc = 1;
 static int oldaa = 8, currentaa = 8;
 static int oldseparations = 0, currentseparations = 0;
-static int oldpage = 0, currentpage = 0;
+static fz_location oldpage = {0,0}, currentpage = {0,0};
 static float oldzoom = DEFRES, currentzoom = DEFRES;
 static float oldrotate = 0, currentrotate = 0;
 
+static fz_output *trace_file = NULL;
 static int isfullscreen = 0;
 static int showoutline = 0;
 static int showlinks = 0;
@@ -180,7 +198,7 @@ static const char *tooltip = NULL;
 
 struct mark
 {
-	int page;
+	fz_location loc;
 	fz_point scroll;
 };
 
@@ -244,13 +262,44 @@ static void read_history_file_as_json(js_State *J)
 	fz_drop_buffer(ctx, buf);
 }
 
+static fz_location try_location(js_State *J)
+{
+	fz_location loc;
+	if (js_isnumber(J, -1))
+		loc = fz_make_location(0, js_tryinteger(J, -1, 1) - 1);
+	else
+	{
+		js_getindex(J, -1, 0);
+		loc.chapter = js_tryinteger(J, -1, 1) - 1;
+		js_pop(J, 1);
+		js_getindex(J, -1, 1);
+		loc.page = js_tryinteger(J, -1, 1) - 1;
+		js_pop(J, 1);
+	}
+	return loc;
+}
+
+static void push_location(js_State *J, fz_location loc)
+{
+	if (loc.chapter == 0)
+		js_pushnumber(J, (double)loc.page+1);
+	else
+	{
+		js_newarray(J);
+		js_pushnumber(J, (double)loc.chapter+1);
+		js_setindex(J, -2, 0);
+		js_pushnumber(J, (double)loc.page+1);
+		js_setindex(J, -2, 1);
+	}
+}
+
 static void load_history(void)
 {
 	js_State *J;
 	char absname[PATH_MAX];
 	int i, n;
 
-	if (!realpath(filename, absname))
+	if (!fz_realpath(filename, absname))
 		return;
 
 	J = js_newstate(NULL, NULL, 0);
@@ -261,7 +310,7 @@ static void load_history(void)
 	{
 		if (js_hasproperty(J, -1, "current"))
 		{
-			currentpage = js_tryinteger(J, -1, 1) - 1;
+			currentpage = try_location(J);
 			js_pop(J, 1);
 		}
 
@@ -273,7 +322,7 @@ static void load_history(void)
 				for (i = 0; i < history_count; ++i)
 				{
 					js_getindex(J, -1, i);
-					history[i].page = js_tryinteger(J, -1, 1) - 1;
+					history[i].loc = try_location(J);
 					js_pop(J, 1);
 				}
 			}
@@ -288,7 +337,7 @@ static void load_history(void)
 				for (i = 0; i < future_count; ++i)
 				{
 					js_getindex(J, -1, i);
-					future[i].page = js_tryinteger(J, -1, 1) - 1;
+					future[i].loc = try_location(J);
 					js_pop(J, 1);
 				}
 			}
@@ -303,7 +352,7 @@ static void load_history(void)
 				for (i = 0; i < n; ++i)
 				{
 					js_getindex(J, -1, i);
-					marks[i].page = js_tryinteger(J, -1, 1) - 1;
+					marks[i].loc = try_location(J);
 					js_pop(J, 1);
 				}
 			}
@@ -327,7 +376,7 @@ static void save_history(void)
 	if (!doc)
 		return;
 
-	if (!realpath(filename, absname))
+	if (!fz_realpath(filename, absname))
 		return;
 
 	J = js_newstate(NULL, NULL, 0);
@@ -336,13 +385,13 @@ static void save_history(void)
 
 	js_newobject(J);
 	{
-		js_pushnumber(J, currentpage+1);
+		push_location(J, currentpage);
 		js_setproperty(J, -2, "current");
 
 		js_newarray(J);
 		for (i = 0; i < history_count; ++i)
 		{
-			js_pushnumber(J, history[i].page+1);
+			push_location(J, history[i].loc);
 			js_setindex(J, -2, i);
 		}
 		js_setproperty(J, -2, "history");
@@ -350,15 +399,15 @@ static void save_history(void)
 		js_newarray(J);
 		for (i = 0; i < future_count; ++i)
 		{
-			js_pushnumber(J, future[i].page+1);
+			push_location(J, future[i].loc);
 			js_setindex(J, -2, i);
 		}
 		js_setproperty(J, -2, "future");
 
 		js_newarray(J);
-		for (i = 0; i < nelem(marks); ++i)
+		for (i = 0; i < (int)nelem(marks); ++i)
 		{
-			js_pushnumber(J, marks[i].page+1);
+			push_location(J, marks[i].loc);
 			js_setindex(J, -2, i);
 		}
 		js_setproperty(J, -2, "marks");
@@ -390,12 +439,64 @@ static void save_history(void)
 	js_freestate(J);
 }
 
+static int convert_to_accel_path(char outname[], char *absname, size_t len)
+{
+	char *tmpdir;
+	char *s;
+
+	tmpdir = getenv("TEMP");
+	if (!tmpdir)
+		tmpdir = getenv("TMP");
+	if (!tmpdir)
+		tmpdir = "/var/tmp";
+	if (!fz_is_directory(ctx, tmpdir))
+		tmpdir = "/tmp";
+
+	if (absname[0] == '/' || absname[0] == '\\')
+		++absname;
+
+	s = absname;
+	while (*s) {
+		if (*s == '/' || *s == '\\' || *s == ':')
+			*s = '%';
+		++s;
+	}
+
+	if (fz_snprintf(outname, len, "%s/%s.accel", tmpdir, absname) >= len)
+		return 0;
+	return 1;
+}
+
+static int get_accelerator_filename(char outname[], size_t len)
+{
+	char absname[PATH_MAX];
+	if (!fz_realpath(filename, absname))
+		return 0;
+	if (!convert_to_accel_path(outname, absname, len))
+		return 0;
+	return 1;
+}
+
+static void save_accelerator(void)
+{
+	char absname[PATH_MAX];
+
+	if (!doc)
+		return;
+	if (!fz_document_supports_accelerator(ctx, doc))
+		return;
+	if (!get_accelerator_filename(absname, sizeof(absname)))
+		return;
+
+	fz_save_accelerator(ctx, doc, absname);
+}
+
 static int search_active = 0;
 static struct input search_input = { { 0 }, 0 };
 static char *search_needle = 0;
 static int search_dir = 1;
-static int search_page = -1;
-static int search_hit_page = -1;
+static fz_location search_page = {-1, -1};
+static fz_location search_hit_page = {-1, -1};
 static int search_hit_count = 0;
 static fz_quad search_hit_quads[5000];
 
@@ -512,6 +613,55 @@ void ui_show_warning_dialog(const char *fmt, ...)
 	ui.dialog = warning_dialog;
 }
 
+static void quit_dialog(void)
+{
+	ui_dialog_begin(500, (ui.gridsize+4)*3);
+	ui_layout(T, NONE, NW, 2, 2);
+	ui_label("%C The document has unsaved changes. Are you sure you want to quit?", 0x26a0); /* WARNING SIGN */
+	ui_layout(B, X, S, 2, 2);
+	ui_panel_begin(0, ui.gridsize, 0, 0, 0);
+	{
+		ui_layout(R, NONE, S, 0, 0);
+		if (ui_button("Save"))
+			do_save_pdf_file();
+		ui_spacer();
+		if (ui_button("Discard") || ui.key == 'q')
+			glutLeaveMainLoop();
+		ui_layout(L, NONE, S, 0, 0);
+		if (ui_button("Cancel") || ui.key == KEY_ESCAPE)
+			ui.dialog = NULL;
+	}
+	ui_panel_end();
+	ui_dialog_end();
+}
+
+void quit(void)
+{
+	if (pdf && pdf_has_unsaved_changes(ctx, pdf))
+		ui.dialog = quit_dialog;
+	else
+		glutLeaveMainLoop();
+}
+
+void trace_action(const char *fmt, ...)
+{
+	va_list args;
+	if (trace_file)
+	{
+		va_start(args, fmt);
+		fz_write_vprintf(ctx, trace_file, fmt, args);
+		fz_flush_output(ctx, trace_file);
+		va_end(args);
+	}
+}
+
+void trace_page_update(void)
+{
+	static int trace_idx = 1;
+	trace_action("page.update();\n");
+	trace_action("page.toPixmap(Identity, DeviceRGB).saveAsPNG(\"trace-%03d.png\");\n", trace_idx++);
+}
+
 void update_title(void)
 {
 	char buf[256];
@@ -519,6 +669,8 @@ void update_title(void)
 	char *extra = "";
 	size_t n;
 
+	int nc = fz_count_chapters(ctx, doc);
+
 	title = strrchr(filename, '/');
 	if (!title)
 		title = strrchr(filename, '\\');
@@ -532,9 +684,24 @@ void update_title(void)
 
 	n = strlen(title);
 	if (n > 50)
-		sprintf(buf, "...%s%s - %d / %d", title + n - 50, extra, currentpage + 1, fz_count_pages(ctx, doc));
+	{
+		if (nc == 1)
+			sprintf(buf, "...%s%s - %d/%d", title + n - 50, extra, currentpage.page + 1, fz_count_pages(ctx, doc));
+		else
+			sprintf(buf, "...%s%s - %d/%d - %d/%d", title + n - 50, extra,
+				currentpage.chapter + 1, nc,
+				currentpage.page + 1, fz_count_chapter_pages(ctx, doc, currentpage.chapter));
+	}
 	else
-		sprintf(buf, "%s%s - %d / %d", title, extra, currentpage + 1, fz_count_pages(ctx, doc));
+	{
+		if (nc == 1)
+			sprintf(buf, "%s%s - %d/%d", title, extra, currentpage.page + 1, fz_count_pages(ctx, doc));
+		else
+
+			sprintf(buf, "%s%s - %d/%d - %d/%d", title, extra,
+				currentpage.chapter + 1, nc,
+				currentpage.page + 1, fz_count_chapter_pages(ctx, doc, currentpage.chapter));
+	}
 	glutSetWindowTitle(buf);
 	glutSetIconTitle(buf);
 }
@@ -549,6 +716,9 @@ void load_page(void)
 {
 	fz_irect area;
 
+	if (trace_file)
+		trace_action("page = doc.loadPage(%d);\n", fz_page_number_from_location(ctx, doc, currentpage));
+
 	/* clear all editor selections */
 	if (selected_annot && pdf_annot_type(ctx, selected_annot) == PDF_ANNOT_WIDGET)
 		pdf_annot_event_blur(ctx, selected_annot);
@@ -563,10 +733,39 @@ void load_page(void)
 	fz_drop_page(ctx, fzpage);
 	fzpage = NULL;
 
-	fzpage = fz_load_page(ctx, doc, currentpage);
+	fzpage = fz_load_chapter_page(ctx, doc, currentpage.chapter, currentpage.page);
 	if (pdf)
 		page = (pdf_page*)fzpage;
 
+	if (trace_file)
+	{
+		pdf_widget *w;
+		int i, s;
+
+		for (i = 0, s = 0, w = pdf_first_widget(ctx, page); w != NULL; i++, w = pdf_next_widget(ctx, w))
+			if (pdf_widget_type(ctx, w) == PDF_WIDGET_TYPE_SIGNATURE)
+			{
+				s++;
+				int signd = pdf_widget_is_signed(ctx, w);
+				trace_action("widget = page.getWidgets()[%d];\n", i);
+				if (signd)
+				{
+					int valid = pdf_validate_signature(ctx, w);
+					trace_action("if (!widget.isSigned()) { print(\"Expected signature %d (chapter %d, page %d) to be signed!\\n\"); }\n",
+						s, currentpage.chapter, currentpage.page);
+					trace_action(
+						"tmp = page.getWidgets()[%d].validateSignature();\n"
+						"if (tmp != %d) { print(\"Signature %d (chapter %d, page %d) was invalidated \" + tmp + \" updates ago - expected %d\\n\"); }\n",
+						i, valid, s, currentpage.chapter, currentpage.page, valid);
+				}
+				else
+				{
+					trace_action("if (widget.isSigned()) { print(\"Expected signature %d (chapter %d, page %d) to be unsigned!\\n\"); }\n",
+						s, currentpage.chapter, currentpage.page);
+				}
+			}
+	}
+
 	links = fz_load_links(ctx, fzpage);
 	page_text = fz_new_stext_page_from_page(ctx, fzpage, NULL);
 
@@ -620,11 +819,14 @@ void render_page(void)
 
 	ui_texture_from_pixmap(&page_tex, pix);
 	fz_drop_pixmap(ctx, pix);
+
+	FZ_LOG_DUMP_STORE(ctx, "Store state after page render:\n");
 }
 
 void render_page_if_changed(void)
 {
-	if (oldpage != currentpage ||
+	if (oldpage.chapter != currentpage.chapter ||
+		oldpage.page != currentpage.page ||
 		oldzoom != currentzoom ||
 		oldrotate != currentrotate ||
 		oldinvert != currentinvert ||
@@ -648,24 +850,40 @@ void render_page_if_changed(void)
 static struct mark save_mark()
 {
 	struct mark mark;
-	mark.page = currentpage;
+	mark.loc = currentpage;
 	mark.scroll = fz_transform_point_xy(scroll_x, scroll_y, view_page_inv_ctm);
 	return mark;
 }
 
 static void restore_mark(struct mark mark)
 {
-	currentpage = mark.page;
+	currentpage = mark.loc;
 	mark.scroll = fz_transform_point(mark.scroll, draw_page_ctm);
 	scroll_x = mark.scroll.x;
 	scroll_y = mark.scroll.y;
 }
 
+static int eqloc(fz_location a, fz_location b)
+{
+	return a.chapter == b.chapter && a.page == b.page;
+}
+
+static int is_first_page(fz_location loc)
+{
+	return (loc.chapter == 0 && loc.page == 0);
+}
+
+static int is_last_page(fz_location loc)
+{
+	fz_location last = fz_last_page(ctx, doc);
+	return (loc.chapter == last.chapter && loc.page == last.page);
+}
+
 static void push_history(void)
 {
-	if (history_count > 0 && history[history_count-1].page == currentpage)
+	if (history_count > 0 && eqloc(history[history_count-1].loc, currentpage))
 		return;
-	if (history_count + 1 >= nelem(history))
+	if (history_count + 1 >= (int)nelem(history))
 	{
 		memmove(history, history + 1, sizeof *history * (nelem(history) - 1));
 		history[history_count] = save_mark();
@@ -678,7 +896,7 @@ static void push_history(void)
 
 static void push_future(void)
 {
-	if (future_count + 1 >= nelem(future))
+	if (future_count + 1 >= (int)nelem(future))
 	{
 		memmove(future, future + 1, sizeof *future * (nelem(future) - 1));
 		future[future_count] = save_mark();
@@ -694,22 +912,41 @@ static void clear_future(void)
 	future_count = 0;
 }
 
-static void jump_to_page(int newpage)
+static void jump_to_location(fz_location loc)
 {
-	newpage = fz_clampi(newpage, 0, fz_count_pages(ctx, doc) - 1);
 	clear_future();
 	push_history();
-	currentpage = newpage;
+	currentpage = fz_clamp_location(ctx, doc, loc);
+	push_history();
+}
+
+static void jump_to_location_xy(fz_location loc, float x, float y)
+{
+	fz_point p = fz_transform_point_xy(x, y, draw_page_ctm);
+	clear_future();
+	push_history();
+	currentpage = fz_clamp_location(ctx, doc, loc);
+	scroll_x = p.x;
+	scroll_y = p.y;
+	push_history();
+}
+
+static void jump_to_page(int newpage)
+{
+	clear_future();
+	push_history();
+	currentpage = fz_location_from_page_number(ctx, doc, newpage);
+	currentpage = fz_clamp_location(ctx, doc, currentpage);
 	push_history();
 }
 
 static void jump_to_page_xy(int newpage, float x, float y)
 {
 	fz_point p = fz_transform_point_xy(x, y, draw_page_ctm);
-	newpage = fz_clampi(newpage, 0, fz_count_pages(ctx, doc) - 1);
 	clear_future();
 	push_history();
-	currentpage = newpage;
+	currentpage = fz_location_from_page_number(ctx, doc, newpage);
+	currentpage = fz_clamp_location(ctx, doc, currentpage);
 	scroll_x = p.x;
 	scroll_y = p.y;
 	push_history();
@@ -717,17 +954,17 @@ static void jump_to_page_xy(int newpage, float x, float y)
 
 static void pop_history(void)
 {
-	int here = currentpage;
+	fz_location here = currentpage;
 	push_future();
-	while (history_count > 0 && currentpage == here)
+	while (history_count > 0 && eqloc(currentpage, here))
 		restore_mark(history[--history_count]);
 }
 
 static void pop_future(void)
 {
-	int here = currentpage;
+	fz_location here = currentpage;
 	push_history();
-	while (future_count > 0 && currentpage == here)
+	while (future_count > 0 && eqloc(currentpage, here))
 		restore_mark(future[--future_count]);
 	push_history();
 }
@@ -757,16 +994,15 @@ static int count_outline(fz_outline *node, int end)
 	while (node)
 	{
 		p = node->page;
-		if (p >= 0)
-		{
-			count += 1;
-			n = end;
-			if (node->next && node->next->page >= 0)
-				n = node->next->page;
-			is_selected = (currentpage == p || (currentpage > p && currentpage < n));
-			if (node->down && (node->is_open || is_selected))
-				count += count_outline(node->down, end);
-		}
+		count += 1;
+		n = end;
+		if (node->next && node->next->page >= 0)
+			n = node->next->page;
+		is_selected = 0;
+		if (fz_count_chapters(ctx, doc) == 1)
+			is_selected = (p>=0) && (currentpage.page == p || (currentpage.page > p && currentpage.page < n));
+		if (node->down && (node->is_open || is_selected))
+			count += count_outline(node->down, end);
 		node = node->next;
 	}
 	return count;
@@ -774,25 +1010,34 @@ static int count_outline(fz_outline *node, int end)
 
 static void do_outline_imp(struct list *list, int end, fz_outline *node, int depth)
 {
-	int selected, was_open, n;
+	int is_selected, was_open, n;
 
 	while (node)
 	{
 		int p = node->page;
-		if (p >= 0)
+		n = end;
+		if (node->next && node->next->page >= 0)
+			n = node->next->page;
+
+		was_open = node->is_open;
+		is_selected = 0;
+		if (fz_count_chapters(ctx, doc) == 1)
+			is_selected = (p>=0) && (currentpage.page == p || (currentpage.page > p && currentpage.page < n));
+		if (ui_tree_item(list, node, node->title, is_selected, depth, !!node->down, &node->is_open))
 		{
-			n = end;
-			if (node->next && node->next->page >= 0)
-				n = node->next->page;
-
-			was_open = node->is_open;
-			selected = (currentpage == p || (currentpage > p && currentpage < n));
-			if (ui_tree_item(list, node, node->title, selected, depth, !!node->down, &node->is_open))
+			if (p < 0)
+			{
+				currentpage = fz_resolve_link(ctx, doc, node->uri, &node->x, &node->y);
+				jump_to_location_xy(currentpage, node->x, node->y);
+			}
+			else
+			{
 				jump_to_page_xy(p, node->x, node->y);
-
-			if (node->down && (was_open || selected))
-				do_outline_imp(list, n, node->down, depth + 1);
+			}
 		}
+
+		if (node->down && (was_open || is_selected))
+			do_outline_imp(list, n, node->down, depth + 1);
 		node = node->next;
 	}
 }
@@ -801,8 +1046,8 @@ static void do_outline(fz_outline *node)
 {
 	static struct list list;
 	ui_layout(L, BOTH, NW, 0, 0);
-	ui_tree_begin(&list, count_outline(node, fz_count_pages(ctx, doc)), outline_w, 0, 1);
-	do_outline_imp(&list, fz_count_pages(ctx, doc), node, 0);
+	ui_tree_begin(&list, count_outline(node, 65535), outline_w, 0, 1);
+	do_outline_imp(&list, 65535, node, 0);
 	ui_tree_end(&list);
 	ui_splitter(&outline_w, 150, 500, R);
 }
@@ -849,11 +1094,8 @@ static void do_links(fz_link *link)
 					open_browser(link->uri);
 				else
 				{
-					int p = fz_resolve_link(ctx, doc, link->uri, &link_x, &link_y);
-					if (p >= 0)
-						jump_to_page_xy(p, link_x, link_y);
-					else
-						fz_warn(ctx, "cannot find link destination '%s'", link->uri);
+					fz_location loc = fz_resolve_link(ctx, doc, link->uri, &link_x, &link_y);
+					jump_to_location_xy(loc, link_x, link_y);
 				}
 			}
 		}
@@ -867,7 +1109,8 @@ static void do_links(fz_link *link)
 static void do_page_selection(void)
 {
 	static fz_point pt = { 0, 0 };
-	fz_quad hits[1000];
+	static fz_quad hits[1000];
+	fz_rect rect;
 	int i, n;
 
 	if (ui_mouse_inside(view_page_area))
@@ -894,7 +1137,20 @@ static void do_page_selection(void)
 		else if (ui.mod == GLUT_ACTIVE_CTRL + GLUT_ACTIVE_SHIFT)
 			fz_snap_selection(ctx, page_text, &page_a, &page_b, FZ_SELECT_LINES);
 
-		n = fz_highlight_selection(ctx, page_text, page_a, page_b, hits, nelem(hits));
+		if (ui.mod == GLUT_ACTIVE_SHIFT)
+		{
+			rect = fz_make_rect(
+					fz_min(page_a.x, page_b.x),
+					fz_min(page_a.y, page_b.y),
+					fz_max(page_a.x, page_b.x),
+					fz_max(page_a.y, page_b.y));
+			n = 1;
+			hits[0] = fz_quad_from_rect(rect);
+		}
+		else
+		{
+			n = fz_highlight_selection(ctx, page_text, page_a, page_b, hits, nelem(hits));
+		}
 
 		glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO); /* invert destination color */
 		glEnable(GL_BLEND);
@@ -917,9 +1173,15 @@ static void do_page_selection(void)
 		{
 			char *s;
 #ifdef _WIN32
-			s = fz_copy_selection(ctx, page_text, page_a, page_b, 1);
+			if (ui.mod == GLUT_ACTIVE_SHIFT)
+				s = fz_copy_rectangle(ctx, page_text, rect, 1);
+			else
+				s = fz_copy_selection(ctx, page_text, page_a, page_b, 1);
 #else
-			s = fz_copy_selection(ctx, page_text, page_a, page_b, 0);
+			if (ui.mod == GLUT_ACTIVE_SHIFT)
+				s = fz_copy_rectangle(ctx, page_text, rect, 0);
+			else
+				s = fz_copy_selection(ctx, page_text, page_a, page_b, 0);
 #endif
 			ui_set_clipboard(s);
 			fz_free(ctx, s);
@@ -1015,10 +1277,38 @@ static void password_dialog(void)
 
 static void load_document(void)
 {
+	char accelpath[PATH_MAX];
+	char *accel = NULL;
+	time_t atime;
+	time_t dtime;
+
 	fz_drop_outline(ctx, outline);
 	fz_drop_document(ctx, doc);
 
-	doc = fz_open_document(ctx, filename);
+	/* If there was an accelerator to load, what would it be called? */
+	if (get_accelerator_filename(accelpath, sizeof(accelpath)))
+	{
+		/* Check whether that file exists, and isn't older than
+		 * the document. */
+		atime = stat_mtime(accelpath);
+		dtime = stat_mtime(filename);
+		if (atime == 0)
+		{
+			/* No accelerator */
+		}
+		else if (atime > dtime)
+			accel = accelpath;
+		else
+		{
+			/* Accelerator data is out of date */
+			unlink(accelpath);
+			accel = NULL; /* In case we have jumped up from below */
+		}
+	}
+
+	trace_action("doc = new Document(%q);\n", filename);
+
+	doc = fz_open_accelerated_document(ctx, filename, accel);
 	if (fz_needs_password(ctx, doc))
 	{
 		if (!fz_authenticate_password(ctx, doc, password))
@@ -1045,7 +1335,28 @@ static void load_document(void)
 	if (pdf)
 	{
 		if (enable_js)
+		{
+			trace_action("doc.enableJS();\n");
 			pdf_enable_js(ctx, pdf);
+		}
+		if (trace_file)
+		{
+			int vsns = pdf_count_versions(ctx, pdf);
+			trace_action(
+				"tmp = doc.countVersions();\n"
+				"if (%d != tmp) {\n"
+				"  print(\"Mismatch in number of versions of document. I expected %d and got \" + tmp + \"\\n\");\n"
+				"}\n", vsns, vsns);
+			if (vsns > 1)
+			{
+				int valid = pdf_validate_change_history(ctx, pdf);
+				trace_action(
+					"tmp = doc.validateChangeHistory();\n"
+					"if (tmp != %d) {\n"
+					"  print(\"Mismatch in change history validation. I expected %d and got \" + tmp + \"\\n\");\n"
+					"}\n", valid, valid);
+			}
+		}
 		if (anchor)
 			jump_to_page(pdf_lookup_anchor(ctx, pdf, anchor, NULL, NULL));
 	}
@@ -1056,12 +1367,13 @@ static void load_document(void)
 	}
 	anchor = NULL;
 
-	currentpage = fz_clampi(currentpage, 0, fz_count_pages(ctx, doc) - 1);
+	oldpage = currentpage = fz_clamp_location(ctx, doc, currentpage);
 }
 
 void reload(void)
 {
 	save_history();
+	save_accelerator();
 	load_document();
 	if (doc)
 	{
@@ -1121,15 +1433,18 @@ static void auto_zoom(void)
 
 static void smart_move_backward(void)
 {
-	if (scroll_y <= 0)
+	int slop_x = page_tex.w / 20;
+	int slop_y = page_tex.h / 20;
+	if (scroll_y <= slop_y)
 	{
-		if (scroll_x <= 0)
+		if (scroll_x <= slop_x)
 		{
-			if (currentpage - 1 >= 0)
+			fz_location prev = fz_previous_page(ctx, doc, currentpage);
+			if (!eqloc(currentpage, prev))
 			{
-				scroll_x = page_tex.w;
-				scroll_y = page_tex.h;
-				currentpage -= 1;
+				scroll_x = (page_tex.w <= canvas_w) ? 0 : page_tex.w - canvas_w;
+				scroll_y = (page_tex.h <= canvas_h) ? 0 : page_tex.h - canvas_h;
+				currentpage = prev;
 			}
 		}
 		else
@@ -1146,15 +1461,18 @@ static void smart_move_backward(void)
 
 static void smart_move_forward(void)
 {
-	if (scroll_y + canvas_h >= page_tex.h)
+	int slop_x = page_tex.w / 20;
+	int slop_y = page_tex.h / 20;
+	if (scroll_y + canvas_h >= page_tex.h - slop_y)
 	{
-		if (scroll_x + canvas_w >= page_tex.w)
+		if (scroll_x + canvas_w >= page_tex.w - slop_x)
 		{
-			if (currentpage + 1 < fz_count_pages(ctx, doc))
+			fz_location next = fz_next_page(ctx, doc, currentpage);
+			if (!eqloc(currentpage, next))
 			{
 				scroll_x = 0;
 				scroll_y = 0;
-				currentpage += 1;
+				currentpage = next;
 			}
 		}
 		else
@@ -1172,14 +1490,15 @@ static void smart_move_forward(void)
 static void clear_search(void)
 {
 	showsearch = 0;
-	search_hit_page = -1;
+	search_page = currentpage;
+	search_hit_page = fz_make_location(-1, -1);
 	search_hit_count = 0;
 }
 
 static void do_app(void)
 {
 	if (ui.key == KEY_F4 && ui.mod == GLUT_ACTIVE_ALT)
-		glutLeaveMainLoop();
+		quit();
 
 	if (ui.down || ui.middle || ui.right || ui.key)
 		showinfo = 0;
@@ -1196,7 +1515,7 @@ static void do_app(void)
 		case 'F': showform = !showform; break;
 		case 'i': showinfo = !showinfo; break;
 		case 'r': reload(); break;
-		case 'q': glutLeaveMainLoop(); break;
+		case 'q': quit(); break;
 		case 'S': do_save_pdf_file(); break;
 
 		case '>': layout_em = number > 0 ? number : layout_em + 1; relayout(); break;
@@ -1216,17 +1535,26 @@ static void do_app(void)
 		case '-': set_zoom(zoom_out(currentzoom), ui.x, ui.y); break;
 		case '[': currentrotate -= 90; break;
 		case ']': currentrotate += 90; break;
-		case 'k': case KEY_UP: scroll_y -= 10; break;
-		case 'j': case KEY_DOWN: scroll_y += 10; break;
-		case 'h': case KEY_LEFT: scroll_x -= 10; break;
-		case 'l': case KEY_RIGHT: scroll_x += 10; break;
+		case 'k': case KEY_UP: scroll_y -= canvas_h/10; break;
+		case 'j': case KEY_DOWN: scroll_y += canvas_h/10; break;
+		case 'h': case KEY_LEFT: scroll_x -= canvas_w/10; break;
+		case 'l': case KEY_RIGHT: scroll_x += canvas_w/10; break;
 
 		case 'b': number = fz_maxi(number, 1); while (number--) smart_move_backward(); break;
 		case ' ': number = fz_maxi(number, 1); while (number--) smart_move_forward(); break;
-		case ',': case KEY_PAGE_UP: currentpage -= fz_maxi(number, 1); break;
-		case '.': case KEY_PAGE_DOWN: currentpage += fz_maxi(number, 1); break;
 		case 'g': jump_to_page(number - 1); break;
-		case 'G': jump_to_page(fz_count_pages(ctx, doc) - 1); break;
+		case 'G': jump_to_location(fz_last_page(ctx, doc)); break;
+
+		case ',': case KEY_PAGE_UP:
+			number = fz_maxi(number, 1);
+			while (number--)
+				currentpage = fz_previous_page(ctx, doc, currentpage);
+			break;
+		case '.': case KEY_PAGE_DOWN:
+			number = fz_maxi(number, 1);
+			while (number--)
+				currentpage = fz_next_page(ctx, doc, currentpage);
+			break;
 
 		case 'A':
 			if (number == 0)
@@ -1238,7 +1566,7 @@ static void do_app(void)
 		case 'm':
 			if (number == 0)
 				push_history();
-			else if (number > 0 && number < nelem(marks))
+			else if (number > 0 && number < (int)nelem(marks))
 				marks[number] = save_mark();
 			break;
 		case 't':
@@ -1247,11 +1575,11 @@ static void do_app(void)
 				if (history_count > 0)
 					pop_history();
 			}
-			else if (number > 0 && number < nelem(marks))
+			else if (number > 0 && number < (int)nelem(marks))
 			{
 				struct mark mark = marks[number];
 				restore_mark(mark);
-				jump_to_page(mark.page);
+				jump_to_location(mark.loc);
 			}
 			break;
 		case 'T':
@@ -1280,29 +1608,33 @@ static void do_app(void)
 			break;
 		case 'N':
 			search_dir = -1;
-			if (search_hit_page == currentpage)
-				search_page = currentpage + search_dir;
-			else
-				search_page = currentpage;
-			if (search_page >= 0 && search_page < fz_count_pages(ctx, doc))
+			search_active = !!search_needle;
+			if (eqloc(search_hit_page, currentpage))
 			{
-				search_hit_page = -1;
-				if (search_needle)
-					search_active = 1;
+				search_page = fz_previous_page(ctx, doc, currentpage);
+				if (is_first_page(search_page))
+					search_active = 0;
 			}
+			else
+			{
+				search_page = currentpage;
+			}
+			search_hit_page = fz_make_location(-1, -1);
 			break;
 		case 'n':
 			search_dir = 1;
-			if (search_hit_page == currentpage)
-				search_page = currentpage + search_dir;
-			else
-				search_page = currentpage;
-			if (search_page >= 0 && search_page < fz_count_pages(ctx, doc))
+			search_active = !!search_needle;
+			if (eqloc(search_hit_page, currentpage))
 			{
-				search_hit_page = -1;
-				if (search_needle)
-					search_active = 1;
+				search_page = fz_next_page(ctx, doc, currentpage);
+				if (is_last_page(search_page))
+					search_active = 0;
 			}
+			else
+			{
+				search_page = currentpage;
+			}
+			search_hit_page = fz_make_location(-1, -1);
 			break;
 		}
 
@@ -1311,22 +1643,85 @@ static void do_app(void)
 		else
 			number = 0;
 
-		currentpage = fz_clampi(currentpage, 0, fz_count_pages(ctx, doc) - 1);
+		currentpage = fz_clamp_location(ctx, doc, currentpage);
 		while (currentrotate < 0) currentrotate += 360;
 		while (currentrotate >= 360) currentrotate -= 360;
 
-		if (search_hit_page != currentpage)
-			search_hit_page = -1; /* clear highlights when navigating */
+		if (!eqloc(search_hit_page, currentpage))
+			search_hit_page = fz_make_location(-1, -1); /* clear highlights when navigating */
 
 		ui.key = 0; /* we ate the key event, so zap it */
 	}
 }
 
+typedef struct
+{
+	int max;
+	int len;
+	pdf_obj **sig;
+} sigs_list;
+
+static void
+process_sigs(fz_context *ctx, pdf_obj *field, void *arg, pdf_obj **ft)
+{
+	sigs_list *sigs = (sigs_list *)arg;
+
+	if (!pdf_name_eq(ctx, pdf_dict_get(ctx, field, PDF_NAME(Type)), PDF_NAME(Annot)) ||
+		!pdf_name_eq(ctx, pdf_dict_get(ctx, field, PDF_NAME(Subtype)), PDF_NAME(Widget)) ||
+		!pdf_name_eq(ctx, pdf_dict_get(ctx, field, ft[0]), PDF_NAME(Sig)))
+		return;
+
+	if (sigs->len == sigs->max)
+	{
+		int newsize = sigs->max * 2;
+		if (newsize == 0)
+			newsize = 4;
+		sigs->sig = fz_realloc_array(ctx, sigs->sig, newsize, pdf_obj *);
+		sigs->max = newsize;
+	}
+
+	sigs->sig[sigs->len++] = field;
+}
+
+static char *short_signature_error_desc(pdf_signature_error err)
+{
+	switch (err)
+	{
+	case PDF_SIGNATURE_ERROR_OKAY:
+		return "OK";
+	case PDF_SIGNATURE_ERROR_NO_SIGNATURES:
+		return "No signatures";
+	case PDF_SIGNATURE_ERROR_NO_CERTIFICATE:
+		return "No certificate";
+	case PDF_SIGNATURE_ERROR_DIGEST_FAILURE:
+		return "Invalid";
+	case PDF_SIGNATURE_ERROR_SELF_SIGNED:
+		return "Self-signed";
+	case PDF_SIGNATURE_ERROR_SELF_SIGNED_IN_CHAIN:
+		return "Self-signed in chain";
+	case PDF_SIGNATURE_ERROR_NOT_TRUSTED:
+		return "Untrusted";
+	default:
+	case PDF_SIGNATURE_ERROR_UNKNOWN:
+		return "Unknown error";
+	}
+}
+
 static void do_info(void)
 {
 	char buf[100];
+	pdf_document *pdoc = pdf_specifics(ctx, doc);
+	sigs_list list = { 0, 0, NULL };
 
-	ui_dialog_begin(500, 14 * ui.lineheight);
+	if (pdoc)
+	{
+		static pdf_obj *ft_list[2] = { PDF_NAME(FT), NULL };
+		pdf_obj *ft;
+		pdf_obj *form_fields = pdf_dict_getp(ctx, pdf_trailer(ctx, pdoc), "Root/AcroForm/Fields");
+		pdf_walk_tree(ctx, form_fields, PDF_NAME(Kids), process_sigs, NULL, &list, &ft_list[0], &ft);
+	}
+
+	ui_dialog_begin(500, (14+list.len) * ui.lineheight);
 	ui_layout(T, X, W, 0, 0);
 
 	if (fz_lookup_metadata(ctx, doc, FZ_META_INFO_TITLE, buf, sizeof buf) > 0)
@@ -1337,8 +1732,10 @@ static void do_info(void)
 		ui_label("Format: %s", buf);
 	if (fz_lookup_metadata(ctx, doc, FZ_META_ENCRYPTION, buf, sizeof buf) > 0)
 		ui_label("Encryption: %s", buf);
-	if (pdf_specifics(ctx, doc))
+	if (pdoc)
 	{
+		int updates = pdf_count_versions(ctx, pdoc);
+
 		if (fz_lookup_metadata(ctx, doc, "info:Creator", buf, sizeof buf) > 0)
 			ui_label("PDF Creator: %s", buf);
 		if (fz_lookup_metadata(ctx, doc, "info:Producer", buf, sizeof buf) > 0)
@@ -1357,8 +1754,63 @@ static void do_info(void)
 		else
 			fz_strlcat(buf, "none", sizeof buf);
 		ui_label("Permissions: %s", buf);
+		ui_label("PDF %sdocument with %d update%s",
+			pdf_doc_was_linearized(ctx, pdoc) ? "linearized " : "",
+			updates, updates > 1 ? "s" : "");
+		if (updates > 0)
+		{
+			int n = pdf_validate_change_history(ctx, pdoc);
+			if (n == 0)
+				ui_label("Change history seems valid.");
+			else if (n == 1)
+				ui_label("Invalid changes made to the document in the last update.");
+			else if (n == 2)
+				ui_label("Invalid changes made to the document in the penultimate update.");
+			else
+				ui_label("Invalid changes made to the document %d updates ago.", n);
+		}
+
+		if (list.len)
+		{
+			int i;
+			for (i = 0; i < list.len; i++)
+			{
+				pdf_obj *field = list.sig[i];
+				fz_try(ctx)
+				{
+					if (pdf_signature_is_signed(ctx, pdf, field))
+					{
+						pdf_pkcs7_verifier *verifier = pkcs7_openssl_new_verifier(ctx);
+						pdf_signature_error sig_cert_error = pdf_check_certificate(ctx, verifier, pdf, field);
+						pdf_signature_error sig_digest_error = pdf_check_digest(ctx, verifier, pdf, field);
+						ui_label("Signature %d: CERT: %s, DIGEST: %s%s", i+1,
+							short_signature_error_desc(sig_cert_error),
+							short_signature_error_desc(sig_digest_error),
+								pdf_signature_incremental_change_since_signing(ctx, pdf, field) ? ", Changed since": "");
+
+						pdf_drop_verifier(ctx, verifier);
+					}
+					else
+						ui_label("Signature %d: Unsigned", i+1);
+				}
+				fz_catch(ctx)
+					ui_label("Signature %d: Error", i+1);
+			}
+			fz_free(ctx, list.sig);
+
+			if (updates == 0)
+				ui_label("No updates since document creation");
+			else
+			{
+				int n = pdf_validate_change_history(ctx, pdf);
+				if (n == 0)
+					ui_label("Document changes conform to permissions");
+				else
+					ui_label("Document permissions violated %d updates ago", n);
+			}
+		}
 	}
-	ui_label("Page: %d / %d", currentpage + 1, fz_count_pages(ctx, doc));
+	ui_label("Page: %d / %d", fz_page_number_from_location(ctx, doc, currentpage)+1, fz_count_pages(ctx, doc));
 	{
 		int w = (int)(page_bounds.x1 - page_bounds.x0 + 0.5f);
 		int h = (int)(page_bounds.y1 - page_bounds.y0 + 0.5f);
@@ -1372,7 +1824,6 @@ static void do_info(void)
 	}
 	ui_label("ICC rendering: %s.", currenticc ? "on" : "off");
 	ui_label("Spot rendering: %s.", currentseparations ? "on" : "off");
-
 	ui_dialog_end();
 }
 
@@ -1468,7 +1919,7 @@ static void do_canvas(void)
 		ui_layout(T, X, NW, 0, 0);
 		ui_panel_begin(0, ui.gridsize+8, 4, 4, 1);
 		ui_layout(L, NONE, W, 2, 0);
-		ui_label("Searching page %d of %d.", search_page + 1, fz_count_pages(ctx, doc));
+		ui_label("Searching chapter %d page %d...", search_page.chapter, search_page.page);
 		ui_panel_end();
 	}
 	else
@@ -1481,7 +1932,7 @@ static void do_canvas(void)
 		do_links(links);
 		do_page_selection();
 
-		if (search_hit_page == currentpage && search_hit_count > 0)
+		if (eqloc(search_hit_page, currentpage) && search_hit_count > 0)
 			do_search_hits();
 	}
 
@@ -1495,7 +1946,7 @@ static void do_canvas(void)
 		if (ui_input(&search_input, 0, 1) == UI_INPUT_ACCEPT)
 		{
 			showsearch = 0;
-			search_page = -1;
+			search_page = fz_make_location(-1, -1);
 			if (search_needle)
 			{
 				fz_free(ctx, search_needle);
@@ -1539,24 +1990,35 @@ void do_main(void)
 		ui.key = ui.mod = ui.plain = 0;
 		ui.down = ui.middle = ui.right = 0;
 
-		while (glutGet(GLUT_ELAPSED_TIME) < start_time + 200)
+		while (search_active && glutGet(GLUT_ELAPSED_TIME) < start_time + 200)
 		{
-			search_hit_count = fz_search_page_number(ctx, doc, search_page, search_needle,
-					search_hit_quads, nelem(search_hit_quads));
+			search_hit_count = fz_search_chapter_page_number(ctx, doc,
+				search_page.chapter, search_page.page,
+				search_needle,
+				search_hit_quads, nelem(search_hit_quads));
 			if (search_hit_count)
 			{
+				float search_hit_x = search_hit_quads[0].ul.x;
+				float search_hit_y = search_hit_quads[0].ul.y;
 				search_active = 0;
 				search_hit_page = search_page;
-				jump_to_page(search_hit_page);
-				break;
+				jump_to_location_xy(search_hit_page, search_hit_x, search_hit_y);
 			}
 			else
 			{
-				search_page += search_dir;
-				if (search_page < 0 || search_page == fz_count_pages(ctx, doc))
+				if (search_dir > 0)
 				{
-					search_active = 0;
-					break;
+					if (is_last_page(search_page))
+						search_active = 0;
+					else
+						search_page = fz_next_page(ctx, doc, search_page);
+				}
+				else
+				{
+					if (is_first_page(search_page))
+						search_active = 0;
+					else
+						search_page = fz_previous_page(ctx, doc, search_page);
 				}
 			}
 		}
@@ -1571,7 +2033,7 @@ void do_main(void)
 	if (showoutline)
 		do_outline(outline);
 
-	if (oldpage != currentpage || oldseparations != currentseparations || oldicc != currenticc)
+	if (!eqloc(oldpage, currentpage) || oldseparations != currentseparations || oldicc != currenticc)
 	{
 		load_page();
 		update_title();
@@ -1636,19 +2098,21 @@ static int document_filter(const char *filename)
 
 static void do_open_document_dialog(void)
 {
-	if (ui_open_file(filename))
+	if (ui_open_file(filename, "Select a document to open:"))
 	{
 		ui.dialog = NULL;
 		if (filename[0] == 0)
 			glutLeaveMainLoop();
 		else
-			load_document();
-		if (doc)
 		{
-			load_page();
-			render_page();
-			shrinkwrap();
-			update_title();
+			load_document();
+			if (doc)
+			{
+				load_page();
+				render_page();
+				shrinkwrap();
+				update_title();
+			}
 		}
 	}
 }
@@ -1656,14 +2120,16 @@ static void do_open_document_dialog(void)
 static void cleanup(void)
 {
 	save_history();
+	save_accelerator();
 
 	ui_finish();
 
 #ifndef NDEBUG
 	if (fz_atoi(getenv("FZ_DEBUG_STORE")))
-		fz_debug_store(ctx);
+		fz_debug_store(ctx, fz_stdout(ctx));
 #endif
 
+	fz_drop_output(ctx, trace_file);
 	fz_drop_stext_page(ctx, page_text);
 	fz_drop_separations(ctx, seps);
 	fz_drop_link(ctx, links);
@@ -1689,6 +2155,7 @@ int main_utf8(int argc, char **argv)
 int main(int argc, char **argv)
 #endif
 {
+	const char *trace_file_name = NULL;
 	int c;
 
 #ifndef _WIN32
@@ -1700,7 +2167,7 @@ int main(int argc, char **argv)
 	screen_w = glutGet(GLUT_SCREEN_WIDTH) - SCREEN_FURNITURE_W;
 	screen_h = glutGet(GLUT_SCREEN_HEIGHT) - SCREEN_FURNITURE_H;
 
-	while ((c = fz_getopt(argc, argv, "p:r:IW:H:S:U:XJA:B:C:")) != -1)
+	while ((c = fz_getopt(argc, argv, "p:r:IW:H:S:U:XJA:B:C:T:")) != -1)
 	{
 		switch (c)
 		{
@@ -1717,11 +2184,22 @@ int main(int argc, char **argv)
 		case 'A': currentaa = fz_atoi(fz_optarg); break;
 		case 'C': currenttint = 1; tint_white = strtol(fz_optarg, NULL, 16); break;
 		case 'B': currenttint = 1; tint_black = strtol(fz_optarg, NULL, 16); break;
+		case 'T': trace_file_name = fz_optarg; break;
 		}
 	}
 
-	ctx = fz_new_context(NULL, NULL, 0);
+	ctx = fz_new_context(NULL, NULL, FZ_STORE_DEFAULT);
 	fz_register_document_handlers(ctx);
+
+	if (trace_file_name)
+	{
+		if (!strcmp(trace_file_name, "-"))
+			trace_file = fz_stdout(ctx);
+		else
+			trace_file = fz_new_output_with_path(ctx, trace_file_name, 0);
+		trace_action("var doc, page, annot, widget, tmp;\n");
+	}
+
 	if (layout_css)
 	{
 		fz_buffer *buf = fz_read_file(ctx, layout_css);
