@@ -38,21 +38,21 @@ abstract public class Device
 	 */
 
 	abstract public void close();
-	abstract public void fillPath(Path path, boolean evenOdd, Matrix ctm, ColorSpace cs, float color[], float alpha, int cp);
-	abstract public void strokePath(Path path, StrokeState stroke, Matrix ctm, ColorSpace cs, float color[], float alpha, int cp);
+	abstract public void fillPath(Path path, boolean evenOdd, Matrix ctm, ColorSpace cs, float[] color, float alpha, int cp);
+	abstract public void strokePath(Path path, StrokeState stroke, Matrix ctm, ColorSpace cs, float[] color, float alpha, int cp);
 	abstract public void clipPath(Path path, boolean evenOdd, Matrix ctm);
 	abstract public void clipStrokePath(Path path, StrokeState stroke, Matrix ctm);
-	abstract public void fillText(Text text, Matrix ctm, ColorSpace cs, float color[], float alpha, int cp);
-	abstract public void strokeText(Text text, StrokeState stroke, Matrix ctm, ColorSpace cs, float color[], float alpha, int cp);
+	abstract public void fillText(Text text, Matrix ctm, ColorSpace cs, float[] color, float alpha, int cp);
+	abstract public void strokeText(Text text, StrokeState stroke, Matrix ctm, ColorSpace cs, float[] color, float alpha, int cp);
 	abstract public void clipText(Text text, Matrix ctm);
 	abstract public void clipStrokeText(Text text, StrokeState stroke, Matrix ctm);
 	abstract public void ignoreText(Text text, Matrix ctm);
 	abstract public void fillShade(Shade shd, Matrix ctm, float alpha, int cp);
 	abstract public void fillImage(Image img, Matrix ctm, float alpha, int cp);
-	abstract public void fillImageMask(Image img, Matrix ctm, ColorSpace cs, float color[], float alpha, int cp);
+	abstract public void fillImageMask(Image img, Matrix ctm, ColorSpace cs, float[] color, float alpha, int cp);
 	abstract public void clipImageMask(Image img, Matrix ctm);
 	abstract public void popClip();
-	abstract public void beginMask(Rect area, boolean luminosity, ColorSpace cs, float bc[], int cp);
+	abstract public void beginMask(Rect area, boolean luminosity, ColorSpace cs, float[] bc, int cp);
 	abstract public void endMask();
 	abstract public void beginGroup(Rect area, ColorSpace cs, boolean isolated, boolean knockout, int blendmode, float alpha);
 	abstract public void endGroup();
