@@ -4,26 +4,18 @@ public class Outline
 {
 	public String title;
 	public String uri;
-	public int page;
-	public Outline down[];
-	public float x;
-	public float y;
+	public Outline[] down;
 
-	public Outline(String title, int page, String uri, float x, float y, Outline[] down) {
+	public Outline(String title, String uri, Outline[] down) {
 		this.title = title;
-		this.page = page;
 		this.uri = uri;
 		this.down = down;
-		this.x = x;
-		this.y = y;
 	}
 
 	public String toString()
 	{
 		StringBuffer s = new StringBuffer();
 
-		s.append(page);
-		s.append(": ");
 		s.append(title);
 		s.append(' ');
 		s.append(uri);
