@@ -1,5 +1,4 @@
 #include "mupdf/fitz.h"
-#include "fitz-imp.h"
 
 #include <string.h>
 
@@ -263,18 +262,6 @@ static const unsigned char pkm[256*8] =
 	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
 };
 
-/*
-	Create a new bitmap.
-
-	w, h: Width and Height for the bitmap
-
-	n: Number of color components (assumed to be a divisor of 8)
-
-	xres, yres: X and Y resolutions (in pixels per inch).
-
-	Returns pointer to created bitmap structure. The bitmap
-	data is uninitialised.
-*/
 fz_bitmap *
 fz_new_bitmap(fz_context *ctx, int w, int h, int n, int xres, int yres)
 {
@@ -282,7 +269,7 @@ fz_new_bitmap(fz_context *ctx, int w, int h, int n, int xres, int yres)
 
 	/* Stride is 32 bit aligned. We may want to make this 64 bit if we use SSE2 etc. */
 	int stride = ((n * w + 31) & ~31) >> 3;
-	if (h > SIZE_MAX / stride)
+	if (h < 0 || ((size_t)h > (size_t)(SIZE_MAX / stride)))
 		fz_throw(ctx, FZ_ERROR_MEMORY, "bitmap too large");
 
 	bit = fz_malloc_struct(ctx, fz_bitmap);
@@ -295,7 +282,7 @@ fz_new_bitmap(fz_context *ctx, int w, int h, int n, int xres, int yres)
 		bit->xres = xres;
 		bit->yres = yres;
 		bit->stride = stride;
-		bit->samples = fz_malloc(ctx, h * bit->stride);
+		bit->samples = Memento_label(fz_malloc(ctx, h * bit->stride), "bitmap_samples");
 	}
 	fz_catch(ctx)
 	{
@@ -525,19 +512,6 @@ fz_save_pixmap_as_pkm(fz_context *ctx, fz_pixmap *pixmap, const char *filename)
 		fz_rethrow(ctx);
 }
 
-/*
-	Retrieve details of a given bitmap.
-
-	bitmap: The bitmap to query.
-
-	w: Pointer to storage to retrieve width (or NULL).
-
-	h: Pointer to storage to retrieve height (or NULL).
-
-	n: Pointer to storage to retrieve number of color components (or NULL).
-
-	stride: Pointer to storage to retrieve bitmap stride (or NULL).
-*/
 void fz_bitmap_details(fz_bitmap *bit, int *w, int *h, int *n, int *stride)
 {
 	if (!bit)
