@@ -2,9 +2,7 @@
 
 /* 4bit greyscale Thunderscan decoding */
 
-typedef struct fz_thunder_s fz_thunder;
-
-struct fz_thunder_s
+typedef struct
 {
 	fz_stream *chain;
 	int lastpixel;
@@ -13,7 +11,7 @@ struct fz_thunder_s
 
 	int len;
 	unsigned char *buffer;
-};
+} fz_thunder;
 
 static int
 next_thunder(fz_context *ctx, fz_stream *stm, size_t max)
@@ -131,7 +129,7 @@ fz_open_thunder(fz_context *ctx, fz_stream *chain, int w)
 		state->pixel = 0;
 		state->lastpixel = 0;
 		state->len = w / 2;
-		state->buffer = fz_malloc(ctx, state->len);
+		state->buffer = Memento_label(fz_malloc(ctx, state->len), "thunder_buffer");
 		state->chain = fz_keep_stream(ctx, chain);
 	}
 	fz_catch(ctx)
