@@ -1,28 +1,18 @@
 #include "mupdf/fitz.h"
 
+#include "glyph-imp.h"
+#include "pixmap-imp.h"
+
 #include <string.h>
 
 #define RLE_THRESHOLD 256
 
-/*
-	Take a reference to a glyph.
-
-	pix: The glyph to increment the reference for.
-
-	Returns pix.
-*/
 fz_glyph *
 fz_keep_glyph(fz_context *ctx, fz_glyph *glyph)
 {
 	return fz_keep_storable(ctx, &glyph->storable);
 }
 
-/*
-	Drop a reference and free a glyph.
-
-	Decrement the reference count for the glyph. When no
-	references remain the glyph will be freed.
-*/
 void
 fz_drop_glyph(fz_context *ctx, fz_glyph *glyph)
 {
@@ -59,18 +49,12 @@ fz_glyph_bbox_no_ctx(fz_glyph *glyph)
 	return bbox;
 }
 
-/*
-	Return the width of the glyph in pixels.
-*/
 int
 fz_glyph_width(fz_context *ctx, fz_glyph *glyph)
 {
 	return glyph->w;
 }
 
-/*
-	Return the height of the glyph in pixels.
-*/
 int
 fz_glyph_height(fz_context *ctx, fz_glyph *glyph)
 {
@@ -143,12 +127,6 @@ fz_dump_glyph(fz_glyph *glyph)
 }
 #endif
 
-/*
-	Create a new glyph from a pixmap
-
-	Returns a pointer to the new glyph. Throws exception on failure to
-	allocate.
-*/
 fz_glyph *
 fz_new_glyph_from_pixmap(fz_context *ctx, fz_pixmap *pix)
 {
@@ -187,20 +165,6 @@ fz_new_glyph_from_pixmap(fz_context *ctx, fz_pixmap *pix)
 	return glyph;
 }
 
-/*
-	Create a new glyph from 8bpp data
-
-	x, y: X and Y position for the glyph
-
-	w, h: Width and Height for the glyph
-
-	sp: Source Pointer to data
-
-	span: Increment from line to line of data
-
-	Returns a pointer to the new glyph. Throws exception on failure to
-	allocate.
-*/
 fz_glyph *
 fz_new_glyph_from_8bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigned char *sp, int span)
 {
@@ -222,7 +186,7 @@ fz_new_glyph_from_8bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigne
 
 		size = h * w;
 		fill = h * sizeof(int);
-		glyph = fz_malloc(ctx, sizeof(fz_glyph) + size);
+		glyph = Memento_label(fz_malloc(ctx, sizeof(fz_glyph) + size), "fz_glyph(8)");
 		FZ_INIT_STORABLE(glyph, 1, fz_drop_glyph_imp);
 		glyph->x = x;
 		glyph->y = y;
@@ -329,7 +293,7 @@ fz_new_glyph_from_8bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigne
 		 * and reenter the try context, and this routine is speed
 		 * critical. */
 try_pixmap:
-		glyph = fz_realloc(ctx, glyph, sizeof(fz_glyph));
+		glyph = Memento_label(fz_realloc(ctx, glyph, sizeof(fz_glyph)), "fz_glyph(8r)");
 		FZ_INIT_STORABLE(glyph, 1, fz_drop_glyph_imp);
 		pix = fz_new_pixmap_from_8bpp_data(ctx, x, y, w, h, orig_sp, span);
 		glyph->x = pix->x;
@@ -349,20 +313,6 @@ try_pixmap:
 	return glyph;
 }
 
-/*
-	Create a new glyph from 1bpp data
-
-	x, y: X and Y position for the glyph
-
-	w, h: Width and Height for the glyph
-
-	sp: Source Pointer to data
-
-	span: Increment from line to line of data
-
-	Returns a pointer to the new glyph. Throws exception on failure to
-	allocate.
-*/
 fz_glyph *
 fz_new_glyph_from_1bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigned char *sp, int span)
 {
@@ -384,7 +334,7 @@ fz_new_glyph_from_1bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigne
 
 		size = h * w;
 		fill = h * sizeof(int);
-		glyph = fz_malloc(ctx, sizeof(fz_glyph) + size);
+		glyph = Memento_label(fz_malloc(ctx, sizeof(fz_glyph) + size), "fz_glyph(1)");
 		FZ_INIT_STORABLE(glyph, 1, fz_drop_glyph_imp);
 		glyph->x = x;
 		glyph->y = y;
