@@ -1,5 +1,8 @@
 #include "mupdf/fitz.h"
 
+#include "image-imp.h"
+#include "pixmap-imp.h"
+
 #include <string.h>
 #include <limits.h>
 
