@@ -1,5 +1,7 @@
 #include "mupdf/fitz.h"
 
+#include "color-imp.h"
+
 #include <assert.h>
 #include <limits.h>
 #include <string.h>
@@ -30,29 +32,6 @@ fz_drop_pixmap_imp(fz_context *ctx, fz_storable *pix_)
 	fz_free(ctx, pix);
 }
 
-/*
-	Create a new pixmap, with its origin at
-	(0,0) using the supplied data block.
-
-	cs: The colorspace to use for the pixmap, or NULL for an alpha
-	plane/mask.
-
-	w: The width of the pixmap (in pixels)
-
-	h: The height of the pixmap (in pixels)
-
-	seps: Details of separations.
-
-	alpha: 0 for no alpha, 1 for alpha.
-
-	stride: The byte offset from the pixel data in a row to the pixel
-	data in the next row.
-
-	samples: The data block to keep the samples in.
-
-	Returns a pointer to the new pixmap. Throws exception on failure to
-	allocate.
-*/
 fz_pixmap *
 fz_new_pixmap_with_data(fz_context *ctx, fz_colorspace *colorspace, int w, int h, fz_separations *seps, int alpha, int stride, unsigned char *samples)
 {
@@ -103,7 +82,7 @@ fz_new_pixmap_with_data(fz_context *ctx, fz_colorspace *colorspace, int w, int h
 		{
 			if (pix->stride - 1 > INT_MAX / pix->n)
 				fz_throw(ctx, FZ_ERROR_GENERIC, "overly wide image");
-			pix->samples = fz_malloc(ctx, pix->h * pix->stride);
+			pix->samples = Memento_label(fz_malloc(ctx, pix->h * pix->stride), "pixmap_data");
 		}
 		fz_catch(ctx)
 		{
@@ -117,23 +96,6 @@ fz_new_pixmap_with_data(fz_context *ctx, fz_colorspace *colorspace, int w, int h
 	return pix;
 }
 
-/*
-	Create a new pixmap, with its origin at (0,0)
-
-	cs: The colorspace to use for the pixmap, or NULL for an alpha
-	plane/mask.
-
-	w: The width of the pixmap (in pixels)
-
-	h: The height of the pixmap (in pixels)
-
-	seps: Details of separations.
-
-	alpha: 0 for no alpha, 1 for alpha.
-
-	Returns a pointer to the new pixmap. Throws exception on failure to
-	allocate.
-*/
 fz_pixmap *
 fz_new_pixmap(fz_context *ctx, fz_colorspace *colorspace, int w, int h, fz_separations *seps, int alpha)
 {
@@ -144,27 +106,6 @@ fz_new_pixmap(fz_context *ctx, fz_colorspace *colorspace, int w, int h, fz_separ
 	return fz_new_pixmap_with_data(ctx, colorspace, w, h, seps, alpha, stride, NULL);
 }
 
-/*
-	Create a pixmap of a given size,
-	location and pixel format.
-
-	The bounding box specifies the size of the created pixmap and
-	where it will be located. The colorspace determines the number
-	of components per pixel. Alpha is always present. Pixmaps are
-	reference counted, so drop references using fz_drop_pixmap.
-
-	colorspace: Colorspace format used for the created pixmap. The
-	pixmap will keep a reference to the colorspace.
-
-	bbox: Bounding box specifying location/size of created pixmap.
-
-	seps: Details of separations.
-
-	alpha: 0 for no alpha, 1 for alpha.
-
-	Returns a pointer to the new pixmap. Throws exception on failure to
-	allocate.
-*/
 fz_pixmap *
 fz_new_pixmap_with_bbox(fz_context *ctx, fz_colorspace *colorspace, fz_irect bbox, fz_separations *seps, int alpha)
 {
@@ -175,29 +116,6 @@ fz_new_pixmap_with_bbox(fz_context *ctx, fz_colorspace *colorspace, fz_irect bbo
 	return pixmap;
 }
 
-/*
-	Create a pixmap of a given size,
-	location and pixel format, using the supplied data block.
-
-	The bounding box specifies the size of the created pixmap and
-	where it will be located. The colorspace determines the number
-	of components per pixel. Alpha is always present. Pixmaps are
-	reference counted, so drop references using fz_drop_pixmap.
-
-	colorspace: Colorspace format used for the created pixmap. The
-	pixmap will keep a reference to the colorspace.
-
-	rect: Bounding box specifying location/size of created pixmap.
-
-	seps: Details of separations.
-
-	alpha: Number of alpha planes (0 or 1).
-
-	samples: The data block to keep the samples in.
-
-	Returns a pointer to the new pixmap. Throws exception on failure to
-	allocate.
-*/
 fz_pixmap *
 fz_new_pixmap_with_bbox_and_data(fz_context *ctx, fz_colorspace *colorspace, fz_irect bbox, fz_separations *seps, int alpha, unsigned char *samples)
 {
@@ -213,17 +131,6 @@ fz_new_pixmap_with_bbox_and_data(fz_context *ctx, fz_colorspace *colorspace, fz_
 	return pixmap;
 }
 
-/*
-	Create a new pixmap that represents
-	a subarea of the specified pixmap. A reference is taken to his
-	pixmap that will be dropped on destruction.
-
-	The supplied rectangle must be wholly contained within the original
-	pixmap.
-
-	Returns a pointer to the new pixmap. Throws exception on failure to
-	allocate.
-*/
 fz_pixmap *fz_new_pixmap_from_pixmap(fz_context *ctx, fz_pixmap *pixmap, const fz_irect *rect)
 {
 	fz_irect local_rect;
@@ -266,9 +173,6 @@ fz_pixmap *fz_clone_pixmap(fz_context *ctx, fz_pixmap *old)
 	return pix;
 }
 
-/*
-	Return the bounding box for a pixmap.
-*/
 fz_irect
 fz_pixmap_bbox(fz_context *ctx, const fz_pixmap *pix)
 {
@@ -291,11 +195,6 @@ fz_pixmap_bbox_no_ctx(const fz_pixmap *pix)
 	return bbox;
 }
 
-/*
-	Return the colorspace of a pixmap
-
-	Returns colorspace.
-*/
 fz_colorspace *
 fz_pixmap_colorspace(fz_context *ctx, fz_pixmap *pix)
 {
@@ -304,100 +203,60 @@ fz_pixmap_colorspace(fz_context *ctx, fz_pixmap *pix)
 	return pix->colorspace;
 }
 
-/*
-	Return the x value of the pixmap in pixels.
-*/
 int
 fz_pixmap_x(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->x;
 }
 
-/*
-	Return the y value of the pixmap in pixels.
-*/
 int
 fz_pixmap_y(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->y;
 }
 
-/*
-	Return the width of the pixmap in pixels.
-*/
 int
 fz_pixmap_width(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->w;
 }
 
-/*
-	Return the height of the pixmap in pixels.
-*/
 int
 fz_pixmap_height(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->h;
 }
 
-/*
-	Return the number of components in a pixmap.
-
-	Returns the number of components (including spots and alpha).
-*/
 int
 fz_pixmap_components(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->n;
 }
 
-/*
-	Return the number of colorants in a pixmap.
-
-	Returns the number of colorants (components, less any spots and alpha).
-*/
 int
 fz_pixmap_colorants(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->n - pix->alpha - pix->s;
 }
 
-/*
-	Return the number of spots in a pixmap.
-
-	Returns the number of spots (components, less colorants and alpha). Does not throw exceptions.
-*/
 int
 fz_pixmap_spots(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->s;
 }
 
-/*
-	Return the number of alpha planes in a pixmap.
-
-	Returns the number of alphas. Does not throw exceptions.
-*/
 int
 fz_pixmap_alpha(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->alpha;
 }
 
-/*
-	Return the number of bytes in a row in the pixmap.
-*/
 int
 fz_pixmap_stride(fz_context *ctx, fz_pixmap *pix)
 {
 	return pix->stride;
 }
 
-/*
-	Returns a pointer to the pixel data of a pixmap.
-
-	Returns the pointer.
-*/
 unsigned char *
 fz_pixmap_samples(fz_context *ctx, fz_pixmap *pix)
 {
@@ -629,12 +488,6 @@ clear_cmyk_bitmap(unsigned char *samples, int w, int h, int spots, int stride, i
 	}
 }
 
-/*
-	Sets all components (including alpha) of
-	all pixels in a pixmap to 0.
-
-	pix: The pixmap to clear.
-*/
 void
 fz_clear_pixmap(fz_context *ctx, fz_pixmap *pix)
 {
@@ -693,17 +546,6 @@ fz_clear_pixmap(fz_context *ctx, fz_pixmap *pix)
 	}
 }
 
-/*
-	Clears a pixmap with the given value.
-
-	pix: The pixmap to clear.
-
-	value: Values in the range 0 to 255 are valid. Each component
-	sample for each pixel in the pixmap will be set to this value,
-	while alpha will always be set to 255 (non-transparent).
-*/
-/* This function is horrible, and should be removed from the
- * API and replaced with a less magic one. */
 void
 fz_clear_pixmap_with_value(fz_context *ctx, fz_pixmap *pix, int value)
 {
@@ -759,9 +601,6 @@ fz_clear_pixmap_with_value(fz_context *ctx, fz_pixmap *pix, int value)
 	}
 }
 
-/*
-	Fill pixmap with solid color.
-*/
 void
 fz_fill_pixmap_with_color(fz_context *ctx, fz_pixmap *pix, fz_colorspace *colorspace, float *color, fz_color_params color_params)
 {
@@ -835,17 +674,6 @@ fz_copy_pixmap_rect(fz_context *ctx, fz_pixmap *dest, fz_pixmap *src, fz_irect b
 	}
 }
 
-/*
-	Clears a subrect of a pixmap with the given value.
-
-	pix: The pixmap to clear.
-
-	value: Values in the range 0 to 255 are valid. Each component
-	sample for each pixel in the pixmap will be set to this value,
-	while alpha will always be set to 255 (non-transparent).
-
-	r: the rectangle.
-*/
 void
 fz_clear_pixmap_rect_with_value(fz_context *ctx, fz_pixmap *dest, int value, fz_irect b)
 {
@@ -959,12 +787,6 @@ fz_alpha_from_gray(fz_context *ctx, fz_pixmap *gray)
 	return alpha;
 }
 
-/*
-	Tint all the pixels in an RGB, BGR, or Gray pixmap.
-
-	black: Map black to this hexadecimal RGB color.
-	white: Map white to this hexadecimal RGB color.
-*/
 void
 fz_tint_pixmap(fz_context *ctx, fz_pixmap *pix, int black, int white)
 {
@@ -1022,33 +844,43 @@ fz_tint_pixmap(fz_context *ctx, fz_pixmap *pix, int black, int white)
 	}
 }
 
-/* Invert luminance in RGB pixmap, but keep the colors as is. */
-static void invert_luminance_rgb(unsigned char *rgb)
+/* Invert luminance in RGB/BGR pixmap, but keep the colors as is. */
+static inline void invert_luminance(int type, unsigned char *s)
 {
-	int r, g, b, y, u, v, c, d, e;
+	int r, g, b, y;
 
 	/* Convert to YUV */
-	r = rgb[0];
-	g = rgb[1];
-	b = rgb[2];
-	y = ((66 * r + 129 * g + 25 * b + 128) >> 8) + 16;
-	u = ((-38 * r - 74 * g + 112 * b + 128) >> 8) + 128;
-	v = ((112 * r - 94 * g - 18 * b + 128) >> 8) + 128;
+	if (type == FZ_COLORSPACE_RGB)
+	{
+		r = s[0];
+		g = s[1];
+		b = s[2];
+	}
+	else
+	{
+		r = s[2];
+		g = s[1];
+		b = s[0];
+	}
 
-	/* Invert luminance */
-	y = 255 - y;
+	y = (39336 * r + 76884 * g + 14900 * b + 32768)>>16;
+	y = 259-y;
+	r += y;
+	g += y;
+	b += y;
 
-	/* Convert to RGB */
-	c = y - 16;
-	d = u - 128;
-	e = v - 128;
-	r = (298 * c + 409 * e + 128) >> 8;
-	g = (298 * c - 100 * d - 208 * e + 128) >> 8;
-	b = (298 * c + 516 * d + 128) >> 8;
-
-	rgb[0] = r > 255 ? 255 : r < 0 ? 0 : r;
-	rgb[1] = g > 255 ? 255 : g < 0 ? 0 : g;
-	rgb[2] = b > 255 ? 255 : b < 0 ? 0 : b;
+	if (type == FZ_COLORSPACE_RGB)
+	{
+		s[0] = r > 255 ? 255 : r < 0 ? 0 : r;
+		s[1] = g > 255 ? 255 : g < 0 ? 0 : g;
+		s[2] = b > 255 ? 255 : b < 0 ? 0 : b;
+	}
+	else
+	{
+		s[2] = r > 255 ? 255 : r < 0 ? 0 : r;
+		s[1] = g > 255 ? 255 : g < 0 ? 0 : g;
+		s[0] = b > 255 ? 255 : b < 0 ? 0 : b;
+	}
 }
 
 void
@@ -1056,25 +888,30 @@ fz_invert_pixmap_luminance(fz_context *ctx, fz_pixmap *pix)
 {
 	unsigned char *s = pix->samples;
 	int x, y, n = pix->n;
+	int type = pix->colorspace ? pix->colorspace->type : FZ_COLORSPACE_NONE;
 
-	if (pix->colorspace->type != FZ_COLORSPACE_RGB)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "can only invert luminance of RGB pixmaps");
-
-	for (y = 0; y < pix->h; y++)
+	if (type == FZ_COLORSPACE_GRAY)
 	{
-		for (x = 0; x < pix->w; x++)
+		fz_invert_pixmap(ctx, pix);
+	}
+	else if (type == FZ_COLORSPACE_RGB || type == FZ_COLORSPACE_BGR)
+	{
+		for (y = 0; y < pix->h; y++)
 		{
-			invert_luminance_rgb(s);
-			s += n;
+			for (x = 0; x < pix->w; x++)
+			{
+				invert_luminance(type, s);
+				s += n;
+			}
+			s += pix->stride - pix->w * n;
 		}
-		s += pix->stride - pix->w * n;
+	}
+	else
+	{
+		fz_throw(ctx, FZ_ERROR_GENERIC, "can only invert luminance of Gray and RGB pixmaps");
 	}
 }
 
-/*
-	Invert all the pixels in a pixmap. All components
-	of all pixels are inverted (except alpha, which is unchanged).
-*/
 void
 fz_invert_pixmap(fz_context *ctx, fz_pixmap *pix)
 {
@@ -1095,11 +932,6 @@ fz_invert_pixmap(fz_context *ctx, fz_pixmap *pix)
 	}
 }
 
-/*
-	Invert all the pixels in a given rectangle of a
-	pixmap. All components of all pixels in the rectangle are inverted
-	(except alpha, which is unchanged).
-*/
 void fz_invert_pixmap_rect(fz_context *ctx, fz_pixmap *image, fz_irect rect)
 {
 	unsigned char *p;
@@ -1122,12 +954,6 @@ void fz_invert_pixmap_rect(fz_context *ctx, fz_pixmap *image, fz_irect rect)
 	}
 }
 
-/*
-	Apply gamma correction to a pixmap. All components
-	of all pixels are modified (except alpha, which is unchanged).
-
-	gamma: The gamma value to apply; 1.0 for no change.
-*/
 void
 fz_gamma_pixmap(fz_context *ctx, fz_pixmap *pix, float gamma)
 {
@@ -1160,25 +986,6 @@ fz_pixmap_size(fz_context *ctx, fz_pixmap * pix)
 	return sizeof(*pix) + pix->n * pix->w * pix->h;
 }
 
-/*
-	Convert an existing pixmap to a desired
-	colorspace. Other properties of the pixmap, such as resolution
-	and position are copied to the converted pixmap.
-
-	pix: The pixmap to convert.
-
-	default_cs: If NULL pix->colorspace is used. It is possible that the data
-	may need to be interpreted as one of the color spaces in default_cs.
-
-	cs_des: Desired colorspace, may be NULL to denote alpha-only.
-
-	prf: Proofing color space through which we need to convert.
-
-	color_params: Parameters that may be used in conversion (e.g. ri).
-
-	keep_alpha: If 0 any alpha component is removed, otherwise
-	alpha is kept if present in the pixmap.
-*/
 fz_pixmap *
 fz_convert_pixmap(fz_context *ctx, fz_pixmap *pix, fz_colorspace *ds, fz_colorspace *prf, fz_default_colorspaces *default_cs, fz_color_params color_params, int keep_alpha)
 {
@@ -1256,6 +1063,32 @@ fz_new_pixmap_from_1bpp_data(fz_context *ctx, int x, int y, int w, int h, unsign
 	return pixmap;
 }
 
+int
+fz_is_pixmap_monochrome(fz_context *ctx, fz_pixmap *pixmap)
+{
+	int n = pixmap->n + pixmap->s + pixmap->alpha;
+	int w = pixmap->w;
+	int h = pixmap->h;
+	unsigned char *s = pixmap->samples;
+	int x;
+
+	if (n != 1)
+		return 0;
+
+	while (h--)
+	{
+		for (x = 0; x < w; ++x)
+		{
+			unsigned char v = s[x];
+			if (v != 0 && v != 255)
+				return 0;
+		}
+		s += pixmap->stride;
+	}
+
+	return 1;
+}
+
 #ifdef ARCH_ARM
 static void
 fz_subsample_pixmap_ARM(unsigned char *ptr, int w, int h, int f, int factor,
@@ -1584,9 +1417,6 @@ fz_subsample_pixmap(fz_context *ctx, fz_pixmap *tile, int factor)
 	tile->samples = fz_realloc(ctx, tile->samples, dst_h * dst_w * n);
 }
 
-/*
-	Set the pixels per inch resolution of the pixmap.
-*/
 void
 fz_set_pixmap_resolution(fz_context *ctx, fz_pixmap *pix, int xres, int yres)
 {
@@ -1645,9 +1475,6 @@ int fz_valgrind_pixmap(const fz_pixmap *pix)
 }
 #endif /* HAVE_VALGRIND */
 
-/*
- * Convert pixmap from indexed to base colorspace.
- */
 fz_pixmap *
 fz_convert_indexed_pixmap_to_base(fz_context *ctx, const fz_pixmap *src)
 {
@@ -1717,9 +1544,6 @@ fz_convert_indexed_pixmap_to_base(fz_context *ctx, const fz_pixmap *src)
 	return dst;
 }
 
-/*
- * Convert pixmap from DeviceN/Separation to base colorspace.
- */
 fz_pixmap *
 fz_convert_separation_pixmap_to_base(fz_context *ctx, const fz_pixmap *src)
 {
@@ -1751,38 +1575,80 @@ fz_convert_separation_pixmap_to_base(fz_context *ctx, const fz_pixmap *src)
 		sn = ss->n;
 		bn = base->n;
 
-		if (src->alpha)
+		if (base->type == FZ_COLORSPACE_LAB)
 		{
-			for (y = 0; y < src->h; y++)
+			if (src->alpha)
 			{
-				for (x = 0; x < src->w; x++)
+				for (y = 0; y < src->h; y++)
 				{
-					for (k = 0; k < sn; ++k)
-						src_v[k] = *s++ / 255.0f;
-					a = *s++;
-					ss->u.separation.eval(ctx, ss->u.separation.tint, src_v, sn, base_v, bn);
-					for (k = 0; k < bn; ++k)
-						*d++ = base_v[k] * 255.0f;
-					*d++ = a;
+					for (x = 0; x < src->w; x++)
+					{
+						for (k = 0; k < sn; ++k)
+							src_v[k] = *s++ / 255.0f;
+						a = *s++;
+						ss->u.separation.eval(ctx, ss->u.separation.tint, src_v, sn, base_v, bn);
+						*d++ = (base_v[0] / 100) * 255.0f;
+						*d++ = base_v[1] + 128;
+						*d++ = base_v[2] + 128;
+						*d++ = a;
+					}
+					s += s_line_inc;
+					d += d_line_inc;
+				}
+			}
+			else
+			{
+				for (y = 0; y < src->h; y++)
+				{
+					for (x = 0; x < src->w; x++)
+					{
+						for (k = 0; k < sn; ++k)
+							src_v[k] = *s++ / 255.0f;
+						ss->u.separation.eval(ctx, ss->u.separation.tint, src_v, sn, base_v, bn);
+						*d++ = (base_v[0] / 100) * 255.0f;
+						*d++ = base_v[1] + 128;
+						*d++ = base_v[2] + 128;
+					}
+					s += s_line_inc;
+					d += d_line_inc;
 				}
-				s += s_line_inc;
-				d += d_line_inc;
 			}
 		}
 		else
 		{
-			for (y = 0; y < src->h; y++)
+			if (src->alpha)
 			{
-				for (x = 0; x < src->w; x++)
+				for (y = 0; y < src->h; y++)
 				{
-					for (k = 0; k < sn; ++k)
-						src_v[k] = *s++ / 255.0f;
-					ss->u.separation.eval(ctx, ss->u.separation.tint, src_v, sn, base_v, bn);
-					for (k = 0; k < bn; ++k)
-						*d++ = base_v[k] * 255.0f;
+					for (x = 0; x < src->w; x++)
+					{
+						for (k = 0; k < sn; ++k)
+							src_v[k] = *s++ / 255.0f;
+						a = *s++;
+						ss->u.separation.eval(ctx, ss->u.separation.tint, src_v, sn, base_v, bn);
+						for (k = 0; k < bn; ++k)
+							*d++ = base_v[k] * 255.0f;
+						*d++ = a;
+					}
+					s += s_line_inc;
+					d += d_line_inc;
+				}
+			}
+			else
+			{
+				for (y = 0; y < src->h; y++)
+				{
+					for (x = 0; x < src->w; x++)
+					{
+						for (k = 0; k < sn; ++k)
+							src_v[k] = *s++ / 255.0f;
+						ss->u.separation.eval(ctx, ss->u.separation.tint, src_v, sn, base_v, bn);
+						for (k = 0; k < bn; ++k)
+							*d++ = base_v[k] * 255.0f;
+					}
+					s += s_line_inc;
+					d += d_line_inc;
 				}
-				s += s_line_inc;
-				d += d_line_inc;
 			}
 		}
 
