@@ -4,6 +4,107 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+/* #define FZ_XML_SEQ */
+
+/* We bend the XML parser slightly when it's reading for HTML.
+ * To do this, we use extra knowledge about HTML tags, expressed
+ * in the table in a header. */
+
+#define START_OPTIONAL 1
+#define END_OPTIONAL 2
+#define END_FORBIDDEN 4
+#define DEPRECATED 8
+#define DTD_LOOSE 16
+#define DTD_FRAMESET 32
+/* The following values are added by us. */
+/* If a tag can contain nested instances of itself, we
+ * mark it as being a container. */
+#define CONTAINER 64
+/* Table tags autoclose each other in complex ways. */
+#define TABLE_SHIFT 7
+#define TABLE_MASK 7
+/* Which level of a table are we? */
+/* 128 * 1 = TABLE
+ * 128 * 2 = COLGROUP
+ * 128 * 3 = COL
+ * 128 * 4 = THEAD/TBODY/TFOOT
+ * 128 * 5 = TR
+ * 128 * 6 = TD/TH
+ * Any table tag, will be autoclosed by the opening of another table tag
+ * (within the same container) of a smaller level than it. */
+#define IMPLIES_SHIFT 10
+#define IMPLIES_SHIFT2 17
+#define IMPLIES_SHIFT3 24
+#define IMPLIES_MASK 127
+/* If a tag should always be contained within another one, we
+ * indicate this with an 'implies'. TABLE tags never imply
+ * out of the current table. */
+
+typedef struct { char tag[16]; int flags; } fz_xml_html_tag_t;
+
+#define HTML_TAG(A,B,C,D,E) fz_xml_html_tag_ ## A
+
+enum
+{
+	fz_xml_html_tag__NONE,
+#include "html-tags.h"
+	, fz_xml_html_tag__NUMTAGS
+};
+
+#define HTML_TAG(A,B,C,D,E) { # A, E | (fz_xml_html_tag_ ## B << IMPLIES_SHIFT) | (fz_xml_html_tag_ ## C << IMPLIES_SHIFT2)  | (fz_xml_html_tag_ ## D << IMPLIES_SHIFT3) }
+
+fz_xml_html_tag_t html_tags[] =
+{
+	{ "", 0 },
+#include "html-tags.h"
+};
+
+/*
+When parsing XML, we assume that all tags are properly terminated.
+i.e. <foo> has </foo>  or <foo />.
+We currently don't check this at all. In fact, we generate an incorrect
+tree if this isn't the case.
+
+For example:
+
+<a><b><c></b><d></d></a>
+
+Will produce:
+  a
+  b
+  cd
+
+Rather than:
+  a
+  bd
+  c
+
+Over the course of a large HTML file, this can lead to HUGE "south easterly" skew
+in the tree.
+
+This happens because (when parsing pure xml) when we hit </d>, we don't check that the tag we are closing is actually a <d>.
+
+So, some heuristics to use when parsing HTML:
+  * When we open a tag 'foo', if we are immediately in another 'foo', then close the first 'foo' first.
+  * When we close a tag 'foo', run up the stack looking for an enclosing 'foo'. If we find one, close
+	everything up to and including that. If we don't find one, don't close anything.
+
+With these heuristics, we get the following for free:
+  * A TD closes any other TD.
+  * A /TR closes and open TD.
+  * A TR closes any open TR.
+
+This leaves problems with:
+  * Nested tables.
+  * Nested divs.
+  * Nested spans.
+
+Tables, divs and spans (let alone nested ones) are problematic anyway. Ignore
+this for now.
+
+We could special case TABLE, DIV and SPAN so appropriate tags don't pop up past them.
+*/
+
 static const struct { const char *name; int c; } html_entities[] = {
 	{"nbsp",160}, {"iexcl",161}, {"cent",162}, {"pound",163},
 	{"curren",164}, {"yen",165}, {"brvbar",166}, {"sect",167},
@@ -72,30 +173,49 @@ struct parser
 	fz_pool *pool;
 	fz_xml *head;
 	int preserve_white;
+	int for_html;
 	int depth;
+#ifdef FZ_XML_SEQ
+	int seq;
+#endif
 };
 
 struct attribute
 {
-	char name[40];
 	char *value;
 	struct attribute *next;
+	char name[1];
 };
 
-struct fz_xml_doc_s
+struct fz_xml_doc
 {
 	fz_pool *pool;
 	fz_xml *root;
 };
 
-struct fz_xml_s
+/* Text nodes never use the down pointer. Therefore
+ * if the down pointer is the MAGIC_TEXT value, we
+ * know there is text. */
+struct fz_xml
 {
-	char name[40];
-	char *text;
-	struct attribute *atts;
-	fz_xml *up, *down, *tail, *prev, *next;
+	fz_xml *up, *down, *prev, *next;
+#ifdef FZ_XML_SEQ
+	int seq;
+#endif
+	union
+	{
+		char text[1];
+		struct
+		{
+			struct attribute *atts;
+			char name[1];
+		} d;
+	} u;
 };
 
+#define MAGIC_TEXT ((fz_xml *)1)
+#define FZ_TEXT_ITEM(item) (item && item->down == MAGIC_TEXT)
+
 static void xml_indent(int n)
 {
 	while (n--) {
@@ -104,14 +224,11 @@ static void xml_indent(int n)
 	}
 }
 
-/*
-	Pretty-print an XML tree to stdout.
-*/
 void fz_debug_xml(fz_xml *item, int level)
 {
-	if (item->text)
+	char *s = fz_xml_text(item);
+	if (s)
 	{
-		char *s = item->text;
 		int c;
 		xml_indent(level);
 		putchar('"');
@@ -135,6 +252,10 @@ void fz_debug_xml(fz_xml *item, int level)
 			case '\t': putchar('\\'); putchar('t'); break;
 			}
 		}
+		putchar('"');
+#ifdef FZ_XML_SEQ
+		printf(" <%d>", item->seq);
+#endif
 		putchar('\n');
 	}
 	else
@@ -143,88 +264,70 @@ void fz_debug_xml(fz_xml *item, int level)
 		struct attribute *att;
 
 		xml_indent(level);
-		printf("(%s\n", item->name);
-		for (att = item->atts; att; att = att->next)
+#ifdef FZ_XML_SEQ
+		printf("(%s <%d>\n", item->u.d.name, item->seq);
+#else
+		printf("(%s\n", item->u.d.name);
+#endif
+		for (att = item->u.d.atts; att; att = att->next)
 		{
 			xml_indent(level);
 			printf("=%s %s\n", att->name, att->value);
 		}
-		for (child = item->down; child; child = child->next)
+		for (child = fz_xml_down(item); child; child = child->next)
 			fz_debug_xml(child, level + 1);
 		xml_indent(level);
-		printf(")%s\n", item->name);
+#ifdef FZ_XML_SEQ
+		printf(")%s <%d>\n", item->u.d.name, item->seq);
+#else
+		printf(")%s\n", item->u.d.name);
+#endif
 	}
 }
 
-/*
-	Return previous sibling of XML node.
-*/
 fz_xml *fz_xml_prev(fz_xml *item)
 {
 	return item ? item->prev : NULL;
 }
 
-/*
-	Return next sibling of XML node.
-*/
 fz_xml *fz_xml_next(fz_xml *item)
 {
 	return item ? item->next : NULL;
 }
 
-/*
-	Return parent of XML node.
-*/
 fz_xml *fz_xml_up(fz_xml *item)
 {
 	return item ? item->up : NULL;
 }
 
-/*
-	Return first child of XML node.
-*/
 fz_xml *fz_xml_down(fz_xml *item)
 {
-	return item ? item->down : NULL;
+	return item && !FZ_TEXT_ITEM(item) ? item->down : NULL;
 }
 
-/*
-	Return the text content of an XML node.
-	Return NULL if the node is a tag.
-*/
 char *fz_xml_text(fz_xml *item)
 {
-	return item ? item->text : NULL;
+	return (item && FZ_TEXT_ITEM(item)) ? item->u.text : NULL;
 }
 
-/*
-	Return tag of XML node. Return NULL for text nodes.
-*/
 char *fz_xml_tag(fz_xml *item)
 {
-	return item && item->name[0] ? item->name : NULL;
+	return item && !FZ_TEXT_ITEM(item) && item->u.d.name[0] ? item->u.d.name : NULL;
 }
 
-/*
-	Return true if the tag name matches.
-*/
 int fz_xml_is_tag(fz_xml *item, const char *name)
 {
-	if (!item)
+	if (!item || FZ_TEXT_ITEM(item))
 		return 0;
-	return !strcmp(item->name, name);
+	return !strcmp(item->u.d.name, name);
 }
 
-/*
-	Return the value of an attribute of an XML node.
-	NULL if the attribute doesn't exist.
-*/
 char *fz_xml_att(fz_xml *item, const char *name)
 {
 	struct attribute *att;
-	if (!item)
+	if (!item || FZ_TEXT_ITEM(item))
 		return NULL;
-	for (att = item->atts; att; att = att->next)
+	for (att = item->u.d.atts; att; att = att->next)
 		if (!strcmp(att->name, name))
 			return att->value;
 	return NULL;
@@ -234,7 +337,7 @@ fz_xml *fz_xml_find(fz_xml *item, const char *tag)
 {
 	while (item)
 	{
-		if (!strcmp(item->name, tag))
+		if (!strcmp(item->u.d.name, tag))
 			return item;
 		item = item->next;
 	}
@@ -251,28 +354,57 @@ fz_xml *fz_xml_find_next(fz_xml *item, const char *tag)
 fz_xml *fz_xml_find_down(fz_xml *item, const char *tag)
 {
 	if (item)
-		item = item->down;
+		item = fz_xml_down(item);
 	return fz_xml_find(item, tag);
 }
 
+int fz_xml_att_eq(fz_xml *item, const char *name, const char *match)
+{
+	const char *val = fz_xml_att(item, name);
+
+	return val ? !strcmp(val, match) : 0;
+}
+
+fz_xml *fz_xml_find_match(fz_xml *item, const char *tag, const char *att, const char *match)
+{
+	while (1)
+	{
+		item = fz_xml_find(item, tag);
+		if (item == NULL || fz_xml_att_eq(item, att, match))
+			break;
+		item = item->next;
+	}
+
+	return item;
+}
+
+fz_xml *fz_xml_find_next_match(fz_xml *item, const char *tag, const char *att, const char *match)
+{
+	do
+	{
+		item = fz_xml_find_next(item, tag);
+	}
+	while (item != NULL && !fz_xml_att_eq(item, att, match));
+
+	return item;
+}
+
+fz_xml *fz_xml_find_down_match(fz_xml *item, const char *tag, const char *att, const char *match)
+{
+	return fz_xml_find_match(fz_xml_down(item), tag, att, match);
+}
+
 fz_xml *fz_xml_root(fz_xml_doc *xml)
 {
 	return xml ? xml->root : NULL;
 }
 
-/*
-	Free the XML node and all its children and siblings.
-*/
 void fz_drop_xml(fz_context *ctx, fz_xml_doc *xml)
 {
 	if (xml)
 		fz_drop_pool(ctx, xml->pool);
 }
 
-/*
-	Detach a node from the tree, unlinking it from its parent,
-	and setting the document root to the node.
-*/
 void fz_detach_xml(fz_context *ctx, fz_xml_doc *xml, fz_xml *node)
 {
 	if (node->up)
@@ -340,63 +472,126 @@ static inline int iswhite(int c)
 	return c == ' ' || c == '\r' || c == '\n' || c == '\t';
 }
 
-static void xml_emit_open_tag(fz_context *ctx, struct parser *parser, char *a, char *b)
+static int
+find_html_tag(const char *tag, size_t len)
+{
+	int low = 0;
+	int high = nelem(html_tags);
+	int mid;
+
+	while (low != high)
+	{
+		int cmp;
+		mid = (low + high)>>1;
+		cmp = strncmp(html_tags[mid].tag, tag, len);
+		if (cmp == 0)
+			cmp = html_tags[mid].tag[len];
+		if (cmp == 0)
+			return mid;
+		if (cmp < 0)
+			low = mid+1;
+		else
+			high = mid;
+	}
+
+	return fz_xml_html_tag__NONE;
+}
+
+static int xml_emit_open_tag(fz_context *ctx, struct parser *parser, char *a, char *b, int is_text)
 {
 	fz_xml *head, *tail;
-	char *ns;
+	size_t size;
+	int autoclose = 0;
 
-	/* skip namespace prefix */
-	for (ns = a; ns < b; ++ns)
-		if (*ns == ':')
-			a = ns + 1;
+	if (is_text)
+		size = offsetof(fz_xml, u.text) + b-a+1;
+	else
+		size = offsetof(fz_xml, u.d.name) + b-a+1;
+	head = fz_pool_alloc(ctx, parser->pool, size);
 
-	head = fz_pool_alloc(ctx, parser->pool, sizeof *head);
-	if (b - a > sizeof(head->name) - 1)
-		b = a + sizeof(head->name) - 1;
-	memcpy(head->name, a, b - a);
-	head->name[b - a] = 0;
+	if (is_text)
+		head->down = MAGIC_TEXT;
+	else
+	{
+		if (parser->for_html)
+		{
+			int tag_num;
+
+			/* Lowercase the tag */
+			char *s = head->u.d.name;
+			char *t = a;
+			while (t != b)
+			{
+				char c = *t++;
+				if (c >= 'A' && c <= 'Z')
+					c += 'a' - 'A';
+				*s++ = c;
+			}
+
+			tag_num = find_html_tag(head->u.d.name, b-a);
+			if (tag_num != fz_xml_html_tag__NONE && (html_tags[tag_num].flags & END_FORBIDDEN))
+				autoclose = 1;
+			/* If parsing as HTML, and we see a FictionBook tag, turn off HTML fixups. */
+			if (tag_num == fz_xml_html_tag_fictionbook)
+			{
+				/* Restore original case. */
+				memcpy(head->u.d.name, "FictionBook", 11);
+				parser->for_html = 0;
+			}
+		}
+		else
+			memcpy(head->u.d.name, a, b - a);
+		head->u.d.name[b - a] = 0;
+		head->u.d.atts = NULL;
+		head->down = NULL;
+	}
 
-	head->atts = NULL;
-	head->text = NULL;
 	head->up = parser->head;
-	head->down = NULL;
-	head->prev = NULL;
 	head->next = NULL;
+#ifdef FZ_XML_SEQ
+	head->seq = parser->seq++;
+#endif
 
+	/* During construction, we use head->next to mean "the
+	 * tail of the children. When we close the tag, we
+	 * rewrite it to be NULL. */
 	if (!parser->head->down) {
 		parser->head->down = head;
-		parser->head->tail = head;
+		parser->head->next = head;
+		head->prev = NULL;
 	}
 	else {
-		tail = parser->head->tail;
+		tail = parser->head->next;
 		tail->next = head;
 		head->prev = tail;
-		parser->head->tail = head;
+		parser->head->next = head;
 	}
 
 	parser->head = head;
 	parser->depth++;
+
+	return autoclose;
 }
 
 static void xml_emit_att_name(fz_context *ctx, struct parser *parser, char *a, char *b)
 {
 	fz_xml *head = parser->head;
 	struct attribute *att;
+	size_t size;
 
-	att = fz_pool_alloc(ctx, parser->pool, sizeof *att);
-	if (b - a > sizeof(att->name) - 1)
-		b = a + sizeof(att->name) - 1;
+	size = offsetof(struct attribute, name) + b-a+1;
+	att = fz_pool_alloc(ctx, parser->pool, size);
 	memcpy(att->name, a, b - a);
 	att->name[b - a] = 0;
 	att->value = NULL;
-	att->next = head->atts;
-	head->atts = att;
+	att->next = head->u.d.atts;
+	head->u.d.atts = att;
 }
 
 static void xml_emit_att_value(fz_context *ctx, struct parser *parser, char *a, char *b)
 {
 	fz_xml *head = parser->head;
-	struct attribute *att = head->atts;
+	struct attribute *att = head->u.d.atts;
 	char *s;
 	int c;
 
@@ -417,13 +612,13 @@ static void xml_emit_att_value(fz_context *ctx, struct parser *parser, char *a,
 static void xml_emit_close_tag(fz_context *ctx, struct parser *parser)
 {
 	parser->depth--;
+	parser->head->next = NULL;
 	if (parser->head->up)
 		parser->head = parser->head->up;
 }
 
 static void xml_emit_text(fz_context *ctx, struct parser *parser, char *a, char *b)
 {
-	static char *empty = "";
 	fz_xml *head;
 	char *s;
 	int c;
@@ -442,11 +637,11 @@ static void xml_emit_text(fz_context *ctx, struct parser *parser, char *a, char
 			return;
 	}
 
-	xml_emit_open_tag(ctx, parser, empty, empty);
+	(void)xml_emit_open_tag(ctx, parser, a, b, 1);
 	head = parser->head;
 
 	/* entities are all longer than UTFmax so runetochar is safe */
-	s = head->text = fz_pool_alloc(ctx, parser->pool, b - a + 1);
+	s = fz_xml_text(head);
 	while (a < b) {
 		if (*a == '&') {
 			a += xml_parse_entity(&c, a);
@@ -463,14 +658,13 @@ static void xml_emit_text(fz_context *ctx, struct parser *parser, char *a, char
 
 static void xml_emit_cdata(fz_context *ctx, struct parser *parser, char *a, char *b)
 {
-	static char *empty = "";
 	fz_xml *head;
 	char *s;
 
-	xml_emit_open_tag(ctx, parser, empty, empty);
+	(void)xml_emit_open_tag(ctx, parser, a, b, 1);
 	head = parser->head;
 
-	s = head->text = fz_pool_alloc(ctx, parser->pool, b - a + 1);
+	s = head->u.text;
 	while (a < b)
 		*s++ = *a++;
 	*s = 0;
@@ -478,17 +672,200 @@ static void xml_emit_cdata(fz_context *ctx, struct parser *parser, char *a, char
 	xml_emit_close_tag(ctx, parser);
 }
 
+static int
+pop_to_tag(fz_context *ctx, struct parser *parser, char *mark, char *p)
+{
+	fz_xml *to, *head;
+
+	/* Run up from the tag */
+	if (parser->for_html)
+	{
+		for (to = parser->head; to; to = to->up)
+		{
+			char *tag = fz_xml_tag(to);
+			if (tag && fz_strncasecmp(tag, mark, p-mark) == 0 && tag[p-mark] == 0)
+				break; /* Found a matching tag */
+		}
+	}
+	else
+	{
+		for (to = parser->head; to; to = to->up)
+		{
+			char *tag = fz_xml_tag(to);
+			if (tag && strncmp(tag, mark, p-mark) == 0 && tag[p-mark] == 0)
+				break; /* Found a matching tag */
+		}
+	}
+
+	if (to == NULL)
+	{
+		/* We didn't find a matching enclosing tag. Don't close anything. */
+		return 0;
+	}
+
+	/* Pop everything up to and including this tag. */
+	for (head = parser->head; head != to; head = head->up)
+		xml_emit_close_tag(ctx, parser);
+	return 1;
+}
+
+static void
+open_implied(fz_context *ctx, struct parser *parser, int tag)
+{
+	fz_xml *head;
+	int implied, implied2, implied3, tag_num;
+	int table_level;
+
+	if (tag == fz_xml_html_tag__NONE)
+		return;
+
+	implied  = (html_tags[tag].flags >> IMPLIES_SHIFT) & IMPLIES_MASK;
+	implied2 = (html_tags[tag].flags >> IMPLIES_SHIFT2) & IMPLIES_MASK;
+	implied3 = (html_tags[tag].flags >> IMPLIES_SHIFT3) & IMPLIES_MASK;
+	if (implied == fz_xml_html_tag__NONE)
+		return;
+	if (implied2 == fz_xml_html_tag__NONE)
+		implied2 = implied;
+	if (implied3 == fz_xml_html_tag__NONE)
+		implied3 = implied;
+
+	/* So, check to see whether implied{,2,3} is present. */
+	table_level = (html_tags[tag].flags>>TABLE_SHIFT) & TABLE_MASK;
+	if (table_level != 0)
+	{
+		/* Table tag. Autoclose anything within the current TABLE
+		 * with >= table_level. */
+		fz_xml *close_to = NULL;
+		int implied_found = 0;
+		for (head = parser->head; head; head = head->up)
+		{
+			char *tag = fz_xml_tag(head);
+			int level;
+
+			if (tag == NULL)
+				continue;
+			tag_num = find_html_tag(tag, strlen(tag));
+			level = (html_tags[tag_num].flags>>TABLE_SHIFT) & TABLE_MASK;
+			if (level >= table_level)
+				close_to = head;
+			if (tag_num == implied || tag_num == implied2 || tag_num == implied3)
+				implied_found = 1;
+			if (tag_num == fz_xml_html_tag_table)
+				break;
+		}
+		if (close_to)
+		{
+			for (head = parser->head; head; head = head->up)
+			{
+				xml_emit_close_tag(ctx, parser);
+				if (head == close_to)
+					break;
+			}
+		}
+		if (!implied_found)
+		{
+			char *tag = html_tags[implied].tag;
+			open_implied(ctx, parser, implied);
+			xml_emit_open_tag(ctx, parser, tag, tag + strlen(tag), 0);
+		}
+	}
+	else
+	{
+		/* Non table tag. Open by implication. */
+		for (head = parser->head; head; head = head->up)
+		{
+			char *tag = fz_xml_tag(head);
+
+			if (tag == NULL)
+				continue;
+			tag_num = find_html_tag(tag, strlen(tag));
+			if (tag_num == implied || tag_num == implied2 || tag_num == implied3)
+				break;
+		}
+		if (head == NULL)
+		{
+			char *s = html_tags[implied].tag;
+			open_implied(ctx, parser, implied);
+			(void)xml_emit_open_tag(ctx, parser, s, s+strlen(s), 0);
+		}
+	}
+}
+
+/* When we meet a new tag, before we open it, there may be
+ * things we should do first... */
+static void
+pre_open_tag(fz_context *ctx, struct parser *parser, char *mark, char *p)
+{
+	fz_xml *head = parser->head;
+	int tag_num;
+
+	if (!parser->for_html)
+		return;
+
+	tag_num = find_html_tag(mark, p-mark);
+
+	if (tag_num == fz_xml_html_tag__NONE)
+		return;
+
+	if ((html_tags[tag_num].flags & CONTAINER) == 0)
+	{
+		/* We aren't a container flag. This means that we should autoclose up to
+		 * any matching tags in the same container. */
+		fz_xml *which;
+		for (which = head; which; which = which->up)
+		{
+			char *tag = fz_xml_tag(which);
+			int tag_num2 = tag ? find_html_tag(tag, strlen(tag)) : fz_xml_html_tag__NONE;
+			if (tag_num == tag_num2)
+			{
+				/* Autoclose everything from head to which inclusive */
+				while (1)
+				{
+					int done = (head == which);
+					xml_emit_close_tag(ctx, parser);
+					head = head->up;
+					if (done)
+						break;
+				}
+				break;
+			}
+			if (html_tags[tag_num2].flags & CONTAINER)
+			{
+				/* Stop searching */
+				break;
+			}
+		}
+	}
+
+	/* Now, autoopen any tags implied by this one. */
+	open_implied(ctx, parser, tag_num);
+}
+
+static char *
+skip_namespace_prefix(char *mark, char *p)
+{
+	char *ns;
+
+	for (ns = mark; ns < p - 1; ++ns)
+		if (*ns == ':')
+			mark = ns + 1;
+
+	return mark;
+}
+
 static char *xml_parse_document_imp(fz_context *ctx, struct parser *parser, char *p)
 {
 	char *mark;
 	int quote;
+	int autoclose;
+	char *q;
 
 parse_text:
 	mark = p;
 	while (*p && *p != '<') ++p;
 	if (*p == '<') {
 		/* skip trailing newline before closing tag */
-		if (p[1] == '/' && p - 1 >= mark && p[-1] == '\n')
+		if (p[1] == '/' && mark < p - 1 && p[-1] == '\n')
 			xml_emit_text(ctx, parser, mark, p - 1);
 		else if (mark < p)
 			xml_emit_text(ctx, parser, mark, p);
@@ -555,19 +932,27 @@ parse_processing_instruction:
 
 parse_closing_element:
 	while (iswhite(*p)) ++p;
+	mark = p;
 	while (isname(*p)) ++p;
+	q = p;
 	while (iswhite(*p)) ++p;
 	if (*p != '>')
 		return "syntax error in closing element";
-	xml_emit_close_tag(ctx, parser);
+	mark = skip_namespace_prefix(mark, q);
+	if (pop_to_tag(ctx, parser, mark, q))
+		xml_emit_close_tag(ctx, parser);
 	++p;
 	goto parse_text;
 
 parse_element_name:
 	mark = p;
 	while (isname(*p)) ++p;
-	xml_emit_open_tag(ctx, parser, mark, p);
+	mark = skip_namespace_prefix(mark, p);
+	pre_open_tag(ctx, parser, mark, p);
+	autoclose = xml_emit_open_tag(ctx, parser, mark, p, 0);
 	if (*p == '>') {
+		if (autoclose)
+			xml_emit_close_tag(ctx, parser);
 		++p;
 		if (*p == '\n') ++p; /* must skip linebreak immediately after an opening tag */
 		goto parse_text;
@@ -586,6 +971,8 @@ parse_attributes:
 	if (isname(*p))
 		goto parse_attribute_name;
 	if (*p == '>') {
+		if (autoclose)
+			xml_emit_close_tag(ctx, parser);
 		++p;
 		if (*p == '\n') ++p; /* must skip linebreak immediately after an opening tag */
 		goto parse_text;
@@ -669,7 +1056,7 @@ static char *convert_to_utf8(fz_context *ctx, unsigned char *s, size_t n, int *d
 
 	if (s[0] == 0xFE && s[1] == 0xFF) {
 		s += 2;
-		dst = d = fz_malloc(ctx, n * FZ_UTFMAX);
+		dst = d = Memento_label(fz_malloc(ctx, n * FZ_UTFMAX), "utf8_from_be");
 		while (s + 1 < e) {
 			c = s[0] << 8 | s[1];
 			d += fz_runetochar(d, c);
@@ -682,7 +1069,7 @@ static char *convert_to_utf8(fz_context *ctx, unsigned char *s, size_t n, int *d
 
 	if (s[0] == 0xFF && s[1] == 0xFE) {
 		s += 2;
-		dst = d = fz_malloc(ctx, n * FZ_UTFMAX);
+		dst = d = Memento_label(fz_malloc(ctx, n * FZ_UTFMAX), "utf8_from_le");
 		while (s + 1 < e) {
 			c = s[0] | s[1] << 8;
 			d += fz_runetochar(d, c);
@@ -695,7 +1082,7 @@ static char *convert_to_utf8(fz_context *ctx, unsigned char *s, size_t n, int *d
 
 	table = find_xml_encoding((char*)s);
 	if (table) {
-		dst = d = fz_malloc(ctx, n * FZ_UTFMAX);
+		dst = d = Memento_label(fz_malloc(ctx, n * FZ_UTFMAX), "utf8");
 		while (*s) {
 			c = table[*s++];
 			d += fz_runetochar(d, c);
@@ -713,23 +1100,19 @@ static char *convert_to_utf8(fz_context *ctx, unsigned char *s, size_t n, int *d
 	return (char*)s;
 }
 
-/*
-	Parse the contents of buffer into a tree of xml nodes.
-
-	preserve_white: whether to keep or delete all-whitespace nodes.
-*/
 fz_xml_doc *
-fz_parse_xml(fz_context *ctx, fz_buffer *buf, int preserve_white)
+fz_parse_xml(fz_context *ctx, fz_buffer *buf, int preserve_white, int for_html)
 {
 	struct parser parser;
 	fz_xml_doc *xml = NULL;
 	fz_xml root, *node;
 	char *p = NULL;
 	char *error;
-	int dofree;
+	int dofree = 0;
 	unsigned char *s;
 	size_t n;
 
+	fz_var(dofree);
 	fz_var(p);
 
 	/* ensure we are zero-terminated */
@@ -740,7 +1123,11 @@ fz_parse_xml(fz_context *ctx, fz_buffer *buf, int preserve_white)
 	parser.pool = fz_new_pool(ctx);
 	parser.head = &root;
 	parser.preserve_white = preserve_white;
+	parser.for_html = for_html;
 	parser.depth = 0;
+#ifdef FZ_XML_SEQ
+	parser.seq = 0;
+#endif
 
 	fz_try(ctx)
 	{
@@ -750,6 +1137,9 @@ fz_parse_xml(fz_context *ctx, fz_buffer *buf, int preserve_white)
 		if (error)
 			fz_throw(ctx, FZ_ERROR_GENERIC, "%s", error);
 
+		for (node = parser.head; node; node = node->up)
+			node->next = NULL;
+
 		for (node = root.down; node; node = node->next)
 			node->up = NULL;
 
