@@ -322,7 +322,7 @@
 "if (match) {\n"
 "var prefix = string.substring(0, match.index);\n"
 "var suffix = string.substring(match.index + match[0].length);\n"
-"return [ prefix + suffix, match[0] ]\n"
+"return [ prefix + suffix, match[0] ];\n"
 "}\n"
 "return null;\n"
 "}\n"
@@ -342,7 +342,7 @@
 "var names = ['jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec'];\n"
 "var month = date.match(/Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec/i);\n"
 "if (month)\n"
-"return names.indexOf(month[0].toLowerCase());\n"
+"return names.indexOf(month[0].toLowerCase()) + 1;\n"
 "return null;\n"
 "}\n"
 "\n"
