@@ -3,7 +3,7 @@
 
 #include <string.h>
 
-typedef struct globals_s
+typedef struct
 {
 	pdf_document *doc;
 	fz_context *ctx;
@@ -313,7 +313,6 @@ static void retainpages(fz_context *ctx, globals *glo, int argc, char **argv)
 	pdf_drop_obj(ctx, root);
 }
 
-/* Read infile, and write selected pages to outfile with the given options. */
 void pdf_clean_file(fz_context *ctx, char *infile, char *outfile, char *password, pdf_write_options *opts, char *argv[], int argc)
 {
 	globals glo = { 0 };
