@@ -3,23 +3,6 @@
 
 #include <string.h>
 
-size_t
-pdf_cmap_size(fz_context *ctx, pdf_cmap *cmap)
-{
-	if (cmap == NULL)
-		return 0;
-	if (cmap->storable.refs < 0)
-		return 0;
-
-	return pdf_cmap_size(ctx, cmap->usecmap) +
-		cmap->rcap * sizeof *cmap->ranges +
-		cmap->xcap * sizeof *cmap->xranges +
-		cmap->mcap * sizeof *cmap->mranges;
-}
-
-/*
- * Load CMap stream in PDF file
- */
 pdf_cmap *
 pdf_load_embedded_cmap(fz_context *ctx, pdf_document *doc, pdf_obj *stmobj)
 {
@@ -79,9 +62,6 @@ pdf_load_embedded_cmap(fz_context *ctx, pdf_document *doc, pdf_obj *stmobj)
 	return cmap;
 }
 
-/*
- * Create an Identity-* CMap (for both 1 and 2-byte encodings)
- */
 pdf_cmap *
 pdf_new_identity_cmap(fz_context *ctx, int wmode, int bytes)
 {
@@ -106,10 +86,6 @@ pdf_new_identity_cmap(fz_context *ctx, int wmode, int bytes)
 	return cmap;
 }
 
-/*
- * Load built-in CMap resource.
- */
-
 #ifdef NO_CJK
 
 pdf_cmap *
@@ -289,9 +265,6 @@ pdf_load_builtin_cmap(fz_context *ctx, const char *name)
 
 #endif
 
-/*
- * Load predefined CMap from system.
- */
 pdf_cmap *
 pdf_load_system_cmap(fz_context *ctx, const char *cmap_name)
 {
