@@ -119,42 +119,77 @@ static void update_field_value(fz_context *ctx, pdf_document *doc, pdf_obj *obj,
 	pdf_field_mark_dirty(ctx, obj);
 }
 
-static pdf_obj *find_field(fz_context *ctx, pdf_obj *dict, const char *name, int len)
+static pdf_obj *
+lookup_field_sub(fz_context *ctx, pdf_obj *dict, const char *str)
 {
-	int i, n = pdf_array_len(ctx, dict);
-	for (i = 0; i < n; i++)
+	pdf_obj *kids;
+	pdf_obj *name;
+
+	name = pdf_dict_get(ctx, dict, PDF_NAME(T));
+
+	/* If we have a name, check it matches. If it matches, consume that
+	 * portion of str. If not, exit. */
+	if (name)
 	{
-		pdf_obj *field = pdf_array_get(ctx, dict, i);
-		const char *part = pdf_dict_get_text_string(ctx, field, PDF_NAME(T));
-		if (strlen(part) == (size_t)len && !memcmp(part, name, len))
-			return field;
+		const char *match = pdf_to_text_string(ctx, name);
+		const char *e = str;
+		size_t len;
+		while (*e && *e != '.')
+			e++;
+		len = e-str;
+		if (strncmp(str, match, len) != 0 || (match[len] != 0 && match[len] != '.'))
+			/* name doesn't match. */
+			return NULL;
+		str = e;
+		if (*str == '.')
+			str++;
 	}
+
+	/* If there is a kids array, walk those looking for the appropriate one. */
+	kids = pdf_dict_get(ctx, dict, PDF_NAME(Kids));
+	if (kids)
+		return pdf_lookup_field(ctx, kids, str);
+
+	/* No Kids, so we're a terminal node. We accept it as the match if we've
+	 * exhausted the match string. */
+	if (*str == 0)
+		return dict;
+
 	return NULL;
 }
 
-pdf_obj *pdf_lookup_field(fz_context *ctx, pdf_obj *form, const char *name)
+pdf_obj *
+pdf_lookup_field(fz_context *ctx, pdf_obj *arr, const char *str)
 {
-	const char *dot;
-	const char *namep;
-	pdf_obj *dict = NULL;
-	int len;
+	int len = pdf_array_len(ctx, arr);
+	int i;
+	pdf_obj *found = NULL;
+	pdf_obj *k = NULL;
 
-	/* Process the fully qualified field name which has
-	* the partial names delimited by '.'. Pretend there
-	* was a preceding '.' to simplify the loop */
-	dot = name - 1;
+	fz_var(k);
 
-	while (dot && form)
+	fz_try(ctx)
 	{
-		namep = dot + 1;
-		dot = strchr(namep, '.');
-		len = dot ? dot - namep : (int)strlen(namep);
-		dict = find_field(ctx, form, namep, len);
-		if (dot)
-			form = pdf_dict_get(ctx, dict, PDF_NAME(Kids));
-	}
+		for (i = 0; found == NULL && i < len; i++)
+		{
+			k = pdf_array_get(ctx, arr, i);
 
-	return dict;
+			if (!pdf_mark_obj(ctx, k))
+			{
+				found = lookup_field_sub(ctx, k, str);
+				pdf_unmark_obj(ctx, k);
+				k = NULL;
+			}
+		}
+	}
+	fz_always(ctx)
+	{
+		pdf_unmark_obj(ctx, k);
+	}
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+
+	return found;
 }
 
 static void reset_form_field(fz_context *ctx, pdf_document *doc, pdf_obj *field)
@@ -440,19 +475,11 @@ static void toggle_check_box(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 	doc->recalculate = 1;
 }
 
-/*
-	Determine whether changes have been made since the
-	document was opened or last saved.
-*/
 int pdf_has_unsaved_changes(fz_context *ctx, pdf_document *doc)
 {
 	return doc->dirty;
 }
 
-/*
-	Toggle the state of a specified annotation. Applies only to check-box
-	and radio-button widgets.
-*/
 int pdf_toggle_widget(fz_context *ctx, pdf_widget *widget)
 {
 	switch (pdf_widget_type(ctx, widget))
@@ -467,17 +494,6 @@ int pdf_toggle_widget(fz_context *ctx, pdf_widget *widget)
 	return 0;
 }
 
-/*
-	Recalculate form fields if necessary.
-
-	Loop through all annotations on the page and update them. Return true
-	if any of them were changed (by either event or javascript actions, or
-	by annotation editing) and need re-rendering.
-
-	If you need more granularity, loop through the annotations and call
-	pdf_update_annot for each one to detect changes on a per-annotation
-	basis.
-*/
 int
 pdf_update_page(fz_context *ctx, pdf_page *page)
 {
@@ -676,39 +692,52 @@ int pdf_field_display(fz_context *ctx, pdf_obj *field)
 static char *get_field_name(fz_context *ctx, pdf_obj *field, int spare)
 {
 	char *res = NULL;
-	pdf_obj *parent = pdf_dict_get(ctx, field, PDF_NAME(Parent));
-	const char *lname = pdf_dict_get_text_string(ctx, field, PDF_NAME(T));
-	int llen = (int)strlen(lname);
+	pdf_obj *parent;
+	const char *lname;
+	int llen;
 
-	/*
-	 * If we found a name at this point in the field hierarchy
-	 * then we'll need extra space for it and a dot
-	 */
-	if (llen)
-		spare += llen+1;
-
-	if (parent)
+	fz_try(ctx)
 	{
-		res = get_field_name(ctx, parent, spare);
-	}
-	else
-	{
-		res = fz_malloc(ctx, spare+1);
-		res[0] = 0;
-	}
+		if (pdf_mark_obj(ctx, field))
+			fz_throw(ctx, FZ_ERROR_GENERIC, "Cycle in field parents");
 
-	if (llen)
-	{
-		if (res[0])
-			strcat(res, ".");
+		parent = pdf_dict_get(ctx, field, PDF_NAME(Parent));
+		lname = pdf_dict_get_text_string(ctx, field, PDF_NAME(T));
+		llen = (int)strlen(lname);
 
-		strcat(res, lname);
+		/*
+		 * If we found a name at this point in the field hierarchy
+		 * then we'll need extra space for it and a dot
+		 */
+		if (llen)
+			spare += llen+1;
+
+		if (parent)
+		{
+			res = get_field_name(ctx, parent, spare);
+		}
+		else
+		{
+			res = Memento_label(fz_malloc(ctx, spare+1), "form_field_name");
+			res[0] = 0;
+		}
+
+		if (llen)
+		{
+			if (res[0])
+				strcat(res, ".");
+
+			strcat(res, lname);
+		}
 	}
+	fz_always(ctx)
+		pdf_unmark_obj(ctx, field);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 
 	return res;
 }
 
-/* Note: This function allocates a string for the return value that you must free manually. */
 char *pdf_field_name(fz_context *ctx, pdf_obj *field)
 {
 	return get_field_name(ctx, field, 0);
@@ -840,22 +869,13 @@ pdf_update_widget(fz_context *ctx, pdf_widget *widget)
 	return pdf_update_annot(ctx, widget);
 }
 
-/*
-	get the maximum number of
-	characters permitted in a text widget
-*/
-int pdf_text_widget_max_len(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
+int pdf_text_widget_max_len(fz_context *ctx, pdf_widget *tw)
 {
 	pdf_annot *annot = (pdf_annot *)tw;
-
 	return pdf_to_int(ctx, pdf_dict_get_inheritable(ctx, annot->obj, PDF_NAME(MaxLen)));
 }
 
-/*
-	get the type of content
-	required by a text widget
-*/
-int pdf_text_widget_format(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
+int pdf_text_widget_format(fz_context *ctx, pdf_widget *tw)
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	int type = PDF_WIDGET_TX_FORMAT_NONE;
@@ -877,11 +897,6 @@ int pdf_text_widget_format(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
 	return type;
 }
 
-/*
-	Update the text of a text widget.
-	The text is first validated by the Field/Keystroke event processing and accepted only if it passes.
-	The function returns whether validation passed.
-*/
 int pdf_set_text_field_value(fz_context *ctx, pdf_widget *widget, const char *new_value)
 {
 	pdf_document *doc = widget->page->doc;
@@ -943,15 +958,7 @@ int pdf_set_choice_field_value(fz_context *ctx, pdf_widget *widget, const char *
 	return pdf_set_text_field_value(ctx, widget, new_value);
 }
 
-/*
-	get the list of options for a list
-	box or combo box. Returns the number of options and fills in their
-	names within the supplied array. Should first be called with a
-	NULL array to find out how big the array should be.  If exportval
-	is true, then the export values will be returned and not the list
-	values if there are export values present.
-*/
-int pdf_choice_widget_options(fz_context *ctx, pdf_document *doc, pdf_widget *tw, int exportval, const char *opts[])
+int pdf_choice_widget_options(fz_context *ctx, pdf_widget *tw, int exportval, const char *opts[])
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	pdf_obj *optarr;
@@ -978,7 +985,23 @@ int pdf_choice_widget_options(fz_context *ctx, pdf_document *doc, pdf_widget *tw
 	return n;
 }
 
-int pdf_choice_widget_is_multiselect(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
+int pdf_choice_field_option_count(fz_context *ctx, pdf_obj *field)
+{
+	pdf_obj *opt = pdf_dict_get_inheritable(ctx, field, PDF_NAME(Opt));
+	return pdf_array_len(ctx, opt);
+}
+
+const char *pdf_choice_field_option(fz_context *ctx, pdf_obj *field, int export, int i)
+{
+	pdf_obj *opt = pdf_dict_get_inheritable(ctx, field, PDF_NAME(Opt));
+	pdf_obj *ent = pdf_array_get(ctx, opt, i);
+	if (pdf_array_len(ctx, ent) == 2)
+		return pdf_array_get_text_string(ctx, ent, export ? 0 : 1);
+	else
+		return pdf_to_text_string(ctx, ent);
+}
+
+int pdf_choice_widget_is_multiselect(fz_context *ctx, pdf_widget *tw)
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 
@@ -993,14 +1016,7 @@ int pdf_choice_widget_is_multiselect(fz_context *ctx, pdf_document *doc, pdf_wid
 	}
 }
 
-/*
-	get the value of a choice widget.
-	Returns the number of options currently selected and fills in
-	the supplied array with their strings. Should first be called
-	with NULL as the array to find out how big the array need to
-	be. The filled in elements should not be freed by the caller.
-*/
-int pdf_choice_widget_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw, const char *opts[])
+int pdf_choice_widget_value(fz_context *ctx, pdf_widget *tw, const char *opts[])
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	pdf_obj *optarr;
@@ -1034,12 +1050,7 @@ int pdf_choice_widget_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw,
 	}
 }
 
-/*
-	set the value of a choice widget. The
-	caller should pass the number of options selected and an
-	array of their names
-*/
-void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw, int n, const char *opts[])
+void pdf_choice_widget_set_value(fz_context *ctx, pdf_widget *tw, int n, const char *opts[])
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	pdf_obj *optarr = NULL, *opt;
@@ -1053,7 +1064,7 @@ void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget
 	{
 		if (n != 1)
 		{
-			optarr = pdf_new_array(ctx, doc, n);
+			optarr = pdf_new_array(ctx, annot->page->doc, n);
 
 			for (i = 0; i < n; i++)
 			{
@@ -1073,8 +1084,8 @@ void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget
 		pdf_dict_del(ctx, annot->obj, PDF_NAME(I));
 
 		pdf_field_mark_dirty(ctx, annot->obj);
-		if (pdf_field_dirties_document(ctx, doc, annot->obj))
-			doc->dirty = 1;
+		if (pdf_field_dirties_document(ctx, annot->page->doc, annot->obj))
+			annot->page->doc->dirty = 1;
 	}
 	fz_catch(ctx)
 	{
@@ -1092,7 +1103,7 @@ int pdf_signature_byte_range(fz_context *ctx, pdf_document *doc, pdf_obj *signat
 	{
 		for (i = 0; i < n; i++)
 		{
-			int offset = pdf_array_get_int(ctx, br, 2*i);
+			int64_t offset = pdf_array_get_int(ctx, br, 2*i);
 			int length = pdf_array_get_int(ctx, br, 2*i+1);
 
 			if (offset < 0 || offset > doc->file_size)
@@ -1145,7 +1156,7 @@ static void validate_certificate_data(fz_context *ctx, pdf_document *doc, fz_ran
 
 		if (c != EOF)
 			fz_throw(ctx, FZ_ERROR_GENERIC, "signature certificate data contains invalid character");
-		if (fz_tell(ctx, stm) != hole->length)
+		if ((size_t)fz_tell(ctx, stm) != hole->length)
 			fz_throw(ctx, FZ_ERROR_GENERIC, "premature end of signature certificate data");
 	}
 	fz_always(ctx)
@@ -1196,9 +1207,6 @@ static void validate_byte_ranges(fz_context *ctx, pdf_document *doc, fz_range *u
 		fz_rethrow(ctx);
 }
 
-/*
-	retrieve an fz_stream to read the bytes hashed for the signature
-*/
 fz_stream *pdf_signature_hash_bytes(fz_context *ctx, pdf_document *doc, pdf_obj *signature)
 {
 	fz_range *byte_range = NULL;
@@ -1274,51 +1282,308 @@ int pdf_signature_incremental_change_since_signing(fz_context *ctx, pdf_document
 
 int pdf_signature_is_signed(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 {
+	pdf_obj *v;
+	pdf_obj* vtype;
+
 	if (pdf_dict_get_inheritable(ctx, field, PDF_NAME(FT)) != PDF_NAME(Sig))
 		return 0;
-	return pdf_dict_get_inheritable(ctx, field, PDF_NAME(V)) != NULL;
+	/* Signatures can only be signed if the value is a dictionary,
+	 * and if the value has a Type, it should be Sig. */
+	v = pdf_dict_get_inheritable(ctx, field, PDF_NAME(V));
+	vtype = pdf_dict_get(ctx, v, PDF_NAME(Type));
+	return pdf_is_dict(ctx, v) && (vtype ? pdf_name_eq(ctx, vtype, PDF_NAME(Sig)) : 1);
 }
 
-/* NOTE: contents is allocated and must be freed by the caller! */
-int pdf_signature_contents(fz_context *ctx, pdf_document *doc, pdf_obj *signature, char **contents)
+int pdf_widget_is_signed(fz_context *ctx, pdf_widget *widget)
 {
-	pdf_obj *v_ref = pdf_dict_get(ctx, signature, PDF_NAME(V));
+	if (widget == NULL)
+		return 0;
+	return pdf_signature_is_signed(ctx, widget->page->doc, widget->obj);
+}
+
+size_t pdf_signature_contents(fz_context *ctx, pdf_document *doc, pdf_obj *signature, char **contents)
+{
+	pdf_obj *v_ref = pdf_dict_get_inheritable(ctx, signature, PDF_NAME(V));
 	pdf_obj *v_obj = pdf_load_unencrypted_object(ctx, doc, pdf_to_num(ctx, v_ref));
-	int len;
+	char *copy = NULL;
+	size_t len;
+
+	fz_var(copy);
 	fz_try(ctx)
 	{
 		pdf_obj *c = pdf_dict_get(ctx, v_obj, PDF_NAME(Contents));
+		char *s;
+
+		s = pdf_to_str_buf(ctx, c);
 		len = pdf_to_str_len(ctx, c);
+
 		if (contents)
 		{
-			*contents = fz_malloc(ctx, len);
-			memcpy(*contents, pdf_to_str_buf(ctx, c), len);
+			copy = Memento_label(fz_malloc(ctx, len), "sig_contents");
+			memcpy(copy, s, len);
 		}
 	}
 	fz_always(ctx)
 		pdf_drop_obj(ctx, v_obj);
 	fz_catch(ctx)
+	{
+		fz_free(ctx, copy);
 		fz_rethrow(ctx);
+	}
+
+	if (contents)
+		*contents = copy;
 	return len;
 }
 
-void pdf_signature_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field, pdf_pkcs7_signer *signer)
+static fz_xml_doc *
+pdf_parse_xml(fz_context *ctx, pdf_obj *obj)
+{
+	fz_buffer *buf = pdf_load_stream(ctx, obj);
+	fz_xml_doc *xml = NULL;
+
+	fz_try(ctx)
+		xml = fz_parse_xml(ctx, buf, 0, 0);
+	fz_always(ctx)
+		fz_drop_buffer(ctx, buf);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+
+	return xml;
+}
+
+static int load_xfa(fz_context *ctx, pdf_document *doc)
+{
+	pdf_obj *xfa;
+	int i, len;
+
+	if (doc->xfa.count)
+		return 1; /* Already loaded, and present. */
+
+	xfa = pdf_dict_getp(ctx, pdf_trailer(ctx, doc), "Root/AcroForm/XFA");
+	if (!pdf_is_array(ctx, xfa))
+		return 0; /* No XFA */
+
+	len = (pdf_array_len(ctx, xfa)+1)>>1;
+	doc->xfa.entries = fz_calloc(ctx, len, sizeof(pdf_xfa_entry));
+	doc->xfa.count = len;
+
+	for(i = 0; i < len; i++)
+	{
+		doc->xfa.entries[i].key = fz_strdup(ctx, pdf_to_text_string(ctx, pdf_array_get(ctx, xfa, i*2)));
+		doc->xfa.entries[i].value = pdf_parse_xml(ctx, pdf_array_get(ctx, xfa, i*2+1));
+	}
+
+	return len > 0;
+}
+
+static fz_xml *
+get_xfa_root(fz_context *ctx, pdf_document *doc, const char *str)
+{
+	int i;
+
+	if (load_xfa(ctx, doc) == 0)
+		return NULL;
+
+	for (i = 0; i < doc->xfa.count; i++)
+		if (strcmp(doc->xfa.entries[i].key, str) == 0)
+			return fz_xml_root(doc->xfa.entries[i].value);
+
+	return NULL;
+}
+
+static int
+find_name_component(char **np, char **sp, char **ep)
+{
+	char *n = *np;
+	char *s, *e;
+	int idx = 0;
+
+	if (*n == '.')
+		n++;
+
+	/* Find the next name we are looking for. */
+	s = e = n;
+	while (*e && *e != '[' && *e != '.')
+		e++;
+
+	/* So the next name is s..e */
+	n = e;
+	if (*n == '[')
+	{
+		n++;
+		while (*n >= '0' && *n <= '9')
+			idx = idx*10 + *n++ - '0';
+		while (*n && *n != ']')
+			n++;
+		if (*n == ']')
+			n++;
+	}
+	*np = n;
+	*sp = s;
+	*ep = e;
+
+	return idx;
+}
+
+static pdf_obj *
+annot_from_name(fz_context *ctx, pdf_document *doc, const char *str)
+{
+	pdf_obj *fields = pdf_dict_getp(ctx, pdf_trailer(ctx, doc), "Root/AcroForm/Fields");
+
+	if (strncmp(str, "xfa[0].", 7) == 0)
+		str += 7;
+	if (strncmp(str, "template[0].", 12) == 0)
+		str += 12;
+
+	return pdf_lookup_field(ctx, fields, str);
+}
+
+static pdf_obj *
+get_locked_fields_from_xfa(fz_context *ctx, pdf_document *doc, pdf_obj *field)
+{
+	char *name = pdf_field_name(ctx, field);
+	char *n = name;
+	const char *use;
+	fz_xml *root, *node;
+
+	if (name == NULL)
+		return NULL;
+
+	root = get_xfa_root(ctx, doc, "template");
+	node = fz_xml_find(root, "template");
+
+	do
+	{
+		char c, *s, *e;
+		int idx = 0;
+		char *key;
+
+		idx = find_name_component(&n, &s, &e);
+		/* We want the idx'th occurrence of s..e */
+
+		/* Hacky */
+		c = *e;
+		*e = 0;
+		key = *n ? "subform" : "field";
+		node = fz_xml_find_down_match(node, key, "name", s);
+		while (node && idx > 0)
+		{
+			node = fz_xml_find_next_match(node, key, "name", s);
+			idx--;
+		}
+		*e = c;
+	}
+	while (node && *n == '.');
+
+	fz_free(ctx, name);
+	if (node == NULL)
+		return NULL;
+
+	node = fz_xml_find_down(node, "ui");
+	node = fz_xml_find_down(node, "signature");
+	node = fz_xml_find_down(node, "manifest");
+
+	use = fz_xml_att(node, "use");
+	if (*use == '#')
+		use++;
+
+	/* Now look for a variables entry in a subform that defines this. */
+	while (node)
+	{
+		fz_xml *variables, *manifest, *ref;
+		pdf_obj *arr;
+
+		/* Find the enclosing subform */
+		do {
+			node = fz_xml_up(node);
+		} while (node && strcmp(fz_xml_tag(node), "subform"));
+
+		/* Look for a variables within that. */
+		variables = fz_xml_find_down(node, "variables");
+		if (variables == NULL)
+			continue;
+
+		manifest = fz_xml_find_down_match(variables, "manifest", "id", use);
+		if (manifest == NULL)
+			continue;
+
+		arr = pdf_new_array(ctx, doc, 16);
+		fz_try(ctx)
+		{
+			ref = fz_xml_find_down(manifest, "ref");
+			while (ref)
+			{
+				const char *s = fz_xml_text(fz_xml_down(ref));
+				pdf_array_push(ctx, arr, annot_from_name(ctx, doc, s));
+				ref = fz_xml_find_next(ref, "ref");
+			}
+		}
+		fz_catch(ctx)
+		{
+			pdf_drop_obj(ctx, arr);
+			fz_rethrow(ctx);
+		}
+		return arr;
+	}
+
+	return NULL;
+}
+
+static void
+lock_field(fz_context *ctx, pdf_obj *f)
+{
+	int ff = pdf_to_int(ctx, pdf_dict_get_inheritable(ctx, f, PDF_NAME(Ff)));
+
+	if ((ff & PDF_FIELD_IS_READ_ONLY) ||
+		!pdf_name_eq(ctx, pdf_dict_get(ctx, f, PDF_NAME(Type)), PDF_NAME(Annot)) ||
+		!pdf_name_eq(ctx, pdf_dict_get(ctx, f, PDF_NAME(Subtype)), PDF_NAME(Widget)))
+		return;
+
+	pdf_dict_put(ctx, f, PDF_NAME(Ff), pdf_new_int(ctx, ff | PDF_FIELD_IS_READ_ONLY));
+}
+
+static void
+lock_xfa_locked_fields(fz_context *ctx, pdf_obj *a)
+{
+	int i;
+	int len = pdf_array_len(ctx, a);
+
+	for (i = 0; i < len; i++)
+	{
+		lock_field(ctx, pdf_array_get(ctx, a, i));
+	}
+}
+
+
+void pdf_signature_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field, pdf_pkcs7_signer *signer, int64_t stime)
 {
 	pdf_obj *v = NULL;
+	pdf_obj *o = NULL;
+	pdf_obj *r = NULL;
+	pdf_obj *t = NULL;
+	pdf_obj *a = NULL;
+	pdf_obj *b = NULL;
+	pdf_obj *l = NULL;
 	pdf_obj *indv;
 	int vnum;
-	pdf_obj *byte_range;
-	pdf_obj *contents;
-	int max_digest_size;
+	size_t max_digest_size;
 	char *buf = NULL;
+	char date_string[40];
 
 	vnum = pdf_create_object(ctx, doc);
 	indv = pdf_new_indirect(ctx, doc, vnum, 0);
 	pdf_dict_put_drop(ctx, field, PDF_NAME(V), indv);
 
-	max_digest_size = signer->max_digest_size(signer);
+	max_digest_size = signer->max_digest_size(ctx, signer);
 
 	fz_var(v);
+	fz_var(o);
+	fz_var(r);
+	fz_var(t);
+	fz_var(a);
+	fz_var(b);
+	fz_var(l);
 	fz_var(buf);
 	fz_try(ctx)
 	{
@@ -1327,15 +1592,66 @@ void pdf_signature_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field,
 
 		buf = fz_calloc(ctx, max_digest_size, 1);
 
-		byte_range = pdf_new_array(ctx, doc, 4);
-		pdf_dict_put_drop(ctx, v, PDF_NAME(ByteRange), byte_range);
-
-		contents = pdf_new_string(ctx, buf, max_digest_size);
-		pdf_dict_put_drop(ctx, v, PDF_NAME(Contents), contents);
-
-		pdf_dict_put(ctx, v, PDF_NAME(Type), PDF_NAME(Sig));
+		/* Ensure that the /Filter entry is the first entry in the
+		   dictionary after the digest contents since we look for
+		   this tag when completing signatures in pdf-write.c in order
+		   to generate the correct byte range. */
+		pdf_dict_put_array(ctx, v, PDF_NAME(ByteRange), 4);
+		pdf_dict_put_string(ctx, v, PDF_NAME(Contents), buf, max_digest_size);
 		pdf_dict_put(ctx, v, PDF_NAME(Filter), PDF_NAME(Adobe_PPKLite));
 		pdf_dict_put(ctx, v, PDF_NAME(SubFilter), PDF_NAME(adbe_pkcs7_detached));
+		pdf_dict_put(ctx, v, PDF_NAME(Type), PDF_NAME(Sig));
+		pdf_format_date(ctx, date_string, sizeof date_string, stime);
+		pdf_dict_put_text_string(ctx, v, PDF_NAME(M), date_string);
+
+		o = pdf_new_array(ctx, doc, 1);
+		pdf_dict_put(ctx, v, PDF_NAME(Reference), o);
+		r = pdf_new_dict(ctx, doc, 4);
+		pdf_array_put(ctx, o, 0, r);
+		pdf_dict_put(ctx, r, PDF_NAME(Data), pdf_dict_get(ctx, pdf_trailer(ctx, doc), PDF_NAME(Root)));
+		pdf_dict_put(ctx, r, PDF_NAME(TransformMethod), PDF_NAME(FieldMDP));
+		pdf_dict_put(ctx, r, PDF_NAME(Type), PDF_NAME(SigRef));
+		t = pdf_new_dict(ctx, doc, 5);
+		pdf_dict_put(ctx, r, PDF_NAME(TransformParams), t);
+
+		l = pdf_dict_getp(ctx, field, "Lock/Action");
+		if (l)
+		{
+			a = pdf_dict_getp(ctx, field, "Lock/Fields");
+		}
+		else
+		{
+			/* Lock action wasn't specified so we need to encode an Include.
+			 * Before we just use an empty array, check in the XFA for locking
+			 * details. */
+			a = get_locked_fields_from_xfa(ctx, doc, field);
+			if (a)
+				lock_xfa_locked_fields(ctx, a);
+
+			/* If we don't get a result from the XFA, just encode an empty array
+			 * (leave a == NULL), even if Lock/Fields exists because we don't really
+			 * know what to do with the information if the action isn't defined. */
+			l = PDF_NAME(Include);
+		}
+
+		pdf_dict_put(ctx, t, PDF_NAME(Action), l);
+
+		if (pdf_name_eq(ctx, l, PDF_NAME(Include)) || pdf_name_eq(ctx, l, PDF_NAME(Exclude)))
+		{
+			/* For action Include and Exclude, we need to encode a Fields array */
+			if (!a)
+			{
+				/* If one wasn't defined or we chose to ignore it because no action
+				 * was defined then use an empty one. */
+				b = pdf_new_array(ctx, doc, 0);
+				a = b;
+			}
+
+			pdf_dict_put_drop(ctx, t, PDF_NAME(Fields), pdf_copy_array(ctx, a));
+		}
+
+		pdf_dict_put(ctx, t, PDF_NAME(Type), PDF_NAME(TransformParams));
+		pdf_dict_put(ctx, t, PDF_NAME(V), PDF_NAME(1_2));
 
 		/* Record details within the document structure so that contents
 		* and byte_range can be updated with their correct values at
@@ -1345,6 +1661,10 @@ void pdf_signature_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field,
 	fz_always(ctx)
 	{
 		pdf_drop_obj(ctx, v);
+		pdf_drop_obj(ctx, o);
+		pdf_drop_obj(ctx, r);
+		pdf_drop_obj(ctx, t);
+		pdf_drop_obj(ctx, b);
 		fz_free(ctx, buf);
 	}
 	fz_catch(ctx)
@@ -1353,14 +1673,6 @@ void pdf_signature_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field,
 	}
 }
 
-/*
-	Update internal state appropriate for editing this field. When editing
-	is true, updating the text of the text widget will not have any
-	side-effects such as changing other widgets or running javascript.
-	This state is intended for the period when a text widget is having
-	characters typed into it. The state should be reverted at the end of
-	the edit sequence and the text newly updated.
-*/
 void pdf_set_widget_editing_state(fz_context *ctx, pdf_widget *widget, int editing)
 {
 	widget->ignore_trigger_events = editing;
@@ -1610,3 +1922,27 @@ void pdf_field_event_calculate(fz_context *ctx, pdf_document *doc, pdf_obj *fiel
 		}
 	}
 }
+
+static void
+count_sigs(fz_context *ctx, pdf_obj *field, void *arg, pdf_obj **ft)
+{
+	int *n = (int *)arg;
+
+	if (!pdf_name_eq(ctx, pdf_dict_get(ctx, field, PDF_NAME(Type)), PDF_NAME(Annot)) ||
+		!pdf_name_eq(ctx, pdf_dict_get(ctx, field, PDF_NAME(Subtype)), PDF_NAME(Widget)) ||
+		!pdf_name_eq(ctx, *ft, PDF_NAME(Sig)))
+		return;
+
+	(*n)++;
+}
+
+static pdf_obj *ft_name[2] = { PDF_NAME(FT), NULL };
+
+int pdf_count_signatures(fz_context *ctx, pdf_document *doc)
+{
+	int n = 0;
+	pdf_obj *ft = NULL;
+	pdf_obj *form_fields = pdf_dict_getp(ctx, pdf_trailer(ctx, doc), "Root/AcroForm/Fields");
+	pdf_walk_tree(ctx, form_fields, PDF_NAME(Kids), count_sigs, NULL, &n, ft_name, &ft);
+	return n;
+}
