@@ -57,7 +57,7 @@ typedef struct
 	unsigned int locked : 1;
 } pdf_ocg_ui;
 
-struct pdf_ocg_descriptor_s
+struct pdf_ocg_descriptor
 {
 	int current;
 	int num_configs;
@@ -72,12 +72,6 @@ struct pdf_ocg_descriptor_s
 	pdf_ocg_ui *ui;
 };
 
-/*
-	Get the number of layer
-	configurations defined in this document.
-
-	doc: The document in question.
-*/
 int
 pdf_count_layer_configs(fz_context *ctx, pdf_document *doc)
 {
@@ -203,16 +197,6 @@ load_ui(fz_context *ctx, pdf_ocg_descriptor *desc, pdf_obj *ocprops, pdf_obj *oc
 	}
 }
 
-/*
-	Set the current configuration.
-	This updates the visibility of the optional content groups
-	within the document.
-
-	doc: The document in question.
-
-	config_num: A value in the 0..n-1 range, where n is the
-	value returned from pdf_count_layer_configs.
-*/
 void
 pdf_select_layer_config(fz_context *ctx, pdf_document *doc, int config)
 {
@@ -300,19 +284,6 @@ pdf_select_layer_config(fz_context *ctx, pdf_document *doc, int config)
 	load_ui(ctx, desc, obj, cobj);
 }
 
-/*
-	Fetch the name (and
-	optionally creator) of the given layer config.
-
-	doc: The document in question.
-
-	config_num: A value in the 0..n-1 range, where n is the
-	value returned from pdf_count_layer_configs.
-
-	info: Pointer to structure to fill in. Pointers within
-	this structure may be set to NULL if no information is
-	available.
-*/
 void
 pdf_layer_config_info(fz_context *ctx, pdf_document *doc, int config_num, pdf_layer_config *info)
 {
@@ -398,12 +369,6 @@ clear_radio_group(fz_context *ctx, pdf_document *doc, pdf_obj *ocg)
 	}
 }
 
-/*
-	Returns the number of entries in the
-	'UI' for this layer configuration.
-
-	doc: The document in question.
-*/
 int pdf_count_layer_config_ui(fz_context *ctx, pdf_document *doc)
 {
 	if (doc == NULL || doc->ocg == NULL)
@@ -412,18 +377,6 @@ int pdf_count_layer_config_ui(fz_context *ctx, pdf_document *doc)
 	return doc->ocg->num_ui_entries;
 }
 
-/*
-	Select a checkbox/radiobox
-	within the 'UI' for this layer configuration.
-
-	Selecting a UI entry that is a radiobox may disable
-	other UI entries.
-
-	doc: The document in question.
-
-	ui: A value in the 0..m-1 range, where m is the value
-	returned by pdf_count_layer_config_ui.
-*/
 void pdf_select_layer_config_ui(fz_context *ctx, pdf_document *doc, int ui)
 {
 	pdf_ocg_ui *entry;
@@ -447,18 +400,6 @@ void pdf_select_layer_config_ui(fz_context *ctx, pdf_document *doc, int ui)
 	doc->ocg->ocgs[entry->ocg].state = 1;
 }
 
-/*
-	Toggle a checkbox/radiobox
-	within the 'UI' for this layer configuration.
-
-	Toggling a UI entry that is a radiobox may disable
-	other UI entries.
-
-	doc: The document in question.
-
-	ui: A value in the 0..m-1 range, where m is the value
-	returned by pdf_count_layer_config_ui.
-*/
 void pdf_toggle_layer_config_ui(fz_context *ctx, pdf_document *doc, int ui)
 {
 	pdf_ocg_ui *entry;
@@ -485,15 +426,6 @@ void pdf_toggle_layer_config_ui(fz_context *ctx, pdf_document *doc, int ui)
 	doc->ocg->ocgs[entry->ocg].state = !selected;
 }
 
-/*
-	Select a checkbox/radiobox
-	within the 'UI' for this layer configuration.
-
-	doc: The document in question.
-
-	ui: A value in the 0..m-1 range, where m is the value
-	returned by pdf_count_layer_config_ui.
-*/
 void pdf_deselect_layer_config_ui(fz_context *ctx, pdf_document *doc, int ui)
 {
 	pdf_ocg_ui *entry;
@@ -514,18 +446,6 @@ void pdf_deselect_layer_config_ui(fz_context *ctx, pdf_document *doc, int ui)
 	doc->ocg->ocgs[entry->ocg].state = 0;
 }
 
-/*
-	Get the info for a given
-	entry in the layer config ui.
-
-	doc: The document in question.
-
-	ui: A value in the 0..m-1 range, where m is the value
-	returned by pdf_count_layer_config_ui.
-
-	info: Pointer to a structure to fill in with information
-	about the requested ui entry.
-*/
 void
 pdf_layer_config_ui_info(fz_context *ctx, pdf_document *doc, int ui, pdf_layer_config_ui *info)
 {
@@ -821,10 +741,6 @@ pdf_read_ocg(fz_context *ctx, pdf_document *doc)
 	pdf_select_layer_config(ctx, doc, 0);
 }
 
-/*
-	Write the current layer
-	config back into the document as the default state.
-*/
 void
 pdf_set_layer_config_as_default(fz_context *ctx, pdf_document *doc)
 {
