@@ -1,15 +1,13 @@
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
 
-typedef struct pdf_output_processor_s pdf_output_processor;
-
-struct pdf_output_processor_s
+typedef struct
 {
 	pdf_processor super;
 	fz_output *out;
 	int ahxencode;
 	int extgstate;
-};
+} pdf_output_processor;
 
 /* general graphics state */
 
@@ -49,9 +47,10 @@ static void
 pdf_out_d(fz_context *ctx, pdf_processor *proc, pdf_obj *array, float phase)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
+	int ahx = ((pdf_output_processor*)proc)->ahxencode;
 	if (!((pdf_output_processor*)proc)->extgstate)
 	{
-		pdf_print_obj(ctx, out, array, 1);
+		pdf_print_obj(ctx, out, array, 1, ahx);
 		fz_write_printf(ctx, out, " %g d\n", phase);
 	}
 }
@@ -61,7 +60,7 @@ pdf_out_ri(fz_context *ctx, pdf_processor *proc, const char *intent)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	if (!((pdf_output_processor*)proc)->extgstate)
-		fz_write_printf(ctx, out, "/%s ri\n", intent);
+		fz_write_printf(ctx, out, "%n ri\n", intent);
 }
 
 static void
@@ -97,7 +96,7 @@ pdf_out_gs_begin(fz_context *ctx, pdf_processor *proc, const char *name, pdf_obj
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	((pdf_output_processor*)proc)->extgstate = 1;
-	fz_write_printf(ctx, out, "/%s gs\n", name);
+	fz_write_printf(ctx, out, "%n gs\n", name);
 }
 
 static void
@@ -303,6 +302,7 @@ pdf_out_Tw(fz_context *ctx, pdf_processor *proc, float wordspace)
 static void
 pdf_out_Tz(fz_context *ctx, pdf_processor *proc, float scale)
 {
+	/* scale is exactly as read from the file. */
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	fz_write_printf(ctx, out, "%g Tz\n", scale);
 }
@@ -319,7 +319,9 @@ pdf_out_Tf(fz_context *ctx, pdf_processor *proc, const char *name, pdf_font_desc
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	if (!((pdf_output_processor*)proc)->extgstate)
-		fz_write_printf(ctx, out, "/%s %g Tf\n", name, size);
+	{
+		fz_write_printf(ctx, out, "%n %g Tf\n", name, size);
+	}
 }
 
 static void
@@ -369,9 +371,9 @@ pdf_out_Tstar(fz_context *ctx, pdf_processor *proc)
 /* text showing */
 
 static void
-fz_write_pdf_string(fz_context *ctx, fz_output *out, const unsigned char *str, int len)
+fz_write_pdf_string(fz_context *ctx, fz_output *out, const unsigned char *str, size_t len)
 {
-	int i;
+	size_t i;
 
 	for (i = 0; i < len; ++i)
 		if (str[i] < 32 || str[i] >= 127)
@@ -406,12 +408,13 @@ static void
 pdf_out_TJ(fz_context *ctx, pdf_processor *proc, pdf_obj *array)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	pdf_print_obj(ctx, out, array, 1);
+	int ahx = ((pdf_output_processor*)proc)->ahxencode;
+	pdf_print_obj(ctx, out, array, 1, ahx);
 	fz_write_string(ctx, out, " TJ\n");
 }
 
 static void
-pdf_out_Tj(fz_context *ctx, pdf_processor *proc, char *str, int len)
+pdf_out_Tj(fz_context *ctx, pdf_processor *proc, char *str, size_t len)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	fz_write_pdf_string(ctx, out, (const unsigned char *)str, len);
@@ -419,7 +422,7 @@ pdf_out_Tj(fz_context *ctx, pdf_processor *proc, char *str, int len)
 }
 
 static void
-pdf_out_squote(fz_context *ctx, pdf_processor *proc, char *str, int len)
+pdf_out_squote(fz_context *ctx, pdf_processor *proc, char *str, size_t len)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	fz_write_pdf_string(ctx, out, (const unsigned char *)str, len);
@@ -427,7 +430,7 @@ pdf_out_squote(fz_context *ctx, pdf_processor *proc, char *str, int len)
 }
 
 static void
-pdf_out_dquote(fz_context *ctx, pdf_processor *proc, float aw, float ac, char *str, int len)
+pdf_out_dquote(fz_context *ctx, pdf_processor *proc, float aw, float ac, char *str, size_t len)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	fz_write_printf(ctx, out, "%g %g ", aw, ac);
@@ -457,14 +460,14 @@ static void
 pdf_out_CS(fz_context *ctx, pdf_processor *proc, const char *name, fz_colorspace *cs)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s CS\n", name);
+	fz_write_printf(ctx, out, "%n CS\n", name);
 }
 
 static void
 pdf_out_cs(fz_context *ctx, pdf_processor *proc, const char *name, fz_colorspace *cs)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s cs\n", name);
+	fz_write_printf(ctx, out, "%n cs\n", name);
 }
 
 static void
@@ -474,7 +477,7 @@ pdf_out_SC_pattern(fz_context *ctx, pdf_processor *proc, const char *name, pdf_p
 	int i;
 	for (i = 0; i < n; ++i)
 		fz_write_printf(ctx, out, "%g ", color[i]);
-	fz_write_printf(ctx, out, "/%s SCN\n", name);
+	fz_write_printf(ctx, out, "%n SCN\n", name);
 }
 
 static void
@@ -484,21 +487,21 @@ pdf_out_sc_pattern(fz_context *ctx, pdf_processor *proc, const char *name, pdf_p
 	int i;
 	for (i = 0; i < n; ++i)
 		fz_write_printf(ctx, out, "%g ", color[i]);
-	fz_write_printf(ctx, out, "/%s scn\n", name);
+	fz_write_printf(ctx, out, "%n scn\n", name);
 }
 
 static void
 pdf_out_SC_shade(fz_context *ctx, pdf_processor *proc, const char *name, fz_shade *shade)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s SCN\n", name);
+	fz_write_printf(ctx, out, "%n SCN\n", name);
 }
 
 static void
 pdf_out_sc_shade(fz_context *ctx, pdf_processor *proc, const char *name, fz_shade *shade)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s scn\n", name);
+	fz_write_printf(ctx, out, "%n scn\n", name);
 }
 
 static void
@@ -566,7 +569,7 @@ pdf_out_k(fz_context *ctx, pdf_processor *proc, float c, float m, float y, float
 /* shadings, images, xobjects */
 
 static void
-pdf_out_BI(fz_context *ctx, pdf_processor *proc, fz_image *img)
+pdf_out_BI(fz_context *ctx, pdf_processor *proc, fz_image *img, const char *colorspace)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
 	int ahx = ((pdf_output_processor*)proc)->ahxencode;
@@ -597,10 +600,10 @@ pdf_out_BI(fz_context *ctx, pdf_processor *proc, fz_image *img)
 		fz_write_string(ctx, out, "/CS/RGB\n");
 	else if (img->colorspace == fz_device_cmyk(ctx))
 		fz_write_string(ctx, out, "/CS/CMYK\n");
-	else if (fz_colorspace_is_indexed(ctx, img->colorspace))
-		fz_write_string(ctx, out, "/CS/I\n");
+	else if (colorspace)
+		fz_write_printf(ctx, out, "/CS%n/n", colorspace);
 	else
-		fz_throw(ctx, FZ_ERROR_GENERIC, "BI operator can only show mask, Gray, RGB, CMYK, or Indexed images");
+		fz_throw(ctx, FZ_ERROR_GENERIC, "BI operator can only show ImageMask, Gray, RGB, or CMYK images");
 	if (img->interpolate)
 		fz_write_string(ctx, out, "/I true\n");
 	fz_write_string(ctx, out, "/D[");
@@ -716,21 +719,21 @@ static void
 pdf_out_sh(fz_context *ctx, pdf_processor *proc, const char *name, fz_shade *shade)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s sh\n", name);
+	fz_write_printf(ctx, out, "%n sh\n", name);
 }
 
 static void
 pdf_out_Do_image(fz_context *ctx, pdf_processor *proc, const char *name, fz_image *image)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s Do\n", name);
+	fz_write_printf(ctx, out, "%n Do\n", name);
 }
 
 static void
 pdf_out_Do_form(fz_context *ctx, pdf_processor *proc, const char *name, pdf_obj *xobj, pdf_obj *page_resources)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s Do\n", name);
+	fz_write_printf(ctx, out, "%n Do\n", name);
 }
 
 /* marked content */
@@ -739,15 +742,16 @@ static void
 pdf_out_MP(fz_context *ctx, pdf_processor *proc, const char *tag)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s MP\n", tag);
+	fz_write_printf(ctx, out, "%n MP\n", tag);
 }
 
 static void
 pdf_out_DP(fz_context *ctx, pdf_processor *proc, const char *tag, pdf_obj *raw, pdf_obj *cooked)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s ", tag);
-	pdf_print_obj(ctx, out, raw, 1);
+	int ahx = ((pdf_output_processor*)proc)->ahxencode;
+	fz_write_printf(ctx, out, "%n ", tag);
+	pdf_print_obj(ctx, out, raw, 1, ahx);
 	fz_write_string(ctx, out, " DP\n");
 }
 
@@ -755,15 +759,16 @@ static void
 pdf_out_BMC(fz_context *ctx, pdf_processor *proc, const char *tag)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s BMC\n", tag);
+	fz_write_printf(ctx, out, "%n BMC\n", tag);
 }
 
 static void
 pdf_out_BDC(fz_context *ctx, pdf_processor *proc, const char *tag, pdf_obj *raw, pdf_obj *cooked)
 {
 	fz_output *out = ((pdf_output_processor*)proc)->out;
-	fz_write_printf(ctx, out, "/%s ", tag);
-	pdf_print_obj(ctx, out, raw, 1);
+	int ahx = ((pdf_output_processor*)proc)->ahxencode;
+	fz_write_printf(ctx, out, "%n ", tag);
+	pdf_print_obj(ctx, out, raw, 1, ahx);
 	fz_write_string(ctx, out, " BDC\n");
 }
 
