@@ -8,9 +8,18 @@
 int
 pdf_count_pages(fz_context *ctx, pdf_document *doc)
 {
+	/* FIXME: We should reset linear_page_count to 0 when editing starts
+	 * (or when linear loading ends) */
+	if (doc->linear_page_count != 0)
+		return doc->linear_page_count;
 	return pdf_to_int(ctx, pdf_dict_getp(ctx, pdf_trailer(ctx, doc), "Root/Pages/Count"));
 }
 
+int pdf_count_pages_imp(fz_context *ctx, fz_document *doc, int chapter)
+{
+	return pdf_count_pages(ctx, (pdf_document*)doc);
+}
+
 static int
 pdf_load_page_tree_imp(fz_context *ctx, pdf_document *doc, pdf_obj *node, int idx)
 {
@@ -59,7 +68,7 @@ pdf_load_page_tree(fz_context *ctx, pdf_document *doc)
 	if (!doc->rev_page_map)
 	{
 		doc->rev_page_count = pdf_count_pages(ctx, doc);
-		doc->rev_page_map = fz_malloc_array(ctx, doc->rev_page_count, sizeof *doc->rev_page_map);
+		doc->rev_page_map = Memento_label(fz_malloc_array(ctx, doc->rev_page_count, pdf_rev_page_map), "pdf_rev_page_map");
 		pdf_load_page_tree_imp(ctx, doc, pdf_dict_getp(ctx, pdf_trailer(ctx, doc), "Root/Pages"), 0);
 		qsort(doc->rev_page_map, doc->rev_page_count, sizeof *doc->rev_page_map, cmp_rev_page_map);
 	}
@@ -109,11 +118,13 @@ pdf_lookup_page_loc_imp(fz_context *ctx, pdf_document *doc, pdf_obj *node, int *
 			{
 				if (stack == &local_stack[0])
 				{
-					stack = fz_malloc_array(ctx, stack_max * 2, sizeof(*stack));
+					stack = fz_malloc_array(ctx, stack_max * 2, pdf_obj*);
 					memcpy(stack, &local_stack[0], stack_max * sizeof(*stack));
 				}
 				else
-					stack = fz_resize_array(ctx, stack, stack_max * 2, sizeof(*stack));
+				{
+					stack = fz_realloc_array(ctx, stack, stack_max * 2, pdf_obj*);
+				}
 				stack_max *= 2;
 			}
 			stack[stack_len++] = node;
@@ -192,7 +203,7 @@ pdf_lookup_page_loc(fz_context *ctx, pdf_document *doc, int needle, pdf_obj **pa
 
 	hit = pdf_lookup_page_loc_imp(ctx, doc, node, &skip, parentp, indexp);
 	if (!hit)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot find page %d in page tree", needle);
+		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot find page %d in page tree", needle+1);
 	return hit;
 }
 
@@ -296,12 +307,6 @@ pdf_lookup_page_number(fz_context *ctx, pdf_document *doc, pdf_obj *page)
 		return pdf_lookup_page_number_slow(ctx, doc, page);
 }
 
-/*
-	Find the page number of a named destination.
-
-	For use with looking up the destination page of a fragment
-	identifier in hyperlinks: foo.pdf#bar or foo.pdf#page=5.
-*/
 int
 pdf_lookup_anchor(fz_context *ctx, pdf_document *doc, const char *name, float *xp, float *yp)
 {
@@ -339,12 +344,6 @@ pdf_flatten_inheritable_page_item(fz_context *ctx, pdf_obj *page, pdf_obj *key)
 		pdf_dict_put(ctx, page, key, val);
 }
 
-/*
-	Make page self sufficient.
-
-	Copy any inheritable page keys into the actual page object, removing
-	any dependencies on the page tree parents.
-*/
 void
 pdf_flatten_inheritable_page_items(fz_context *ctx, pdf_obj *page)
 {
@@ -588,14 +587,6 @@ pdf_page_presentation(fz_context *ctx, pdf_page *page, fz_transition *transition
 	return transition;
 }
 
-/*
-	Determine the size of a page.
-
-	Determine the page size in user space units, taking page rotation
-	into account. The page size is taken to be the crop box if it
-	exists (visible area after cropping), otherwise the media box will
-	be used (possibly including printing marks).
-*/
 fz_rect
 pdf_bound_page(fz_context *ctx, pdf_page *page)
 {
@@ -697,11 +688,21 @@ pdf_page_transform(fz_context *ctx, pdf_page *page, fz_rect *page_mediabox, fz_m
 }
 
 static void
-find_seps(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
+find_seps(fz_context *ctx, fz_separations **seps, pdf_obj *obj, pdf_obj *clearme)
 {
 	int i, n;
-	pdf_obj *nameobj = pdf_array_get(ctx, obj, 0);
+	pdf_obj *nameobj;
 
+	/* Indexed and DeviceN may have cyclic references */
+	if (pdf_is_indirect(ctx, obj))
+	{
+		if (pdf_mark_obj(ctx, obj))
+			return; /* already been here */
+		/* remember to clear this colorspace dictionary at the end */
+		pdf_array_push(ctx, clearme, obj);
+	}
+
+	nameobj = pdf_array_get(ctx, obj, 0);
 	if (pdf_name_eq(ctx, nameobj, PDF_NAME(Separation)))
 	{
 		fz_colorspace *cs;
@@ -726,7 +727,10 @@ find_seps(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 		fz_try(ctx)
 			cs = pdf_load_colorspace(ctx, obj);
 		fz_catch(ctx)
+		{
+			fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 			return; /* ignore broken colorspace */
+		}
 		fz_try(ctx)
 		{
 			if (!*seps)
@@ -740,7 +744,7 @@ find_seps(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 	}
 	else if (pdf_name_eq(ctx, nameobj, PDF_NAME(Indexed)))
 	{
-		find_seps(ctx, seps, pdf_array_get(ctx, obj, 1));
+		find_seps(ctx, seps, pdf_array_get(ctx, obj, 1), clearme);
 	}
 	else if (pdf_name_eq(ctx, nameobj, PDF_NAME(DeviceN)))
 	{
@@ -749,12 +753,12 @@ find_seps(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 		pdf_obj *cols = pdf_dict_get(ctx, pdf_array_get(ctx, obj, 4), PDF_NAME(Colorants));
 		n = pdf_dict_len(ctx, cols);
 		for (i = 0; i < n; i++)
-			find_seps(ctx, seps, pdf_dict_get_val(ctx, cols, i));
+			find_seps(ctx, seps, pdf_dict_get_val(ctx, cols, i), clearme);
 	}
 }
 
 static void
-find_devn(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
+find_devn(fz_context *ctx, fz_separations **seps, pdf_obj *obj, pdf_obj *clearme)
 {
 	int i, j, n, m;
 	pdf_obj *arr;
@@ -791,7 +795,10 @@ find_devn(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 			fz_try(ctx)
 				cs = pdf_load_colorspace(ctx, obj);
 			fz_catch(ctx)
+			{
+				fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 				continue; /* ignore broken colorspace */
+			}
 			fz_try(ctx)
 			{
 				if (!*seps)
@@ -806,7 +813,7 @@ find_devn(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 	}
 }
 
-typedef void (res_finder_fn)(fz_context *ctx, fz_separations **seps, pdf_obj *obj);
+typedef void (res_finder_fn)(fz_context *ctx, fz_separations **seps, pdf_obj *obj, pdf_obj *clearme);
 
 static void
 scan_page_seps(fz_context *ctx, pdf_obj *res, fz_separations **seps, res_finder_fn *fn, pdf_obj *clearme)
@@ -826,7 +833,7 @@ scan_page_seps(fz_context *ctx, pdf_obj *res, fz_separations **seps, res_finder_
 	for (i = 0; i < n; i++)
 	{
 		obj = pdf_dict_get_val(ctx, dict, i);
-		fn(ctx, seps, obj);
+		fn(ctx, seps, obj, clearme);
 	}
 
 	dict = pdf_dict_get(ctx, res, PDF_NAME(Shading));
@@ -834,7 +841,7 @@ scan_page_seps(fz_context *ctx, pdf_obj *res, fz_separations **seps, res_finder_
 	for (i = 0; i < n; i++)
 	{
 		obj = pdf_dict_get_val(ctx, dict, i);
-		fn(ctx, seps, pdf_dict_get(ctx, obj, PDF_NAME(ColorSpace)));
+		fn(ctx, seps, pdf_dict_get(ctx, obj, PDF_NAME(ColorSpace)), clearme);
 	}
 
 	dict = pdf_dict_get(ctx, res, PDF_NAME(XObject));
@@ -842,15 +849,12 @@ scan_page_seps(fz_context *ctx, pdf_obj *res, fz_separations **seps, res_finder_
 	for (i = 0; i < n; i++)
 	{
 		obj = pdf_dict_get_val(ctx, dict, i);
-		fn(ctx, seps, pdf_dict_get(ctx, obj, PDF_NAME(ColorSpace)));
+		fn(ctx, seps, pdf_dict_get(ctx, obj, PDF_NAME(ColorSpace)), clearme);
 		/* Recurse on XObject forms. */
 		scan_page_seps(ctx, pdf_dict_get(ctx, obj, PDF_NAME(Resources)), seps, fn, clearme);
 	}
 }
 
-/*
-	Get the separation details for a page.
-*/
 fz_separations *
 pdf_page_separations(fz_context *ctx, pdf_page *page)
 {
@@ -957,49 +961,44 @@ pdf_load_default_colorspaces_imp(fz_context *ctx, fz_default_colorspaces *defaul
 	pdf_obj *cs_obj;
 
 	/* The spec says to ignore any colors we can't understand */
-	fz_try(ctx)
+
+	cs_obj = pdf_dict_get(ctx, obj, PDF_NAME(DefaultGray));
+	if (cs_obj)
 	{
-		cs_obj = pdf_dict_get(ctx, obj, PDF_NAME(DefaultGray));
-		if (cs_obj)
+		fz_try(ctx)
 		{
 			fz_colorspace *cs = pdf_load_colorspace(ctx, cs_obj);
 			fz_set_default_gray(ctx, default_cs, cs);
 			fz_drop_colorspace(ctx, cs);
 		}
-	}
-	fz_catch(ctx)
-	{
-		fz_warn(ctx, "Error while reading DefaultGray: %s", fz_caught_message(ctx));
+		fz_catch(ctx)
+			fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 	}
 
-	fz_try(ctx)
+	cs_obj = pdf_dict_get(ctx, obj, PDF_NAME(DefaultRGB));
+	if (cs_obj)
 	{
-		cs_obj = pdf_dict_get(ctx, obj, PDF_NAME(DefaultRGB));
-		if (cs_obj)
+		fz_try(ctx)
 		{
 			fz_colorspace *cs = pdf_load_colorspace(ctx, cs_obj);
 			fz_set_default_rgb(ctx, default_cs, cs);
 			fz_drop_colorspace(ctx, cs);
 		}
-	}
-	fz_catch(ctx)
-	{
-		fz_warn(ctx, "Error while reading DefaultRGB: %s", fz_caught_message(ctx));
+		fz_catch(ctx)
+			fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 	}
 
-	fz_try(ctx)
+	cs_obj = pdf_dict_get(ctx, obj, PDF_NAME(DefaultCMYK));
+	if (cs_obj)
 	{
-		cs_obj = pdf_dict_get(ctx, obj, PDF_NAME(DefaultCMYK));
-		if (cs_obj)
+		fz_try(ctx)
 		{
 			fz_colorspace *cs = pdf_load_colorspace(ctx, cs_obj);
 			fz_set_default_cmyk(ctx, default_cs, cs);
 			fz_drop_colorspace(ctx, cs);
 		}
-	}
-	fz_catch(ctx)
-	{
-		fz_warn(ctx, "Error while reading DefaultCMYK: %s", fz_caught_message(ctx));
+		fz_catch(ctx)
+			fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 	}
 }
 
@@ -1026,16 +1025,17 @@ pdf_load_default_colorspaces(fz_context *ctx, pdf_document *doc, pdf_page *page)
 	}
 	fz_catch(ctx)
 	{
-		fz_drop_default_colorspaces(ctx, default_cs);
-		fz_rethrow(ctx);
+		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
+		{
+			fz_drop_default_colorspaces(ctx, default_cs);
+			fz_rethrow(ctx);
+		}
+		page->super.incomplete = 1;
 	}
 
 	return default_cs;
 }
 
-/*
-	Update default colorspaces for an xobject.
-*/
 fz_default_colorspaces *
 pdf_update_default_colorspaces(fz_context *ctx, fz_default_colorspaces *old_cs, pdf_obj *res)
 {
@@ -1047,21 +1047,17 @@ pdf_update_default_colorspaces(fz_context *ctx, fz_default_colorspaces *old_cs,
 		return fz_keep_default_colorspaces(ctx, old_cs);
 
 	new_cs = fz_clone_default_colorspaces(ctx, old_cs);
-	pdf_load_default_colorspaces_imp(ctx, new_cs, obj);
+	fz_try(ctx)
+		pdf_load_default_colorspaces_imp(ctx, new_cs, obj);
+	fz_catch(ctx)
+	{
+		fz_drop_default_colorspaces(ctx, new_cs);
+		fz_rethrow(ctx);
+	}
 
 	return new_cs;
 }
 
-/*
-	Load a page and its resources.
-
-	Locates the page in the PDF document and loads the page and its
-	resources. After pdf_load_page is it possible to retrieve the size
-	of the page using pdf_bound_page, or to render the page using
-	pdf_run_page_*.
-
-	number: page number, where 0 is the first page of the document.
-*/
 pdf_page *
 pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 {
@@ -1069,7 +1065,14 @@ pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 	pdf_annot *annot;
 	pdf_obj *pageobj, *obj;
 
-	pageobj = pdf_lookup_page_obj(ctx, doc, number);
+	if (doc->file_reading_linearly)
+	{
+		pageobj = pdf_progressive_advance(ctx, doc, number);
+		if (pageobj == NULL)
+			fz_throw(ctx, FZ_ERROR_TRYLATER, "page %d not available yet", number);
+	}
+	else
+		pageobj = pdf_lookup_page_obj(ctx, doc, number);
 
 	page = pdf_new_page(ctx, doc);
 	page->obj = pdf_keep_obj(ctx, pageobj);
@@ -1089,8 +1092,14 @@ pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 	}
 	fz_catch(ctx)
 	{
-		fz_drop_page(ctx, &page->super);
-		fz_rethrow(ctx);
+		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
+		{
+			fz_drop_page(ctx, &page->super);
+			fz_rethrow(ctx);
+		}
+		page->super.incomplete = 1;
+		fz_drop_link(ctx, page->links);
+		page->links = NULL;
 	}
 
 	/* Scan for transparency and overprint */
@@ -1112,22 +1121,22 @@ pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 	}
 	fz_catch(ctx)
 	{
-		fz_drop_page(ctx, &page->super);
-		fz_rethrow(ctx);
+		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
+		{
+			fz_drop_page(ctx, &page->super);
+			fz_rethrow(ctx);
+		}
+		page->super.incomplete = 1;
 	}
 
 	return page;
 }
 
-/*
-	Delete a page from the page tree of
-	a document. This does not remove the page contents
-	or resources from the file.
+fz_page *pdf_load_page_imp(fz_context *ctx, fz_document *doc, int chapter, int number)
+{
+	return (fz_page*)pdf_load_page(ctx, (pdf_document*)doc, number);
+}
 
-	doc: The document to operate on.
-
-	number: The page to remove (numbered from 0)
-*/
 void
 pdf_delete_page(fz_context *ctx, pdf_document *doc, int at)
 {
@@ -1146,18 +1155,6 @@ pdf_delete_page(fz_context *ctx, pdf_document *doc, int at)
 	}
 }
 
-/*
-	Delete a range of pages from the
-	page tree of a document. This does not remove the page
-	contents or resources from the file.
-
-	doc: The document to operate on.
-
-	start, end: The range of pages (numbered from 0)
-	(inclusive, exclusive) to remove. If end is negative or
-	greater than the number of pages in the document, it
-	will be taken to be the end of the document.
-*/
 void
 pdf_delete_page_range(fz_context *ctx, pdf_document *doc, int start, int end)
 {
@@ -1174,30 +1171,6 @@ pdf_delete_page_range(fz_context *ctx, pdf_document *doc, int start, int end)
 	}
 }
 
-/*
-	Create a pdf_obj within a document that
-	represents a page, from a previously created resources
-	dictionary and page content stream. This should then be
-	inserted into the document using pdf_insert_page.
-
-	After this call the page exists within the document
-	structure, but is not actually ever displayed as it is
-	not linked into the PDF page tree.
-
-	doc: The document to which to add the page.
-
-	mediabox: The mediabox for the page (should be identical
-	to that used when creating the resources/contents).
-
-	rotate: 0, 90, 180 or 270. The rotation to use for the
-	page.
-
-	resources: The resources dictionary for the new page
-	(typically created by pdf_page_write).
-
-	contents: The page contents for the new page (typically
-	create by pdf_page_write).
-*/
 pdf_obj *
 pdf_add_page(fz_context *ctx, pdf_document *doc, fz_rect mediabox, int rotate, pdf_obj *resources, fz_buffer *contents)
 {
@@ -1226,18 +1199,6 @@ pdf_add_page(fz_context *ctx, pdf_document *doc, fz_rect mediabox, int rotate, p
 	return pdf_add_object_drop(ctx, doc, page_obj);
 }
 
-/*
-	Insert a page previously created by
-	pdf_add_page into the pages tree of the document.
-
-	doc: The document to insert into.
-
-	at: The page number to insert at. 0 inserts at the start.
-	negative numbers, or INT_MAX insert at the end. Otherwise
-	n inserts after page n.
-
-	page: The page to insert.
-*/
 void
 pdf_insert_page(fz_context *ctx, pdf_document *doc, int at, pdf_obj *page_ref)
 {
