@@ -13,8 +13,10 @@
 #endif
 
 #include <string.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <sys/stat.h>
 #ifdef _MSC_VER
 struct timeval;
 struct timezone;
@@ -22,6 +24,16 @@ int gettimeofday(struct timeval *tv, struct timezone *tz);
 #else
 #include <sys/time.h>
 #endif
+#ifdef _WIN32
+#include <windows.h>
+#include <direct.h> /* for getcwd */
+#else
+#include <unistd.h> /* for getcwd */
+#endif
+
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
 
 /* Allow for windows stdout being made binary */
 #ifdef _WIN32
@@ -141,6 +153,17 @@ static const format_cs_table_t format_cs_table[] =
 	{ OUT_STEXT, CS_RGB, { CS_RGB } },
 };
 
+time_t
+stat_mtime(const char *path)
+{
+	struct stat info;
+
+	if (stat(path, &info) < 0)
+		return 0;
+
+	return info.st_mtime;
+}
+
 /*
 	In the presence of pthreads or Windows threads, we can offer
 	a multi-threaded option. In the absence, of such, we degrade
@@ -265,6 +288,7 @@ static int spots = SPOTS_OVERPRINT_SIM;
 static int spots = SPOTS_NONE;
 #endif
 static int alpha;
+static int useaccel = 1;
 static char *filename;
 static int files = 0;
 static int num_workers = 0;
@@ -297,6 +321,10 @@ static struct {
 	int minpage, maxpage;
 	char *minfilename;
 	char *maxfilename;
+	int layout;
+	int minlayout, maxlayout;
+	char *minlayoutfilename;
+	char *maxlayoutfilename;
 } timing;
 
 static void usage(void)
@@ -336,6 +364,7 @@ static void usage(void)
 		"\t-S -\tfont size for EPUB layout\n"
 		"\t-U -\tfile name of user stylesheet for EPUB layout\n"
 		"\t-X\tdisable document styles for EPUB layout\n"
+		"\t-a\tdisable usage of accelerator file\n"
 		"\n"
 		"\t-c -\tcolorspace (mono, gray, grayalpha, rgb, rgba, cmyk, cmykalpha, filename of ICC profile)\n"
 		"\t-e -\tproof icc profile (filename of ICC profile)\n"
@@ -669,7 +698,7 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 				out = fz_stdout(ctx);
 			else
 			{
-				fz_snprintf(buf, sizeof(buf), output, pagenum);
+				fz_format_output_path(ctx, buf, sizeof buf, output, pagenum);
 				out = fz_new_output_with_path(ctx, buf, 0);
 			}
 
@@ -904,7 +933,14 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		fz_always(ctx)
 		{
 			if (output_format != OUT_PCLM)
+			{
 				fz_drop_band_writer(ctx, bander);
+				/* bander must be set to NULL to avoid use-after-frees. A use-after-free
+				 * would occur when a valid page was followed by a page with invalid
+				 * pixmap dimensions, causing bander -- a static -- to point to previously
+				 * freed memory instead of a new band_writer. */
+				bander = NULL;
+			}
 			fz_drop_bitmap(ctx, bit);
 			bit = NULL;
 			if (num_workers > 0)
@@ -987,7 +1023,7 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		fz_empty_store(ctx);
 
 	if (showmemory)
-		fz_dump_glyph_cache_stats(ctx);
+		fz_dump_glyph_cache_stats(ctx, fz_stderr(ctx));
 
 	fz_flush_warnings(ctx);
 
@@ -1131,7 +1167,7 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 			fz_close_output(ctx, out);
 			fz_drop_output(ctx, out);
 		}
-		fz_snprintf(text_buffer, sizeof(text_buffer), output, pagenum);
+		fz_format_output_path(ctx, text_buffer, sizeof text_buffer, output, pagenum);
 		out = fz_new_output_with_path(ctx, text_buffer, 0);
 	}
 
@@ -1208,7 +1244,7 @@ parse_colorspace(const char *name)
 {
 	int i;
 
-	for (i = 0; i < nelem(cs_name_table); i++)
+	for (i = 0; i < (int)nelem(cs_name_table); i++)
 	{
 		if (!strcmp(name, cs_name_table[i].name))
 			return cs_name_table[i].colorspace;
@@ -1442,6 +1478,58 @@ static void apply_layer_config(fz_context *ctx, fz_document *doc, const char *lc
 #endif
 }
 
+static int convert_to_accel_path(fz_context *ctx, char outname[], char *absname, size_t len)
+{
+	char *tmpdir;
+	char *s;
+
+	tmpdir = getenv("TEMP");
+	if (!tmpdir)
+		tmpdir = getenv("TMP");
+	if (!tmpdir)
+		tmpdir = "/var/tmp";
+	if (!fz_is_directory(ctx, tmpdir))
+		tmpdir = "/tmp";
+
+	if (absname[0] == '/' || absname[0] == '\\')
+		++absname;
+
+	s = absname;
+	while (*s) {
+		if (*s == '/' || *s == '\\' || *s == ':')
+			*s = '%';
+		++s;
+	}
+
+	if (fz_snprintf(outname, len, "%s/%s.accel", tmpdir, absname) >= len)
+		return 0;
+	return 1;
+}
+
+static int get_accelerator_filename(fz_context *ctx, char outname[], size_t len, const char *filename)
+{
+	char absname[PATH_MAX];
+	if (!fz_realpath(filename, absname))
+		return 0;
+	if (!convert_to_accel_path(ctx, outname, absname, len))
+		return 0;
+	return 1;
+}
+
+static void save_accelerator(fz_context *ctx, fz_document *doc, const char *filename)
+{
+	char absname[PATH_MAX];
+
+	if (!doc)
+		return;
+	if (!fz_document_supports_accelerator(ctx, doc))
+		return;
+	if (!get_accelerator_filename(ctx, absname, sizeof(absname), filename))
+		return;
+
+	fz_save_accelerator(ctx, doc, absname);
+}
+
 #ifdef MUDRAW_STANDALONE
 int main(int argc, char **argv)
 #else
@@ -1458,7 +1546,7 @@ int mudraw_main(int argc, char **argv)
 
 	fz_var(doc);
 
-	while ((c = fz_getopt(argc, argv, "qp:o:F:R:r:w:h:fB:c:e:G:Is:A:DiW:H:S:T:U:XLvPl:y:NO:")) != -1)
+	while ((c = fz_getopt(argc, argv, "qp:o:F:R:r:w:h:fB:c:e:G:Is:A:DiW:H:S:T:U:XLvPl:y:NO:a")) != -1)
 	{
 		switch (c)
 		{
@@ -1535,6 +1623,7 @@ int mudraw_main(int argc, char **argv)
 			break;
 #endif
 		case 'y': layer_config = fz_optarg; break;
+		case 'a': useaccel = 0; break;
 
 		case 'v': fprintf(stderr, "mudraw version %s\n", FZ_VERSION); return 1;
 		}
@@ -1656,7 +1745,7 @@ int mudraw_main(int argc, char **argv)
 		{
 			int i;
 
-			for (i = 0; i < nelem(suffix_table); i++)
+			for (i = 0; i < (int)nelem(suffix_table); i++)
 			{
 				if (!strcmp(format, suffix_table[i].suffix+1))
 				{
@@ -1669,7 +1758,7 @@ int mudraw_main(int argc, char **argv)
 					break;
 				}
 			}
-			if (i == nelem(suffix_table))
+			if (i == (int)nelem(suffix_table))
 			{
 				fprintf(stderr, "Unknown output format '%s'\n", format);
 				exit(1);
@@ -1680,7 +1769,7 @@ int mudraw_main(int argc, char **argv)
 			char *suffix = output;
 			int i;
 
-			for (i = 0; i < nelem(suffix_table); i++)
+			for (i = 0; i < (int)nelem(suffix_table); i++)
 			{
 				char *s = strstr(suffix, suffix_table[i].suffix);
 
@@ -1715,18 +1804,18 @@ int mudraw_main(int argc, char **argv)
 		{
 			int i, j;
 
-			for (i = 0; i < nelem(format_cs_table); i++)
+			for (i = 0; i < (int)nelem(format_cs_table); i++)
 			{
 				if (format_cs_table[i].format == output_format)
 				{
 					if (out_cs == CS_UNSET)
 						out_cs = format_cs_table[i].default_cs;
-					for (j = 0; j < nelem(format_cs_table[i].permitted_cs); j++)
+					for (j = 0; j < (int)nelem(format_cs_table[i].permitted_cs); j++)
 					{
 						if (format_cs_table[i].permitted_cs[j] == out_cs)
 							break;
 					}
-					if (j == nelem(format_cs_table[i].permitted_cs))
+					if (j == (int)nelem(format_cs_table[i].permitted_cs))
 					{
 						fprintf(stderr, "Unsupported colorspace for this format\n");
 						exit(1);
@@ -1787,11 +1876,11 @@ int mudraw_main(int argc, char **argv)
 
 			/* Check to make sure this icc profile is ok with the output format */
 			okay = 0;
-			for (i = 0; i < nelem(format_cs_table); i++)
+			for (i = 0; i < (int)nelem(format_cs_table); i++)
 			{
 				if (format_cs_table[i].format == output_format)
 				{
-					for (j = 0; j < nelem(format_cs_table[i].permitted_cs); j++)
+					for (j = 0; j < (int)nelem(format_cs_table[i].permitted_cs); j++)
 					{
 						switch (format_cs_table[i].permitted_cs[j])
 						{
@@ -1866,6 +1955,11 @@ int mudraw_main(int argc, char **argv)
 		timing.maxpage = 0;
 		timing.minfilename = "";
 		timing.maxfilename = "";
+		timing.layout = 0;
+		timing.minlayout = 1 << 30;
+		timing.maxlayout = 0;
+		timing.minlayoutfilename = "";
+		timing.maxlayoutfilename = "";
 		if (showtime && bgprint.active)
 			timing.total = gettime();
 
@@ -1875,12 +1969,42 @@ int mudraw_main(int argc, char **argv)
 
 			while (fz_optind < argc)
 			{
+				char accelpath[PATH_MAX];
+				char *accel = NULL;
+				time_t atime;
+				time_t dtime;
+				int layouttime;
+
 				fz_try(ctx)
 				{
 					filename = argv[fz_optind++];
 					files++;
 
-					doc = fz_open_document(ctx, filename);
+					if (!useaccel)
+						accel = NULL;
+					/* If there was an accelerator to load, what would it be called? */
+					else if (get_accelerator_filename(ctx, accelpath, sizeof(accelpath), filename))
+					{
+						/* Check whether that file exists, and isn't older than
+						 * the document. */
+						atime = stat_mtime(accelpath);
+						dtime = stat_mtime(filename);
+						if (atime == 0)
+						{
+							/* No accelerator */
+						}
+						else if (atime > dtime)
+							accel = accelpath;
+						else
+						{
+							/* Accelerator data is out of date */
+							unlink(accelpath);
+							accel = NULL; /* In case we have jumped up from below */
+						}
+
+					}
+
+					doc = fz_open_accelerated_document(ctx, filename, accel);
 
 					if (fz_needs_password(ctx, doc))
 					{
@@ -1905,7 +2029,22 @@ int mudraw_main(int argc, char **argv)
 						}
 					}
 
+					layouttime = gettime();
 					fz_layout_document(ctx, doc, layout_w, layout_h, layout_em);
+					(void) fz_count_pages(ctx, doc);
+					layouttime = gettime() - layouttime;
+
+					timing.layout += layouttime;
+					if (layouttime < timing.minlayout)
+					{
+						timing.minlayout = layouttime;
+						timing.minlayoutfilename = filename;
+					}
+					if (layouttime > timing.maxlayout)
+					{
+						timing.maxlayout = layouttime;
+						timing.maxlayoutfilename = filename;
+					}
 
 					if (layer_config)
 						apply_layer_config(ctx, doc, layer_config);
@@ -1916,6 +2055,9 @@ int mudraw_main(int argc, char **argv)
 						drawrange(ctx, doc, argv[fz_optind++]);
 
 					bgprint_flush();
+
+					if (useaccel)
+						save_accelerator(ctx, doc, filename);
 				}
 				fz_always(ctx)
 				{
@@ -1966,8 +2108,8 @@ int mudraw_main(int argc, char **argv)
 
 			if (files == 1)
 			{
-				fprintf(stderr, "total %dms / %d pages for an average of %dms\n",
-						timing.total, timing.count, timing.total / timing.count);
+				fprintf(stderr, "total %dms (%dms layout) / %d pages for an average of %dms\n",
+						timing.total, timing.layout, timing.count, timing.total / timing.count);
 				if (bgprint.active)
 				{
 					fprintf(stderr, "fastest page %d: %dms (interpretation) %dms (rendering) %dms(total)\n",
@@ -1983,8 +2125,10 @@ int mudraw_main(int argc, char **argv)
 			}
 			else
 			{
-				fprintf(stderr, "total %dms / %d pages for an average of %dms in %d files\n",
-						timing.total, timing.count, timing.total / timing.count, files);
+				fprintf(stderr, "total %dms (%dms layout) / %d pages for an average of %dms in %d files\n",
+						timing.total, timing.layout, timing.count, timing.total / timing.count, files);
+				fprintf(stderr, "fastest layout: %dms (%s)\n", timing.minlayout, timing.minlayoutfilename);
+				fprintf(stderr, "slowest layout: %dms (%s)\n", timing.maxlayout, timing.maxlayoutfilename);
 				fprintf(stderr, "fastest page %d: %dms (%s)\n", timing.minpage, timing.min, timing.minfilename);
 				fprintf(stderr, "slowest page %d: %dms (%s)\n", timing.maxpage, timing.max, timing.maxfilename);
 			}
