@@ -27,6 +27,10 @@ static void usage(void)
 		"\t-gggg\tin addition to -ggg check streams for duplication\n"
 		"\t-l\tlinearize PDF\n"
 		"\t-D\tsave file without encryption\n"
+		"\t-E -\tsave file with new encryption (rc4-40, rc4-128, aes-128, or aes-256)\n"
+		"\t-O -\towner password (only if encrypting)\n"
+		"\t-U -\tuser password (only if encrypting)\n"
+		"\t-P -\tpermission flags (only if encrypting)\n"
 		"\t-a\tascii hex encode binary streams\n"
 		"\t-d\tdecompress streams\n"
 		"\t-z\tdeflate uncompressed streams\n"
@@ -41,17 +45,26 @@ static void usage(void)
 	exit(1);
 }
 
+static int encrypt_method_from_string(const char *name)
+{
+	if (!strcmp(name, "rc4-40")) return PDF_ENCRYPT_RC4_40;
+	if (!strcmp(name, "rc4-128")) return PDF_ENCRYPT_RC4_128;
+	if (!strcmp(name, "aes-128")) return PDF_ENCRYPT_AES_128;
+	if (!strcmp(name, "aes-256")) return PDF_ENCRYPT_AES_256;
+	return PDF_ENCRYPT_UNKNOWN;
+}
+
 int pdfclean_main(int argc, char **argv)
 {
 	char *infile;
 	char *outfile = "out.pdf";
 	char *password = "";
 	int c;
-	pdf_write_options opts = { 0 };
+	pdf_write_options opts = pdf_default_write_options;
 	int errors = 0;
 	fz_context *ctx;
 
-	while ((c = fz_getopt(argc, argv, "adfgilp:sczDA")) != -1)
+	while ((c = fz_getopt(argc, argv, "adfgilp:sczDAE:O:U:P:")) != -1)
 	{
 		switch (c)
 		{
@@ -66,8 +79,14 @@ int pdfclean_main(int argc, char **argv)
 		case 'l': opts.do_linear += 1; break;
 		case 'c': opts.do_clean += 1; break;
 		case 's': opts.do_sanitize += 1; break;
-		case 'D': opts.do_decrypt += 1; break;
 		case 'A': opts.do_appearance += 1; break;
+
+		case 'D': opts.do_encrypt = PDF_ENCRYPT_NONE; break;
+		case 'E': opts.do_encrypt = encrypt_method_from_string(fz_optarg); break;
+		case 'P': opts.permissions = fz_atoi(fz_optarg); break;
+		case 'O': fz_strlcpy(opts.opwd_utf8, fz_optarg, sizeof opts.opwd_utf8); break;
+		case 'U': fz_strlcpy(opts.upwd_utf8, fz_optarg, sizeof opts.upwd_utf8); break;
+
 		default: usage(); break;
 		}
 	}
