@@ -19,28 +19,36 @@ struct closure
 	xps_resource *dict;
 	fz_xml *root;
 	void *user;
-	void (*func)(fz_context *ctx, xps_document*, const fz_matrix *, const fz_rect *, char*, xps_resource*, fz_xml*, void*);
+	void (*func)(fz_context *ctx, xps_document*, fz_matrix, fz_rect, char*, xps_resource*, fz_xml*, void*);
 };
 
 static void
-xps_paint_tiling_brush_clipped(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const fz_rect *viewbox, struct closure *c)
+xps_paint_tiling_brush_clipped(fz_context *ctx, xps_document *doc, fz_matrix ctm, fz_rect viewbox, struct closure *c)
 {
 	fz_device *dev = doc->dev;
+	fz_path *path;
+
+	path = fz_new_path(ctx);
+	fz_try(ctx)
+	{
+		fz_moveto(ctx, path, viewbox.x0, viewbox.y0);
+		fz_lineto(ctx, path, viewbox.x0, viewbox.y1);
+		fz_lineto(ctx, path, viewbox.x1, viewbox.y1);
+		fz_lineto(ctx, path, viewbox.x1, viewbox.y0);
+		fz_closepath(ctx, path);
+		fz_clip_path(ctx, dev, path, 0, ctm, fz_infinite_rect);
+	}
+	fz_always(ctx)
+		fz_drop_path(ctx, path);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 
-	fz_path *path = fz_new_path(ctx);
-	fz_moveto(ctx, path, viewbox->x0, viewbox->y0);
-	fz_lineto(ctx, path, viewbox->x0, viewbox->y1);
-	fz_lineto(ctx, path, viewbox->x1, viewbox->y1);
-	fz_lineto(ctx, path, viewbox->x1, viewbox->y0);
-	fz_closepath(ctx, path);
-	fz_clip_path(ctx, dev, path, 0, ctm, NULL);
-	fz_drop_path(ctx, path);
 	c->func(ctx, doc, ctm, viewbox, c->base_uri, c->dict, c->root, c->user);
 	fz_pop_clip(ctx, dev);
 }
 
 static void
-xps_paint_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const fz_rect *viewbox, int tile_mode, struct closure *c)
+xps_paint_tiling_brush(fz_context *ctx, xps_document *doc, fz_matrix ctm, fz_rect viewbox, int tile_mode, struct closure *c)
 {
 	fz_matrix ttm;
 
@@ -48,30 +56,27 @@ xps_paint_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 
 	if (tile_mode == TILE_FLIP_X || tile_mode == TILE_FLIP_X_Y)
 	{
-		ttm = *ctm;
-		fz_pre_scale(fz_pre_translate(&ttm, viewbox->x1 * 2, 0), -1, 1);
-		xps_paint_tiling_brush_clipped(ctx, doc, &ttm, viewbox, c);
+		ttm = fz_pre_scale(fz_pre_translate(ctm, viewbox.x1 * 2, 0), -1, 1);
+		xps_paint_tiling_brush_clipped(ctx, doc, ttm, viewbox, c);
 	}
 
 	if (tile_mode == TILE_FLIP_Y || tile_mode == TILE_FLIP_X_Y)
 	{
-		ttm = *ctm;
-		fz_pre_scale(fz_pre_translate(&ttm, 0, viewbox->y1 * 2), 1, -1);
-		xps_paint_tiling_brush_clipped(ctx, doc, &ttm, viewbox, c);
+		ttm = fz_pre_scale(fz_pre_translate(ctm, 0, viewbox.y1 * 2), 1, -1);
+		xps_paint_tiling_brush_clipped(ctx, doc, ttm, viewbox, c);
 	}
 
 	if (tile_mode == TILE_FLIP_X_Y)
 	{
-		ttm = *ctm;
-		fz_pre_scale(fz_pre_translate(&ttm, viewbox->x1 * 2, viewbox->y1 * 2), -1, -1);
-		xps_paint_tiling_brush_clipped(ctx, doc, &ttm, viewbox, c);
+		ttm = fz_pre_scale(fz_pre_translate(ctm, viewbox.x1 * 2, viewbox.y1 * 2), -1, -1);
+		xps_paint_tiling_brush_clipped(ctx, doc, ttm, viewbox, c);
 	}
 }
 
 void
-xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const fz_rect *area,
+xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, fz_matrix ctm, fz_rect area,
 	char *base_uri, xps_resource *dict, fz_xml *root,
-	void (*func)(fz_context *ctx, xps_document*, const fz_matrix*, const fz_rect*, char*, xps_resource*, fz_xml*, void*), void *user)
+	void (*func)(fz_context *ctx, xps_document*, fz_matrix, fz_rect, char*, xps_resource*, fz_xml*, void*), void *user)
 {
 	fz_device *dev = doc->dev;
 	fz_xml *node;
@@ -85,7 +90,6 @@ xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 
 	fz_xml *transform_tag = NULL;
 
-	fz_matrix local_ctm;
 	fz_rect viewbox;
 	fz_rect viewport;
 	float xstep, ystep;
@@ -114,15 +118,15 @@ xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 
 	xps_resolve_resource_reference(ctx, doc, dict, &transform_att, &transform_tag, NULL);
 
-	xps_parse_transform(ctx, doc, transform_att, transform_tag, &local_ctm, ctm);
+	ctm = xps_parse_transform(ctx, doc, transform_att, transform_tag, ctm);
 
 	viewbox = fz_unit_rect;
 	if (viewbox_att)
-		xps_parse_rectangle(ctx, doc, viewbox_att, &viewbox);
+		viewbox = xps_parse_rectangle(ctx, doc, viewbox_att);
 
 	viewport = fz_unit_rect;
 	if (viewport_att)
-		xps_parse_rectangle(ctx, doc, viewport_att, &viewport);
+		viewport = xps_parse_rectangle(ctx, doc, viewport_att);
 
 	if (fabsf(viewport.x1 - viewport.x0) < 0.01f || fabsf(viewport.y1 - viewport.y0) < 0.01f)
 		fz_warn(ctx, "not drawing tile for viewport size %.4f x %.4f", viewport.x1 - viewport.x0, viewport.y1 - viewport.y0);
@@ -161,22 +165,22 @@ xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 	if (tile_mode == TILE_FLIP_Y || tile_mode == TILE_FLIP_X_Y)
 		ystep *= 2;
 
-	xps_begin_opacity(ctx, doc, &local_ctm, area, base_uri, dict, opacity_att, NULL);
+	xps_begin_opacity(ctx, doc, ctm, area, base_uri, dict, opacity_att, NULL);
 
-	fz_pre_translate(&local_ctm, viewport.x0, viewport.y0);
-	fz_pre_scale(&local_ctm, xscale, yscale);
-	fz_pre_translate(&local_ctm, -viewbox.x0, -viewbox.y0);
+	ctm = fz_pre_translate(ctm, viewport.x0, viewport.y0);
+	ctm = fz_pre_scale(ctm, xscale, yscale);
+	ctm = fz_pre_translate(ctm, -viewbox.x0, -viewbox.y0);
 
 	if (tile_mode != TILE_NONE)
 	{
 		int x0, y0, x1, y1;
 		fz_matrix invctm;
-		fz_rect local_area = *area;
-		fz_transform_rect(&local_area, fz_invert_matrix(&invctm, &local_ctm));
-		x0 = floorf(local_area.x0 / xstep);
-		y0 = floorf(local_area.y0 / ystep);
-		x1 = ceilf(local_area.x1 / xstep);
-		y1 = ceilf(local_area.y1 / ystep);
+		invctm = fz_invert_matrix(ctm);
+		area = fz_transform_rect(area, invctm);
+		x0 = floorf(area.x0 / xstep);
+		y0 = floorf(area.y0 / ystep);
+		x1 = ceilf(area.x1 / xstep);
+		y1 = ceilf(area.y1 / ystep);
 
 #ifdef TILE
 		if ((x1 - x0) * (y1 - y0) > 1)
@@ -187,8 +191,8 @@ xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 			fz_rect bigview = viewbox;
 			bigview.x1 = bigview.x0 + xstep;
 			bigview.y1 = bigview.y0 + ystep;
-			fz_begin_tile(ctx, dev, &local_area, &bigview, xstep, ystep, &local_ctm);
-			xps_paint_tiling_brush(ctx, doc, &local_ctm, &viewbox, tile_mode, &c);
+			fz_begin_tile(ctx, dev, area, bigview, xstep, ystep, ctm);
+			xps_paint_tiling_brush(ctx, doc, ctm, viewbox, tile_mode, &c);
 			fz_end_tile(ctx, dev);
 		}
 		else
@@ -198,30 +202,29 @@ xps_parse_tiling_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 			{
 				for (x = x0; x < x1; x++)
 				{
-					fz_matrix ttm = local_ctm;
-					fz_pre_translate(&ttm, xstep * x, ystep * y);
-					xps_paint_tiling_brush(ctx, doc, &ttm, &viewbox, tile_mode, &c);
+					fz_matrix ttm = fz_pre_translate(ctm, xstep * x, ystep * y);
+					xps_paint_tiling_brush(ctx, doc, ttm, viewbox, tile_mode, &c);
 				}
 			}
 		}
 	}
 	else
 	{
-		xps_paint_tiling_brush(ctx, doc, &local_ctm, &viewbox, tile_mode, &c);
+		xps_paint_tiling_brush(ctx, doc, ctm, viewbox, tile_mode, &c);
 	}
 
 	xps_end_opacity(ctx, doc, base_uri, dict, opacity_att, NULL);
 }
 
 static void
-xps_paint_visual_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const fz_rect *area,
+xps_paint_visual_brush(fz_context *ctx, xps_document *doc, fz_matrix ctm, fz_rect area,
 	char *base_uri, xps_resource *dict, fz_xml *root, void *visual_tag)
 {
 	xps_parse_element(ctx, doc, ctm, area, base_uri, dict, (fz_xml *)visual_tag);
 }
 
 void
-xps_parse_visual_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const fz_rect *area,
+xps_parse_visual_brush(fz_context *ctx, xps_document *doc, fz_matrix ctm, fz_rect area,
 	char *base_uri, xps_resource *dict, fz_xml *root)
 {
 	fz_xml *node;
@@ -249,7 +252,7 @@ xps_parse_visual_brush(fz_context *ctx, xps_document *doc, const fz_matrix *ctm,
 }
 
 void
-xps_parse_canvas(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const fz_rect *area, char *base_uri, xps_resource *dict, fz_xml *root)
+xps_parse_canvas(fz_context *ctx, xps_document *doc, fz_matrix ctm, fz_rect area, char *base_uri, xps_resource *dict, fz_xml *root)
 {
 	fz_device *dev = doc->dev;
 	xps_resource *new_dict = NULL;
@@ -265,8 +268,6 @@ xps_parse_canvas(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const
 	fz_xml *clip_tag = NULL;
 	fz_xml *opacity_mask_tag = NULL;
 
-	fz_matrix local_ctm;
-
 	transform_att = fz_xml_att(root, "RenderTransform");
 	clip_att = fz_xml_att(root, "Clip");
 	opacity_att = fz_xml_att(root, "Opacity");
@@ -306,15 +307,15 @@ xps_parse_canvas(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const
 		xps_resolve_resource_reference(ctx, doc, dict, &clip_att, &clip_tag, NULL);
 		xps_resolve_resource_reference(ctx, doc, dict, &opacity_mask_att, &opacity_mask_tag, &opacity_mask_uri);
 
-		xps_parse_transform(ctx, doc, transform_att, transform_tag, &local_ctm, ctm);
+		ctm = xps_parse_transform(ctx, doc, transform_att, transform_tag, ctm);
 
 		if (clip_att || clip_tag)
-			xps_clip(ctx, doc, &local_ctm, dict, clip_att, clip_tag);
+			xps_clip(ctx, doc, ctm, dict, clip_att, clip_tag);
 
-		xps_begin_opacity(ctx, doc, &local_ctm, area, opacity_mask_uri, dict, opacity_att, opacity_mask_tag);
+		xps_begin_opacity(ctx, doc, ctm, area, opacity_mask_uri, dict, opacity_att, opacity_mask_tag);
 
 		for (node = fz_xml_down(root); node; node = fz_xml_next(node))
-			xps_parse_element(ctx, doc, &local_ctm, area, base_uri, dict, node);
+			xps_parse_element(ctx, doc, ctm, area, base_uri, dict, node);
 
 		xps_end_opacity(ctx, doc, opacity_mask_uri, dict, opacity_att, opacity_mask_tag);
 
@@ -325,18 +326,16 @@ xps_parse_canvas(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, const
 		xps_drop_resource_dictionary(ctx, doc, new_dict);
 	fz_catch(ctx)
 		fz_rethrow(ctx);
-
 }
 
 void
-xps_parse_fixed_page(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, xps_page *page)
+xps_parse_fixed_page(fz_context *ctx, xps_document *doc, fz_matrix ctm, xps_page *page)
 {
 	fz_xml *root, *node;
 	xps_resource *dict;
 	char base_uri[1024];
 	fz_rect area;
 	char *s;
-	fz_matrix scm;
 
 	fz_strlcpy(base_uri, page->fix->name, sizeof base_uri);
 	s = strrchr(base_uri, '/');
@@ -352,8 +351,7 @@ xps_parse_fixed_page(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, x
 	if (!root)
 		return;
 
-	area = fz_unit_rect;
-	fz_transform_rect(&area, fz_scale(&scm, page->fix->width, page->fix->height));
+	area = fz_transform_rect(fz_unit_rect, fz_scale(page->fix->width, page->fix->height));
 
 	fz_try(ctx)
 	{
@@ -366,7 +364,7 @@ xps_parse_fixed_page(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, x
 				else
 					dict = xps_parse_resource_dictionary(ctx, doc, base_uri, fz_xml_down(node));
 			}
-			xps_parse_element(ctx, doc, ctm, &area, base_uri, dict, node);
+			xps_parse_element(ctx, doc, ctm, area, base_uri, dict, node);
 		}
 	}
 	fz_always(ctx)
@@ -376,17 +374,17 @@ xps_parse_fixed_page(fz_context *ctx, xps_document *doc, const fz_matrix *ctm, x
 }
 
 void
-xps_run_page(fz_context *ctx, fz_page *page_, fz_device *dev, const fz_matrix *ctm, fz_cookie *cookie)
+xps_run_page(fz_context *ctx, fz_page *page_, fz_device *dev, fz_matrix ctm, fz_cookie *cookie)
 {
 	xps_page *page = (xps_page*)page_;
 	xps_document *doc = page->doc;
-	fz_matrix page_ctm = *ctm;
+	fz_matrix page_ctm;
 
-	fz_pre_scale(&page_ctm, 72.0f / 96.0f, 72.0f / 96.0f);
+	page_ctm = fz_pre_scale(ctm, 72.0f / 96.0f, 72.0f / 96.0f);
 
 	doc->cookie = cookie;
 	doc->dev = dev;
-	xps_parse_fixed_page(ctx, doc, &page_ctm, page);
+	xps_parse_fixed_page(ctx, doc, page_ctm, page);
 	doc->cookie = NULL;
 	doc->dev = NULL;
 }
